Package: mutoss
Type: Package
Title: Unified Multiple Testing Procedures
Version: 0.1-14
Authors@R: c(
    person(given = c("MuToss", "Coding"),
           family = "Team",
           role = "aut",
           comment = "Berlin 2010"),
    person(given = "Gilles",
           family = "Blanchard",
           role = "aut"),
    person(given = "Thorsten",
           family = "Dickhaus",
           role = "aut"),
    person(given = "Niklas",
           family = "Hack",
           role = "aut"),
    person(given = "Frank",
           family = "Konietschke",
           role = "aut"),
    person(given = "Kornelius",
           family = "Rohmeyer",
           role = c("aut", "cre"),
           email = "rohmeyer@small-projects.de"),
    person(given = "Jonathan",
           family = "Rosenblatt",
           role = "aut"),
    person(given = "Marsel",
           family = "Scheer",
           role = "aut"),
    person(given = "Wiebke",
           family = "Werft",
           role = "aut")
    )
Description: Designed to ease the application and comparison of multiple
    hypothesis testing procedures for FWER, gFWER, FDR and FDX. Methods are 
    standardized and usable by the accompanying 'mutossGUI'.
Depends: R (>= 2.10.0), mvtnorm
Suggests: fdrtool, qvalue, testthat, lattice
Imports: plotrix, multtest (>= 2.2.0), multcomp (>= 1.1-0), methods
License: GPL
LazyLoad: yes
LazyData: true
URL: https://github.com/kornl/mutoss/
BugReports: https://github.com/kornl/mutoss/issues/
NeedsCompilation: no
Packaged: 2025-12-26 10:08:10 UTC; kornel
Author: MuToss Coding Team [aut] (Berlin 2010),
  Gilles Blanchard [aut],
  Thorsten Dickhaus [aut],
  Niklas Hack [aut],
  Frank Konietschke [aut],
  Kornelius Rohmeyer [aut, cre],
  Jonathan Rosenblatt [aut],
  Marsel Scheer [aut],
  Wiebke Werft [aut]
Maintainer: Kornelius Rohmeyer <rohmeyer@small-projects.de>
Repository: CRAN
Date/Publication: 2026-01-08 06:10:37 UTC
Built: R 4.6.0; ; 2026-04-28 00:26:17 UTC; unix

import(stats)
import(graphics)
importFrom(utils, install.packages)
import(methods)
importFrom(multcomp, contrMat, glht, mcp)
importFrom(multtest, MTP, mt.rawp2adjp, fwer2fdr, fwer2tppfp, fwer2gfwer)
importFrom(plotrix, plotCI)
importFrom(mvtnorm, pmvt, qmvnorm, qmvt, pmvnorm)
export(augmentation)
export(compareMutoss)
export(mu.compare.adjusted)
export(mu.compare.critical)
export(mu.compare.summary)
export(fisher23.marginal)
export(fisher23_fast)
export(fisher22.marginal)
export(fisher22_fast)
export(gao)
export(requireLibrary)
export(hommel)
export(oracleBH)
export(Qvalue)
export(BlaRoq)
export(pval2qval)
export(pval2locfdr)
export(linearStepUp)
export(two.stage)
export(multiple.down.adjust)
export(multiple.down)
export(multcomp.wrapper)
export(onesamp.marginal)
export(paired.marginal)
export(twosamp.marginal)
export(ftest.marginal)
export(mutoss.apply)
exportClasses(ErrorControl)
exportClasses(Mutoss)
export(mutoss.plotCI)
export(gatherParameters)
export(gatherStatistics)
export(simulation)
export(nparcomp)
export(nparcomp.wrapper)
export(storey_pi0_est)
export(ABH_pi0_est)
export(TSBKY_pi0_est)
export(BR_pi0_est)
export(ranktruncated)
export(regwq)
export(snk)
export(printRejected)
export(rom)
export(holm)
export(calculateBetaAdjustment)
export(aorc)
export(BL)
export(BH)
export(BY)
export(hochberg)
export(adaptiveBH)
export(adaptiveSTS)
export(SidakSD)
export(indepBR)
export(twostageBR)
export(SUD)
export(SD)
export(SU)
export(pValuesPlot)
export(adjPValuesPlot)
export(tukey.wrapper)
export(bonferroni)
export(sidak)
export(twosamp.model)
export(paired.model)
export(onesamp.model)
export(ftest.model)
export(fisher23.model)
export(fisher22.model)
export(jointCDF.orderedUnif)

# A File for temporarily functions to test the GUI.
# 
# Author: MarselScheer
###############################################################################


pValuesPlot = function(pValues) {
	plot(ecdf(pValues), do.points=FALSE, verticals = TRUE, main="ecdf", ylim=c(0,1))
	abline(0,1, col=2)
}

adjPValuesPlot = function(adjPValues, alpha) {
	plot(sort(adjPValues), main="Adjusted p-values", ylab="adjusted p-values", xlab="ordered index", ylim=c(0,1))
	if (!missing(alpha)) { 
		abline(alpha,0, col=2) 
	}
}

augmentation <- function(adjPValues, newErrorControl, newK, newQ, silent=FALSE) {
	if (newErrorControl == "gFWER") {	
		out = fwer2gfwer(adjPValues, k = newK)
		if(!silent) cat('\n\n\t\tGeneralized Family-Wise Error Rate\n\n')
		return(list(adjPValues=as.numeric(out), rejected=NULL, errorControl = new(Class='ErrorControl',type="gFWER")))
	} else if (newErrorControl == "FDX") {	
		out = fwer2tppfp(adjPValues, q = newQ)
		if(!silent) cat('\n\n\t\tTail Probability of the Proportion of False Positives\n\n')
		return(list(adjPValues=as.numeric(out), rejected=NULL, errorControl = new(Class='ErrorControl',type="FDX")))
	} else if (newErrorControl == "FDR") {	
		out = fwer2fdr(adjPValues, method = "restricted")
		if(!silent) cat('\n\n\t\tFalse Discovery Rate\n\n')
		return(list(adjPValues=as.numeric(out$adjp), rejected=NULL, errorControl = new(Class='ErrorControl',type="FDR")))		
	} else{ 
		if(!silent)cat('\n\n\t\tUnknown newErrorControl method')
	}
}

mutoss.augmentation <- function() { return(new(Class="MutossMethod",
					label="Augmentation MTP adjusted p-values",
					errorControl=c("FWER"),
					callFunction="augmentation",
					output=c("adjPValues", "rejected", "errorControl"),
					info="<h2>Augmentation MTP adjusted p-values</h2>
<p>Wrapper function to the augmentation methods of the multtest package.</p>

<p>The augmentation method turns a vector of p-values which are already adjusted for FWER control
into p-values that are adjusted for gFWER, FDX or FDR. The underlying idea (for gFWER and FDX) 
is that the set of hypotheses rejected at a given level alpha under FWER can be 'augmented'
by rejecting some additional hypotheses while still ensuring (strong) control of the desired weaker type I criterion.
For FDR, it uses the fact that FDX control for q=alpha=1-sqrt(1-beta) entails FDR control at level beta.</p>

<p>Use of these augmentation methods is recommended only in the situation where FWER-controlled p-values are
directly available from the data (using some specific method). When only marginal p-values are available,
it is generally prerefable to use other adjustment methods directly aimed at the intended criterion
(as opposed to first adjust for FWER, then augment)</p>

<p>Note: In the multtest package, two methods ('restricted' and 'conservative') are available for FDR augmentation. 
Here the 'restricted' method is forced for FDR augmentation since it is in fact always valid and better than 
'conservative' (M. van der Laan, personal communication) with respect to power.</p>
<h3>Reference:</h3>
<ul>
<li> S. Dudoit, M.J. van der Laan. \"<i> Multiple Testing Procedures with Applications to Genomics</i>\", Springer, 2008. (chapter 6) </li>\n\
<li></li>
</ul>",
# TODO: <- Add the possibility of filling the rejected slot? (need additional input alpha)
					parameters=list(adjPValues=list(type="numeric"),
								newErrorControl=list(type="character", label="New error control", choices=c("FDR","FDX","gFWER")),
								newK=list(type="numeric", optional=TRUE, label="For gFWER set k"),
                                newQ=list(type="numeric", optional=TRUE, label="For FDX set q")
						
                                    	      ))) }

.onLoad <- function(libname, pkgname) {
#	if(!require("multtest", character.only=TRUE)) {
#		if (interactive()) {
#			answer <- readline("Multtest is missing - do you want to install it (y/N)? ")
#			if (substr(answer, 1, 1) %in% c("y","Y")) {
#				source("http://bioconductor.org/biocLite.R")
#				biocLite("multtest")
#				require("multtest")
#			} else {
#				warning("Package multtest is not avaible - please install it!")
#			}
#		} else {
#			warning("Package multtest is not avaible - please install it!")
#		}
#	}
}  

# 
# Author: JonathanRosenblatt
###############################################################################



pval2qval<- function(pValues, cutoff){
	requireLibrary('fdrtool')
	fdrtool <- get("fdrtool", envir=asNamespace("fdrtool"))
	qvals<-fdrtool(
			pValues,
			statistic= 'pvalue', 
			plot=FALSE,verbose=FALSE)$qval
	
	if (missing(cutoff)) {
		return(list(qValues=qvals))
	}
	return(list(qValues=qvals, rejected= qvals<=cutoff ))		
	
}



pval2locfdr<- function(pValues, cutoff){
	requireLibrary('fdrtool')
	fdrtool <- get("fdrtool", envir=asNamespace("fdrtool"))
	locfdr<-fdrtool(
			pValues,
			statistic= 'pvalue', 
			plot=FALSE,verbose=FALSE)$lfdr
	
	if (missing(cutoff)) {
		return(list(locFDR=locfdr))
	}
	return(list(locFDR=locfdr, rejected= locfdr<=cutoff ))		
}

mutoss.locfdr <- function() { 
	return(new(Class="MutossMethod",
					label="Local FDR (fdr)",
					callFunction="pval2locfdr",
					output=c("locFDR", "rejected"),  
					info=
							"<h2> Name: </h2> Local fdr.\n
							<h3> Also known as: </h3> fdr, empirical posterior probability of the null. \n
							<h3> Error Type: </h3> Motivated by Bayesian considerations. Does not guarantee control of frequentist error types like FWER or FDR.\n
							<h3> Recommended Usage: </h3> Typically used when a massive amount of hypotheses is being tested as in microarray analyses.\n
							<h3> Related procedures: </h3> See FDR methods for similar procedures for frequentist error control.\n
							<h3> References: </h3> \n
							<ul>
							<li> Efron B., Tibshirani R., Storey J. D. and Tusher, V. (2001).<i> Empirical Bayes Analysis of a Microarray Experiment. </i>\n
								 Journal of the American Statistical Association 96(456):1151-1160. </li>
							</ul>",				
					parameters=list(
							pValues=list(type="numeric"),
							cutoff=list(type="numeric", label="Local fdr cutoff for rejection", optional=TRUE))))
}



mutoss.qvalues <- function() { 
	return(new(Class="MutossMethod",
					label="q Values (Fdr)",
					callFunction="pval2qval",
					output=c("qValues", "rejected"),
					info=
							"<h2> Name: </h2> q-Values.\n
							<h3> Also known as: </h3> \n
							<ul> 
								<li> Estimated pFDR</li>\n 
								<li> Estimated Positive FDR </li>\n
								<li> Empirical tail-area posterior probability of the null</li> \n
							<h3> Error Type: </h3> Motivated by Bayesian considerations. Guarantees FDR control only when masses of hypotheses are being tested.\n
							<h3> Recommended Usage: </h3> Typically used when a massive amount of hypotheses is being tested as in microarray analyses.\n
							<h3> Related procedures: </h3> See FDR methods for similar procedures with frequentist error control.\n
							<h3> References: </h3> \n
							<ul>
							<li> Storey, J. D. (2003)<i>The Positive False Discovery Rate: A Bayesian Interpretation and the q-Value.</i>
							 The Annals of Statistics 31(6): 2013-2035. </li>
							</ul>",
					parameters=list(
							pValues=list(type="numeric"),
							cutoff=list(type="numeric", label="q-value (pFDR) cutoff for rejection", optional=TRUE))))
}

# Tukey HSD Test in parametric factorial designs
# 
# Author: FrankKonietschke
###############################################################################












tukey.wrapper <- function(model, alpha,factorC) {
	
	model <- aov(model)
	tukeyObj <- TukeyHSD(model,conf.level=1-alpha, factorC)
	

	estimates <- c(tukeyObj)[[1]][,1]
	confintL <- c(tukeyObj)[[1]][,2]
	confintU <- c(tukeyObj)[[1]][,3]
	confi <- cbind(estimates, confintL, confintU)
	pvalues <- c(tukeyObj)[[1]][,4]
	
	
	
	rejected1 <- (pvalues < alpha)
	#confi <- cbind(confint)
	print(cbind(confi,pvalues))
	return(list(adjPValues=pvalues,rejected=rejected1,confIntervals= confi,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
}


#tukey.wrapper(aov(breaks~wool*tension,data=warpbreaks),alpha=0.05,"tension")

mutoss.tukey<- function() { return(new(Class="MutossMethod",
					label="Tukey HSD Test",
					errorControl="FWER",
					callFunction="tukey.wrapper",
					output=c("adjPValues", "rejected","confIntervals","errorControl"),
					info="<h2>Tukey HSD test and simultaneous confidence intervals in parametric factorial designs</h2>
							<p>'With this function, it is possible to compute all pairs comparisons for expectations and 
                              simultaneous confidence intervals in factorial linear models. Hereby, the all-pairs comparisons
                              can be performed for user given effects. The overall variance is estimated by the linear model
                              as well as the degree of freedom used by the studentized range distribution. <p> 
							<p></p>
							<h3>Reference:</h3>
							<ul>
							<li>Tukey, J.W. \"<i>The problem of multiple comparisons. Unpublished manuscript
reprinted in: The Collected Works of John W. Tukey (1953).</i>\" Volume 8 (1994),Braun, H.I. (ed.), Chapman and Hall, New York.</li>
							</ul>",
					parameters=list(model=list(type="ANY"),
							
							alpha=list(type="numeric"),
							factorC=list(type="character", label="Factor for Comparison", fromR="FactorVar")
					
					)
			)) }

# Simultaneous confidence intervals for relative contrast effects
# 
# Author: FrankKonietschke
###############################################################################


nparcomp <- function (formula, data, type = c("UserDefined", "Tukey", "Dunnett", "Sequen",
				"Williams", "Changepoint", "AVE", "McDermott", "Marcus","UmbrellaWilliams"),
		control = NULL, conflevel = 0.95, alternative = c("two.sided",
				"less", "greater"), rounds = 3, correlation = FALSE,
		asy.method = c("logit", "probit", "normal", "mult.t"), plot.simci = FALSE,
		info = TRUE, contrastMatrix = NULL)
{
	corr.mat <- function(m, nc) {
		rho <- matrix(c(0), ncol = nc, nrow = nc)
		for (i in 1:nc) {
			for (j in 1:nc) {
				rho[i, j] <- m[i, j]/sqrt(m[i, i] * m[j, j])
			}
		}
		return(rho)
	}
	ssq <- function(x) {
		sum(x * x)
	}
	logit <- function(p) {
		log(p/(1 - p))
	}
	probit <- function(p) {
		qnorm(p)
	}
	expit <- function(G) {
		exp(G)/(1 + exp(G))
	}
	index <- function(char, test) {
		nc <- length(char)
		for (i in 1:nc) {
			if (char[i] == test) {
				return(i)
			}
		}
	}
	z.quantile <- function(conflevel = conflevel, corr, a, df = df.sw,
			dbs) {
		if (dbs == "n") {
			if (a == "two.sided") {
				z <- qmvnorm(conflevel, corr = corr, tail = "both")$quantile
			}
			if (a == "less" || a == "greater") {
				z <- qmvnorm(conflevel, corr = corr, tail = "lower")$quantile
			}
		}
		if (dbs == "t") {
			if (a == "two.sided") {
				z <- qmvt(conflevel, df = df.sw, interval = c(-10,
								10), corr = corr, tail = "both")$quantile
			}
			if (a == "less" || a == "greater") {
				z <- qmvt(conflevel, df = df.sw, interval = c(-10,
								10), corr = corr, tail = "lower")$quantile
			}
		}
		return(z)
	}
	if (conflevel >= 1 || conflevel <= 0) {
		stop("The confidence level must be between 0 and 1!")
		if (is.null(alternative)) {
			stop("Please declare the alternative! (two.sided, less, greater)")
		}
	}
	type <- match.arg(type)
	alternative <- match.arg(alternative)
	asy.method <- match.arg(asy.method)
	if (length(formula) != 3) {
		stop("You can only analyse one-way layouts!")
	}
	dat <- model.frame(formula, data)
	if (ncol(dat) != 2) {
		stop("Specify one response and only one class variable in the formula")
	}
	if (is.numeric(dat[, 1]) == FALSE) {
		stop("Response variable must be numeric")
	}
	response <- dat[, 1]
	factorx <- as.factor(dat[, 2])
	fl <- levels(factorx)
	a <- nlevels(factorx)
	if (a <= 2) {
		stop("You want to perform a two-sample test. Please use the function npar.t.test")
	}
	samples <- split(response, factorx)
	n <- sapply(samples, length)
	if (any(n <= 1)) {
		warn <- paste("The factor level", fl[n <= 1], "has got only one observation!")
		stop(warn)
	}
	ntotal <- sum(n)
	a <- length(n)
	tmp <- expand.grid(1:a, 1:a)
	ind <- tmp[[1]] > tmp[[2]]
	vi <- tmp[[2]][ind]
	vj <- tmp[[1]][ind]
	nc <- length(vi)
	gn <- n[vi] + n[vj]
	intRanks <- lapply(samples, rank)
	pairRanks <- lapply(1:nc, function(arg) {
				rank(c(samples[[vi[arg]]], samples[[vj[arg]]]))
			})
	pd <- sapply(1:nc, function(arg) {
				i <- vi[arg]
				j <- vj[arg]
				(sum(pairRanks[[arg]][(n[i] + 1):gn[arg]])/n[j] - (n[j] +
								1)/2)/n[i]
			})
	dij <- dji <- list(0)
	sqij <- sapply(1:nc, function(arg) {
				i <- vi[arg]
				j <- vj[arg]
				pr <- pairRanks[[arg]][(n[i] + 1):gn[arg]]
				dij[[arg]] <<- pr - sum(pr)/n[j] - intRanks[[j]] + (n[j] +
							1)/2
				ssq(dij[[arg]])/(n[i] * n[i] * (n[j] - 1))
			})
	sqji <- sapply(1:nc, function(arg) {
				i <- vi[arg]
				j <- vj[arg]
				pr <- pairRanks[[arg]][1:n[i]]
				dji[[arg]] <<- pr - sum(pr)/n[i] - intRanks[[i]] + (n[i] +
							1)/2
				ssq(dji[[arg]])/(n[j] * n[j] * (n[i] - 1))
			})
	vd.bf <- ntotal * (sqij/n[vj] + sqji/n[vi])
	singular.bf <- (vd.bf == 0)
	vd.bf[singular.bf] <- 1e-05
	df.sw <- (n[vi] * sqij + n[vj] * sqji)^2/((n[vi] * sqij)^2/(n[vj] -
					1) + (n[vj] * sqji)^2/(n[vi] - 1))
	lambda <- sqrt(n[vi]/(gn + 1))
	cov.bf1 <- diag(nc)
	rho.bf <- diag(nc)
	for (x in 1:(nc - 1)) {
		for (y in (x + 1):nc) {
			i <- vi[x]
			j <- vj[x]
			v <- vi[y]
			w <- vj[y]
			p <- c(i == v, j == w, i == w, j == v)
			if (sum(p) == 1) {
				cl <- list(function() (t(dji[[x]]) %*% dji[[y]])/(n[j] *
										n[w] * n[i] * (n[i] - 1)), function() (t(dij[[x]]) %*%
										dij[[y]])/(n[i] * n[v] * n[j] * (n[j] - 1)),
						function() -(t(dji[[x]]) %*% dij[[y]])/(n[v] *
										n[j] * n[i] * (n[i] - 1)), function() -(t(dij[[x]]) %*%
										dji[[y]])/(n[i] * n[w] * n[j] * (n[j] - 1)))
				case <- (1:4)[p]
				rho.bf[x, y] <- rho.bf[y, x] <- sqrt(ntotal *
								ntotal)/sqrt(vd.bf[x] * vd.bf[y]) * cl[[case]]()
				cov.bf1[x, y] <- cov.bf1[y, x] <- sqrt(vd.bf[x] *
								vd.bf[y])
			}
		}
	}
	V <- (cov.bf1 + diag(vd.bf - 1)) * rho.bf
	cov.bf1 <- cbind(V,-1*V)
	cov.bf2<-cbind(-1*V,V)
	cov.bf <- rbind(cov.bf1,cov.bf2)
	switch(type,
			UserDefined = {
				if (is.null(contrastMatrix)){stop("Give a contrast matrix with the contrast.matrix = <> option or choose a contrast!")}
				
				if (is.null(control)) {
					nc <- nrow(contrastMatrix)
					weights.help <- weightMatrix(n, contrast.matrix = contrastMatrix)
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- paste("C", 1:nc)
					type.of.contrast <- "User Defined"
				}
				else {
					stop("Please declare the control group via your contrast matrix!")
				}
				type.of.contrast <- "User Defined"
			},
			
			Tukey = {
				if (is.null(control)) {
					if (alternative!="two.sided"){stop("The Tukey contrast can only be tested two-sided!")}
					nc <- a * (a - 1)/2
					cmpid <- sapply(1:nc, function(arg) {
								i <- vi[arg]
								j <- vj[arg]
								paste("p", "(", fl[i], ",", fl[j], ")", sep = "")
							})
					
					weights.help <- weightMatrix(n, "Tukey")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
				}
				else {
					stop("The Tukey contrast hasn't got a control group!")
				}
				type.of.contrast <- "Tukey"
			}, Dunnett = {
				nc <- a - 1
				if (is.null(control)) {
					cont <- 1
				}
				else {
					if (!any(fl == control)) {
						stop("The dataset doesn't contain this control group!")
					}
					cont <- which(fl == control)
				}
				vj <- which((1:a) != cont)
				vi <- rep(cont, a - 1)
				weights.help <- weightMatrix(n, "Dunnett",cont)
				weight<-weights.help$weight.matrix
				weight.help <- weights.help$weight.help
				cmpid <- sapply(1:nc, function(arg) {
							i <- vi[arg]
							j <- vj[arg]
							paste("p", "(", fl[i], ",", fl[j], ")", sep = "")
						})
				
				type.of.contrast <- "Dunnett"
			}, Sequen = {
				if (is.null(control)) {
					nc <- a - 1
					vi <- 1:(a - 1)
					vj <- 2:a
					weights.help <- weightMatrix(n, "Sequen")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- sapply(1:nc, function(arg) {
								i <- vj[arg]
								j <- vi[arg]
								paste("p", "(", fl[j], ",", fl[i], ")", sep = "")
							})
					
					type.of.contrast <- "Sequen"
				}
				else {
					stop("The Sequen-Contrast hasn't got a control group!")
				}
			},
			
			
			Williams = {
				if (is.null(control)) {
					
					nc <- a - 1
					weights.help <- weightMatrix(n, "Williams")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- paste("C", 1:(a - 1))
					type.of.contrast <- "Williams"
				}
				else {
					stop("The Williams contrast hasn't got a control group!")
				}
			}, Changepoint = {
				if (is.null(control)) {
					nc <- a - 1
					weights.help <- weightMatrix(n, "Changepoint")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- paste("C", 1:(a - 1))
					type.of.contrast <- "Changepoint"
				}
				else {
					stop("The Changepoint-Contrast hasn't got a control group!")
				}
			}, AVE = {
				if (is.null(control)) {
					nc <- a
					weights.help <- weightMatrix(n, "AVE")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- paste("C", 1:a)
					type.of.contrast <- "Average"
				}
				else {
					stop("The Average-Contrast hasn't got a control group!")
				}
			}, McDermott = {
				if (is.null(control)) {
					nc <- a - 1
					weights.help <- weightMatrix(n, "McDermott")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- paste("C", 1:(a - 1))
					
					type.of.contrast <- "McDermott"
				}
				else {
					stop("The McDermott-Contrast hasn't got a control group!")
				}
			}, UmbrellaWilliams = {
				if (is.null(control)) {
					nc <- a * (a - 1)/2
					weights.help <- weightMatrix(n, "UmbrellaWilliams")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- paste("C", 1:(a * (a - 1)/2))
					
					type.of.contrast <- "Umbrella Williams"
				}
				else {
					stop("The Umbrella Williams-Contrast hasn't got a control group!")
				}
			},
			
			Marcus = {
				if (is.null(control)) {
					nc <- a * (a - 1)/2
					weights.help <- weightMatrix(n, "Marcus")
					weight<-weights.help$weight.matrix
					weight.help <- weights.help$weight.help
					cmpid <- paste("C", 1:(a * (a - 1)/2))
					
					type.of.contrast <- "Marcus"
				}
				else {
					stop("The Marcus-Contrast hasn't got a control group!")
				}
			}
	
	
	
	)
	
	pd1 <- (pd == 1)
	pd0 <- (pd == 0)
	pd[pd1] <- 0.999
	pd[pd0] <- 0.001
	pd.help1 <- c(pd, 1-pd)
	pd <- c(weight%*%pd.help1)
	cov.bf <- weight %*% cov.bf %*% t(weight)
	for (i in 1:nc) {
		if (cov.bf[i, i] == 0) {
			cov.bf[i, i] <- 0.001
		}
	}
	
	vd.bf <- c(diag(cov.bf))
	vd.bf <- c(vd.bf)
	rho.bf <- corr.mat(cov.bf, nc)
	t.bf <- sqrt(ntotal) * (pd - 1/2)/sqrt(vd.bf)
	rownames(weight) <- paste("C", 1:nc)
	ncomp <- a * (a - 1)/2
	tmp <- expand.grid(1:a, 1:a)
	ind <- tmp[[1]] > tmp[[2]]
	v2 <- tmp[[2]][ind]
	v1 <- tmp[[1]][ind]
	namen1 <- sapply(1:ncomp, function(arg) {
				i <- v2[arg]
				j <- v1[arg]
				paste("p", "(", fl[i], ",", fl[j], ")", sep = "")
			})
	namen2 <- sapply(1:ncomp, function(arg) {
				i <- v2[arg]
				j <- v1[arg]
				paste("p", "(", fl[j], ",", fl[i], ")", sep = "")
			})
	colnames(weight) <- c(namen1,namen2)
	df.sw[is.nan(df.sw)] <- 1000
	df.sw <- weight %*% c(df.sw,df.sw)
	df.sw <- max(4, min(df.sw))
	pd <- c(pd)
	logit.pd <- logit(pd)
	logit.dev <- diag(1/(pd * (1 - pd)))
	logit.cov <- logit.dev %*% cov.bf %*% t(logit.dev)
	vd.logit <- c(diag(logit.cov))
	t.logit <- (logit.pd) * sqrt(ntotal/vd.logit)
	probit.pd <- qnorm(pd)
	probit.dev <- diag(sqrt(2 * pi)/(exp(-0.5 * qnorm(pd) * qnorm(pd))))
	probit.cov <- probit.dev %*% cov.bf %*% t(probit.dev)
	vd.probit <- c(diag(probit.cov))
	t.probit <- (probit.pd) * sqrt(ntotal/vd.probit)
	p.bfn = p.bft = p.bflogit = p.bfprobit = c()
	p.n = p.t = p.logit = p.probit = c()
	
	if (alternative == "two.sided") {
		z.bft <- z.quantile(conflevel = conflevel, corr = rho.bf,
				"two.sided", df = df.sw, dbs = "t")
		z.bfn <- z.quantile(conflevel = conflevel, corr = rho.bf,
				"two.sided", df = 0, dbs = "n")
		lower.bft <- pd - sqrt(vd.bf/ntotal) * z.bft
		upper.bft <- pd + sqrt(vd.bf/ntotal) * z.bft
		lower.bfn <- pd - sqrt(vd.bf/ntotal) * z.bfn
		upper.bfn <- pd + sqrt(vd.bf/ntotal) * z.bfn
		lower.logit <- expit(logit.pd - sqrt(vd.logit/ntotal) *
						z.bfn)
		upper.logit <- expit(logit.pd + sqrt(vd.logit/ntotal) *
						z.bfn)
		lower.probit <- pnorm(probit.pd - sqrt(vd.probit/ntotal) *
						z.bfn)
		upper.probit <- pnorm(probit.pd + sqrt(vd.probit/ntotal) *
						z.bfn)
		#---------------------------------------------------------#
		#--------------Berechnung von p-Werten -------------------#
		for (i in 1:nc) {
			p.bft[i] <- 1-pmvt(lower=-abs(t.bf[i]), abs(t.bf[i]), df=df.sw, corr=rho.bf,delta=rep(0,nc))
			p.bfn[i] <- 1-pmvnorm(lower=-abs(t.bf[i]), abs(t.bf[i]),corr=rho.bf,mean=rep(0,nc))
			p.bflogit[i] <- 1-pmvnorm(lower=-abs(t.logit[i]), abs(t.logit[i]),corr=rho.bf ,mean=rep(0,nc))
			p.bfprobit[i] <- 1-pmvnorm(lower=-abs(t.probit[i]), abs(t.probit[i]),corr=rho.bf,mean=rep(0,nc))
			p.tt <- pt(t.bf[i], df.sw)
			p.t[i] <- min(2*p.tt,2-2*p.tt)
			p.nn <- pnorm(t.bf[i])
			p.n[i] <- min(2*p.nn, 2-2*p.nn)
			p.ll <- pnorm(t.logit[i])
			p.logit[i] <- min(2*p.ll, 2-2*p.ll)
			p.pp <- pnorm(t.probit[i])
			p.probit[i] <- min(2*p.pp, 2-2*p.pp)
		}
		
		text.output.p <- "H_0: p(i,j)=1/2"
		text.output.KI <- paste(100 * conflevel, "%", "2-sided",
				"Simultaneous-Confidence-Intervals for Relative Effects")
		upper <- "]"
		lower <- "["
	}
	if (alternative == "greater") {
		z.bft <- z.quantile(conflevel = conflevel, corr = rho.bf,
				"less", df = df.sw, dbs = "t")
		z.bfn <- qmvnorm(conflevel, corr = rho.bf, tail = "lower")$quantile
		lower.bft <- pd - sqrt(vd.bf/ntotal) * z.bft
		lower.bfn <- pd - sqrt(vd.bf/ntotal) * z.bfn
		lower.logit <- expit(logit.pd - sqrt(vd.logit/ntotal) *
						z.bfn)
		lower.probit <- pnorm(probit.pd - sqrt(vd.probit/ntotal) *
						z.bfn)
		upper.bft = upper.probit = upper.logit = upper.bfn = 1
		for (i in 1:nc) {
			p.bfn[i] <- 1 - pmvnorm(lower = -Inf, upper = t.bf[i],
					mean = rep(0, nc), corr = rho.bf)
			p.bft[i] <- 1 - pmvt(lower = -Inf, upper = t.bf[i],
					delta = rep(0, nc), df = df.sw, corr = rho.bf)
			p.bflogit[i] <- 1 - pmvnorm(lower = -Inf, upper = t.logit[i],
					mean = rep(0, nc), corr = rho.bf)
			p.bfprobit[i] <- 1 - pmvnorm(lower = -Inf, upper = t.probit[i],
					mean = rep(0, nc), corr = rho.bf)
			
			p.t[i] <-1-pt(t.bf[i], df.sw)
			p.n[i] <- 1-pnorm(t.bf[i])
			p.logit[i] <- 1- pnorm(t.logit[i])
			p.probit[i] <-1- pnorm(t.probit[i])
			
		}
		text.output.p <- "H_0: p(i,j)<=1/2"
		text.output.KI <- paste(100 * conflevel, "%", "1-sided",
				"Simultaneous-Confidence-Intervals for Relative Effects")
		upper <- "]"
		lower <- "("
	}
	if (alternative == "less") {
		z.bft <- z.quantile(conflevel = conflevel, corr = rho.bf,
				"less", df = df.sw, dbs = "t")
		z.bfn <- z.quantile(conflevel = conflevel, corr = rho.bf,
				"less", df = 0, dbs = "n")
		upper.bft <- pd + sqrt(vd.bf/ntotal) * z.bft
		upper.bfn <- pd + sqrt(vd.bf/ntotal) * z.bfn
		upper.logit <- expit(logit.pd + sqrt(vd.logit/ntotal) *
						z.bfn)
		upper.probit <- pnorm(probit.pd + sqrt(vd.probit/ntotal) *
						z.bfn)
		lower.bft = lower.probit = lower.logit = lower.bfn = 0
		for (i in 1:nc) {
			p.bfn[i] <- 1 - pmvnorm(lower = t.bf[i], upper = Inf,
					mean = rep(0, nc), corr = rho.bf)
			p.bft[i] <- 1 - pmvt(lower = t.bf[i], upper = Inf,
					delta = rep(0, nc), df = df.sw, corr = rho.bf)
			p.bflogit[i] <- 1 - pmvnorm(lower = t.logit[i], upper = Inf,
					mean = rep(0, nc), corr = rho.bf)
			p.bfprobit[i] <- 1 - pmvnorm(lower = t.probit[i],
					upper = Inf, mean = rep(0, nc), corr = rho.bf)
			p.t[i] <-  pt(t.bf[i], df.sw)
			p.n[i] <- pnorm(t.bf[i])
			p.logit[i] <- pnorm(t.logit[i])
			p.probit[i] <- pnorm(t.probit[i])
		}
		text.output.p <- " H_0: p(i,j)>=1/2"
		text.output.KI <- paste(100 * conflevel, "%", "1-sided",
				"Simultaneous-Confidence-Intervals for Relative Effects")
		upper <- ")"
		lower <- "["
	}
	bfn.lower <-  round(lower.bfn, rounds)
	bfn.upper <- round(upper.bfn, rounds)
	bft.lower <-  round(lower.bft, rounds)
	bft.upper <- round(upper.bft,rounds)
	logit.lower <- round(lower.logit, rounds)
	logit.upper <- round(upper.logit,rounds)
	probit.lower <- round(lower.probit, rounds)
	probit.upper <- round(upper.probit,rounds)
	p.bflogit <- round(p.bflogit, rounds)
	p.bfprobit <- round(p.bfprobit, rounds)
	p.bft <- round(p.bft, rounds)
	p.bfn <- round(p.bfn, rounds)
	p.logit <- round(p.logit, rounds)
	p.probit <- round(p.probit, rounds)
	p.t <- round(p.t, rounds)
	p.n <- round(p.n, rounds)
	pd <- round(pd, rounds)
	if (correlation == TRUE) {
		Correlation <- list(Correlation.matrix.N = rho.bf, Covariance.matrix.N = cov.bf,
				Warning = paste("Attention! The covariance matrix is multiplied with N",
						"=", ntotal))
	}
	else {
		Correlation <- NA
	}
	data.info <- data.frame(row.names = 1:a, Sample = fl, Size = n)
	switch(asy.method, logit = {
				x.werte = cbind(lower.logit, pd, upper.logit)
				result <- list(weight.matrix = weight, Data.Info = data.info,
						Analysis.of.relative.effects = data.frame(row.names = c(1:nc),
								comparison = cmpid, rel.effect = pd, lower= logit.lower, upper=logit.upper,
								t.value = t.logit, p.adj = p.bflogit, p.raw= p.logit), Mult.Distribution = data.frame(Quantile = z.bfn,
								p.Value.global = min(p.bflogit)), Correlation = Correlation)
				Asymptotic.Method <- "Multivariate Delta-Method (Logit)"
			}, probit = {
				x.werte = cbind(lower.probit, pd, upper.probit)
				result <- list(weight.matrix = weight, Data.Info = data.info,
						Analysis.of.relative.effects = data.frame(row.names = c(1:nc),
								comparison = cmpid, rel.effect = pd, lower = probit.lower, upper = probit.upper,
								t.value = t.probit, p.adj = p.bfprobit, p.raw = p.probit), Mult.Distribution = data.frame(Quantile = z.bfn,
								p.Value.global = min(p.bfprobit)), Correlation = Correlation)
				Asymptotic.Method <- "Multivariate Delta-Method (Probit)"
			}, normal = {
				x.werte = cbind(lower.bfn, pd, upper.bfn)
				result <- list(weight.matrix = weight, Data.Info = data.info,
						Analysis.of.relative.effects = data.frame(row.names = c(1:nc),
								comparison = cmpid, rel.effect = pd, lower = bfn.lower, upper=bfn.upper,
								t.value = t.bf, p.adj = p.bfn, p.raw = p.n), Mult.Distribution = data.frame(Quantile = z.bfn,
								p.Value.global = min(p.bfn)), Correlation = Correlation)
				Asymptotic.Method <- "Multivariate Normal Distribution"
			}, mult.t = {
				x.werte = cbind(lower.bft, pd, upper.bft)
				result <- list(weight.matrix = weight, Data.Info = data.info,
						Analysis.of.relative.effects = data.frame(row.names = c(1:nc),
								comparison = cmpid, rel.effect = pd, lower = bft.lower, upper=bft.upper,
								t.value = t.bf, p.adj = p.bft,p.raw = p.t), Mult.Distribution = data.frame(Quantile = z.bft,
								p.Value.global = min(p.bft), d.f. = df.sw), Correlation = Correlation)
				Asymptotic.Method <- paste("Multi t - Distribution with d.f.= ",
						round(df.sw, 4))
			})
	if (plot.simci == TRUE) {
		test <- matrix(c(1:nc), ncol = nc, nrow = nc)
		angaben <- c(cmpid)
		angaben <- matrix(c(angaben), ncol = nc, nrow = nc)
		k <- c(1:nc)
		plot(x.werte[, 2], k, xlim = c(0, 1), axes = FALSE, type = "p",
				pch = 15, xlab = "", ylab = "")
		abline(v = 0.5, col = "red", lty = 1, lwd = 2)
		axis(1, at = seq(0, 1, 0.1))
		axis(2, at = test, labels = angaben)
		axis(4, at = test, labels = test)
		points(x = x.werte[, 3], y = test[, 1], pch = upper)
		points(x = x.werte[, 1], y = test[, 1], pch = lower)
		for (i in 1:nc) {
			polygon(c(x.werte[i, 1], x.werte[i, 3]), c(i, i))
		}
		box()
		title(main = c(text.output.KI, paste("Type of Contrast:",
								"", type.of.contrast, sep = ""), paste("Method:",
								"", Asymptotic.Method, sep = "")), ylab = "Comparison",
				xlab = paste("lower", lower, "-----", "p", "------",
						upper, "upper"))
	}
	if (info == TRUE) {
		cat("\n", "", "Nonparametric Multiple Comparison Procedure based on relative contrast effects",
				",", "Type of Contrast", ":", type.of.contrast, "\n",
				"NOTE:", "\n", "*-------------------Weight Matrix------------------*",
				"\n", "-", "Weight matrix for choosen contrast based on all-pairs comparisons",
				"\n", "\n", "*-----------Analysis of relative effects-----------*",
				"\n", "-", "Simultaneous Confidence Intervals for relative effects p(i,j)\n      with confidence level",
				conflevel, "\n", "-", "Method", "=", Asymptotic.Method,
				"\n", "-", "p-Values for ", text.output.p, "\n",
				"\n", "*----------------Interpretation--------------------*",
				"\n", "p(a,b)", ">", "1/2", ":", "b tends to be larger than a",
				"\n", "*--------------Mult.Distribution-------------------*",
				"\n", "-", "Equicoordinate Quantile", "\n", "-", "Global p-Value",
				"\n", "*--------------------------------------------------*",
				"\n")
		
	}
	
	return(result)
	
}

weightMatrix <- function(n,type = c("UserDefined","Tukey","AVE","Dunnett", "Sequen",
				"Changepoint", "Marcus",
				"McDermott", "Williams", "UmbrellaWilliams"), base = 1, contrast.matrix=NULL) {
	a  <- length (n)
	
	n.col <- a*(a-1)/2
	tmp <- expand.grid(1:a, 1:a)
	ind <- tmp[[1]] > tmp[[2]]
	vi <- tmp[[2]][ind]
	vj <- tmp[[1]][ind]
	
	
	type <- match.arg(type)
	
	switch (type,
			UserDefined = {
				if (is.null(contrast.matrix)){stop("Choose a contrast or give a contrast matrix by using 'contrast.matrix = matrix'")}
				#if(any(abs(contrast.matrix)>1)){stop("The contrast weights must be between 0 and 1!")}
				if (ncol(contrast.matrix)!=a){stop("The contrast matrix has more or less columns than samples!")}
				nc<-nrow(contrast.matrix)
				
				for (i in 1:nc){
					places_pos<-(contrast.matrix[i,]>0)
					places_neg<-(contrast.matrix[i,]<0)
					sum_pos<-sum(contrast.matrix[i,][places_pos])
					sum_neg<-sum(contrast.matrix[i,][places_neg])
					if (abs(sum_neg)!= sum(sum_pos)) {
						stop(" Wrong contrast matrix!The sum of negative and positive weights must be equal") }
					
					for ( j in 1:a){
						
						if (contrast.matrix[i,j]<0){ contrast.matrix[i,j]<-contrast.matrix[i,j]/abs(sum_neg)}
						if (contrast.matrix[i,j]>0){ contrast.matrix[i,j]<-contrast.matrix[i,j]/sum_pos}
						
					}
				}
				n.row <- nrow(contrast.matrix)
				
				
				ch <- contrast.matrix
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
				
			},
			
			AVE = {
				
				
				n.row <- a
				
				ch <- contrMat(n, type="AVE")
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			},
			
			Changepoint = {
				
				n.row <- a - 1
				
				ch <- contrMat(n, type = "Changepoint")
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			},
			
			Dunnett = {
				
				n.row <- a - 1
				
				ch <- contrMat(n, type = "Dunnett", base = base)
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			},
			
			Sequen =  {
				
				n.row <- a - 1
				
				ch <- contrMat(n, type = "Sequen", base)
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			},
			
			Marcus = {
				
				n.row <- a*(a-1)/2
				
				ch <- contrMat(n, type = "Marcus", base)
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			},
			
			McDermott = {
				
				n.row <- a -1
				
				ch <- contrMat(n, type = "McDermott", base)
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			},
			
			Williams = {
				
				n.row <- a - 1
				
				ch <- contrMat(n, type = "Williams")
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			} ,
			
			Tukey = {
				
				
				n.row <- a*(a-1)/2
				
				ch <- contrMat(n, type = "Tukey", base)
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			},
			UmbrellaWilliams = {
				
				n.row <- a*(a-1)/2
				
				ch <- contrMat(n, type = "UmbrellaWilliams", base)
				w<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help2<-matrix(0,nrow = n.row , ncol=n.col )
				weight.help1<-matrix(0,nrow = n.row , ncol=n.col )
				for (i in 1:n.row){
					for (j in 1:n.col){
						
						help <- c(rep(i,n.col))
						a <- help[j]
						b <- vi[j]
						d <- vj[j]
						
						w[i,j] <- ch[a,b] *ch[a,d]
						
						if (w[i,j]>0){w[i,j]<-0}
						if (ch[a,b] > 0 && ch[a,d] < 0) {weight.help2[i,j]<-1}
						if(ch[a,b] < 0 && ch[a,d] > 0){weight.help1[i,j] <-1}
					}
				}
				w1<-w*weight.help1
				w2<-w*weight.help2
				w<--1*cbind(w1,w2)
				w
			}
	)
	ch.out <- matrix(c(ch),nrow=n.row)
	
	result <- list(weight.matrix = w, weight.help1 = weight.help1,weight.help2 = weight.help2,contrast.matrix = ch.out)
	result
}

nparcomp.wrapper <- function(model, data, hypotheses, alpha, alternative, asy.method) {
	
	control <- NULL
	type <- NULL
	contrastMatrix <- NULL	
	
	if (hypotheses %in% c("Tukey", "Dunnett", "Sequen",
					"Williams", "Changepoint", "AVE", "McDermott", "Marcus", "UmbrellaWilliams")) {
		type <- hypotheses
	} else {
		type <- "UserDefined" 
		contrastMatrix <- hypotheses
	}	
	
	result <- nparcomp(formula=formula(model), 
			data, 
			type = type,
			control = control, 
			conflevel = 1-alpha,
			alternative,
			rounds = 3, 
			correlation = TRUE,
			asy.method,
			plot.simci = FALSE,
			info = TRUE,
			contrastMatrix = contrastMatrix)
	pvalues <- result$Analysis.of.relative.effects$p.adj
	rejected1 <- (pvalues < alpha)
	confi <- cbind(result$Analysis.of.relative.effects$rel.effect, result$Analysis.of.relative.effects$lower, result$Analysis.of.relative.effects$upper)
	rownames(confi)<-result$Analysis.of.relative.effects$comparison
	temp <- cbind(confi,pvalues)
	colnames(temp) <- c("Estimate", "Lower","Upper","pValue")
	print(temp)
	return(list(adjPValues=pvalues,
					rejected=rejected1, confIntervals= confi,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
}

mutoss.nparcomp <- function() { return(new(Class="MutossMethod",
					label="Nonparametric relative contrast effects",
					errorControl="FWER",
					callFunction="nparcomp.wrapper",
					output=c("adjPValues", "rejected","confIntervals", "errorControl"),
					info="<h2>Nonparametric relative contrast effects</h2>
							<p>With this function, it is possible to compute nonparametric simultaneous confidence\
							intervals for relative contrast effects in the unbalanced one way layout. Moreover, it computes\
							adjusted p-values. The simultaneous confidence intervals can be computed using\
							multivariate normal distribution, multivariate t-distribution with a Satterthwaite Approximation\
							of the degree of freedom or using multivariate range preserving transformations with Logit or\
							Probit as transformation function. There is no assumption on the underlying distribution function, only\
							that the data have to be at least ordinal numbers.</p> 
							<p></p>
						  <h3>Reference:</h3>
							<ul>
							  <li>Konietschke, F. \"<i>Simultane Konfidenzintervalle fuer nichtparametrische relative Kontrasteffekte.</i>\" Dissertation, University of Goettingen, 2009.</li>
							  <li>Konietschke, F., Brunner, E., Hothorn, L.A.  \"<i>Simultaneous confidence intervals for nonparametric relative contrast effects.</i>\" Research report at the University of Hannover, 2009.</li>
							</ul>",
					parameters=list(
							data=list(type="data.frame"),
							model=list(type="ANY"),
							hypotheses=list(type="ANY"),
							alpha=list(type="numeric"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							asy.method=list(type="character", label="Asymptotic approx. method", choices=c("logit", "probit", "normal", "mult.t"))
					)
			)) }

# Implements the elementary functions for a general
# step-up-down test. 
# Step-Up and step-down are derived from that.
# 
# Author: MarselScheer
###############################################################################


SUD <- function(pValues, criticalValues, startIDX_SUD) 
{		
	len <- length(criticalValues)
	
	# +++++++++++++++++   Plausis    ++++++++++++++
	if (len == 1)
		stop("SUD(): There is only 1 critical Value. Use the function SS()!")
	
	if (len != length(pValues))
		stop("SUD(): Length of critical Values and pValues need to be of the same length!")
	
	if (startIDX_SUD < 1 || len < startIDX_SUD )
		stop("SUD(): startIDX out of bound. criticalValues[startIDX] will not exist.")		
	# -----------------   Plausis    ---------------
	
	rejected <- rep(FALSE, times = len)

	# need to work with orderd pValues
	sortedPV <- sort(pValues, index.return = TRUE)
	
	suspiciousPV <- (sortedPV$x <= criticalValues)
	
	if (suspiciousPV[startIDX_SUD])
	{# Suspicious pValue.		
		# Actually doing now a StepDown on startIDX:len.
		# Additionally reject anything before startIDX
			
		nonSuspAboveStartIDX <- which(!suspiciousPV[startIDX_SUD:len]) 
		# ! looking only at the subset startIDX:len probably gives a shift of the index !
		# ! gonna correct this soon !
		
		# perhaps any pValue from startIDX to the end is suspicious, thus reject all!
		if (length(nonSuspAboveStartIDX) == 0)
			return(rep(TRUE, times = len))		
		
		# Correcting the shift
		nonSuspAboveStartIDX <- nonSuspAboveStartIDX + startIDX_SUD - 1
		
		# There must be some pValue between startIDX and the end that is not suspicious
		# Searching the first one. Anything immediately BEFORE that pValue will be rejected.		
		minIDX <- min(nonSuspAboveStartIDX) - 1
		
		rejected[sortedPV$ix[1:minIDX]] <- TRUE
	}
	else
	{# not suspicious pValue
		# Actually doing now a StepUp on 1:startIDX
		# The rejected are only the one rejected by this StepUp

		suspiciousIDX <- which(suspiciousPV[1:startIDX_SUD])
		
		# perhaps no pValue is suspicious, thus we do not reject anything
		if (length(suspiciousIDX) == 0)
			return(rep(FALSE, times = len))
		
		# There must be some pValue between 1 and startIDX that is suspicious
		# Searching the last one. Anything before (including the maximum) will be rejected.
		maxIDX <- max(suspiciousIDX)
		
		rejected[sortedPV$ix[1:maxIDX]] <- TRUE 
	}
	
	return(rejected)
}

SD <- function(pValues, criticalValues) 
{
	SUD(criticalValues = criticalValues, 
			pValues = pValues,
			startIDX_SUD = 1)
}

SU <- function(pValues, criticalValues) 
{
	SUD(criticalValues = criticalValues, 
			pValues = pValues,
			startIDX_SUD = length(criticalValues))
}

setClass("MutossMethod",
		representation    = representation(
				label			= "character", # this label will be shown in the menus
				errorControl    = "character", # FWER, FWER.weak, FDR, FDX, gFWER, perComparison (?)
				callFunction    = "character", # the function to call
				output          = "character", # this is the character vector of the _possible_ output of the function
				info            = "character", # info text, should contain small description, author, reference etc.
				assumptions     = "character", # assumptions for this method
				parameters		= "list",      # optional description of parameters - see MuToss developer handbook
				misc			= "list"       # a list where you can put all your  miscellaneous stuff
		)
)

bonferroni <- function(pValues, alpha, silent=FALSE) {
	adjPValues=sapply(pValues*length(pValues),function(x){min(x,1)})
	if (missing(alpha)) {
		return(list(adjPValues=adjPValues))
	} else {
	rejected <- (adjPValues<=alpha)
	if (! silent)
	{
		cat("\n\n\t\tBonferroni correction\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, rejected=rejected,
					errorControl = new(Class='ErrorControl', type="FWER", alpha=alpha)))
}
}

mutoss.bonferroni <- function() { return(new(Class="MutossMethod",
		label="Bonferroni correction",
		errorControl="FWER",
		callFunction="bonferroni",
		output=c("adjPValues", "rejected", "errorControl"),
		info="<h2>Bonferroni correction</h2>\n\n\
<p>The classical Bonferroni correction outputs adjusted p-values, ensuring strong FWER control under arbitrary
dependence of the input p-values. It simply multiplies each input p-value by the total number of hypotheses
(and ceils at value 1).</p> 
<p>It is recommended to use Holm's step-down instead, which is valid under the exact same assumptions and more powerful.
</p>
<h3>Reference:</h3><ul><li>Bonferroni, C. E. \"<i>Il calcolo delle assicurazioni su gruppi di teste.</i>\" In Studi in Onore del Professore Salvatore Ortu Carboni. Rome: Italy, pp. 13-60, 1935.</li>\n
<li>Bonferroni, C. E. \"<i>Teoria statistica delle classi e calcolo delle probabilita.</i>\" Pubblicazioni del R Istituto Superiore di Scienze Economiche e Commerciali di Firenze 8, 3-62, 1936.</li></ul>",
		parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric", optional=TRUE))
)) }


sidak <- function(pValues, alpha, silent=FALSE) {
	adjPValues <- sapply(1-(1-pValues)^length(pValues), function(x){min(x,1)})
	if (missing(alpha)) {
		return(list(adjPValues=adjPValues))
	} else {
	rejected <- (adjPValues <= alpha)
	if (! silent)
		{
		cat("\n\n\t\tSidak correction\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
			}
}

mutoss.sidak <- function() { return(new(Class="MutossMethod",
					label="Sidak correction",
					errorControl="FWER",
					callFunction="sidak",
					output=c("adjPValues", "rejected", "errorControl"),
					assumptions=c("test independence"),
					info="<h2>Sidak correction</h2>\n\n\
<p>The classical Sidak correction returns adjusted p-values, ensuring strong FWER control under
the assumption of independence of the input p-values. It only uses the fact that the probability of no incorrect
rejection is the product over true nulls of those marginal probabilities (using the assumed independence of p-values).
The procedure is more generally valid for positive orthant dependent test statistics.</p> 
<p>It is recommended to use the step-down version of the Sidak correction instead, 
which is valid under the exact same assumptions and more powerful.</p>
<h3>Reference:</h3>
<ul>
<li>  Sidak, Z. (1967).<i> Rectangular confidence regions for the means of multivariate normal distributions.</i> 
Journal of the American Statistical Association, 62:626-633.</li>
</ul>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric", optional=TRUE))
			)) }

# REGWQ - Ryan / Einot and Gabriel / Welsch  test procedure
# 
# Author: FrankKonietschke
###############################################################################




regwq <- function(formula, data,alpha, MSE=NULL, df=NULL, silent = FALSE){
	
	dat <- model.frame(formula, data)
	if (ncol(dat) != 2) {
		stop("Specify one response and only one class variable in the formula")
	}
	if (is.numeric(dat[, 1]) == FALSE) {
		stop("Response variable must be numeric")
	}
	response <- dat[, 1]
	group <- as.factor(dat[, 2])
	fl <- levels(group)
	a <-nlevels(group)
	N <- length(response)
	samples <- split(response,group)
	n <- sapply(samples,"length")
	mm <- sapply(samples,"mean")
	vv <- sapply(samples,"var")
	if (is.null(MSE)){
		MSE <- sum((n-1)*vv)/(N-a)
	}
	if (is.null(df)){
		df <- N-a
	}
	
	nc <- a*(a-1)/2
	order.h1 <- data.frame(Sample=fl, Size=n, Means=mm,Variance=vv)
	
	ordered <- order.h1[order(order.h1$Means,decreasing=FALSE), ]
	rownames(ordered) <- 1:a
	
	
	
	#---------------- Compute helping indices ----------#
	
	i <- 1:(a-1)
	
	h1 <- list()
	for(s in 1:(a-1)){
		
		h1[[s]]<- i[1:s]
		
	}
	vi <- unlist(h1)
	
	j <- a:2
	h2 <-list()
	
	for (s in 1:(a-1)){
		h2[[s]] <- j[s:1]
	}
	
	vj <- unlist(h2)
	
	h3 <- list()
	h4 <- list()
	
	for (s in 1:(a-1)){
		h3[[s]] <- rep(j[s],s)
		h4[[s]] <- rep(i[s],s)
	}
	Nmean <- unlist(h3)
	Step  <- unlist(h4)
	
	
	#--------Compute the Mean Differences---------#
	
	mean.difference <- sapply(1:nc,function(arg){
				i <- vi[arg]
				j <- vj[arg]
				(ordered$Means[j]-ordered$Means[i])
				
			})
	mean.difference <- round(mean.difference, 4)
	
	# ------- Compute the test statistics --------#
	
	T <- sapply(1:nc,function(arg){
				i<-vi[arg]
				j<-vj[arg]
				(ordered$Means[j]-ordered$Means[i])/sqrt(MSE/2*(1/ordered$Size[i] + 1/ordered$Size[j]))
				
			})
	
	T <- round(T, 4)
	#-------Compute the adjusted p-Values-------#
    pvalues <- ptukey(T,Nmean,df,lower.tail=FALSE)
	
	#------Compute the adjusted alpha-levels----#

    alpha.level <- 1-(1-alpha)^(Nmean/a)
	level1 <- (Nmean==a)
	level2 <- (Nmean==a-1)
	level3 <- level1 + level2
	alpha.level[level3==1] <- alpha
	alpha.level <- round(alpha.level,4)
	


	# ----- Compute now the critical value -----#
	
	quantiles <- qtukey(1-alpha.level,Nmean,df)
	
	for (h in 1:(nc-1)){
		if (quantiles[h+1] >=quantiles[h]){
			quantiles[h+1] <- quantiles[h]
		}
		
	}
	

	#---- Calculate the rejected Hypotheses ------# 
	
	Rejected1 <- (pvalues<alpha.level)
	
	#------------ Names for the Output -----------# 
	
	names.ordered <- sapply(1:nc, function(arg){
				i <- vi[arg]
				j <- vj[arg]
				paste(ordered$Sample[j], "-", ordered$Sample[i], sep="")
			})
	
	# ------ Compute now the rejected statistics-----#
	
	for (s in 1:nc){
		if (Rejected1[s]==FALSE){
			Under1 <- (vj[s]>=vj)
			Under2 <- (vi[s]<=vi)
			Under3 <- Under1 * Under2
			Under4 <- which(Under3==1)
			
			Rejected1[Under4] <- FALSE
			
		}
	}
	
	
	#-----Prepare the pValues for the Output----#
	
	Out1 <- (pvalues < alpha.level)
	Out2 <- (Rejected1 == FALSE)
	Out3 <- Out1 * Out2
	Out4 <- (Out3 == 1)
	pvalues <- round(pvalues,4)
	quantiles <- round(quantiles,4)
	pvalues[Out4] <- paste(">",alpha.level[Out4])
	quantiles[Out4] <- paste(">", T[Out4])
	variances.output <- data.frame(Overall=MSE, df=df)
	Comparison <- data.frame(Comparison=names.ordered,Diff=mean.difference, Statistic=T,Quantiles=quantiles,  Adj.P=pvalues, Alpha.Level=alpha.level, Rejected=Rejected1, Layer = Step)
	
	
	if (!silent)
	{
		cat("#----REGWQ - Ryan / Einot and Gabriel / Welsch  test procedure  \n\n")
		printRejected(Comparison$Rejected, pvalues, Comparison$Adj.P)
	}
	
	
	#result <- list(Ordered.Means = ordered, Variances=variances.output,
	#		REGWQ = Comparison)
	
	#diffm<-matrix(c(Comparison["Diff"],rep(NA,length(Comparison["Diff"])*2)),nrow=length(Comparison["Diff"]))
	diffm<-cbind(Comparison$Diff,rep(NA,length(Comparison$Diff)),rep(NA,length(Comparison$Diff)))	
	diffm<-matrix(diffm,nrow=length(Comparison$Diff))
	
	rownames(diffm)<-Comparison$Comparison
	
	return(list(adjPValues=Comparison$Adj.P, rejected=Comparison$Rejected, statistic=Comparison$Statistic,
					confIntervals=diffm,errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))	
}


mutoss.regwq <- function() { return(new(Class="MutossMethod",
					label="Ryan / Einot and Gabriel / Welsch  test",
					errorControl="FWER",
					callFunction="regwq",
					output=c("adjPValues","rejected","statistic","confIntervals","errorControl"),
					info="<h2>Ryan / Einot and Gabriel / Welsch  test procedure.
							The procedure controls the FWER. </h2>\n\n\
							<p> It is based on a stepwise or \
							layer approach to significance testing. Sample means are \
							ordered from the smallest to the largest. The largest \
							difference, which involves means that are r = p steps apart, \
							is tested first at alpha level of significance; if significant, \
							means that are r < p  steps apart are tested at an adjusted alpha level \
							of significance and so on. \ 
                            The alpha levels are adjusted for the p-1 different\
                            layers by the formula alpha_p= alpha, if p=k or p=k-1,\
                            alpha_p = 1-(1-\alpha)^{p/k}  otherwise.
							</p>\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Hochberg, Y., Yamhane, A.C. (1987).  \"<i> Multiple Comparison Procedures \
							
							</i>\" Wiley, New York. </li>\n\
							</ul>",
					parameters=list(formula=list(type="formula"), data=list(type="ANY"), alpha=list(type="numeric"), 
							MSE=list(type="numeric"), df=list(type="numeric"))
			)
	) }




	
	

# 
# Author: JonathanRosenblatt
###############################################################################

#----------- Validating input of compare procedure------------#



mu.test.class<- function(classes){
	if (any(classes!='Mutoss')) stop('Input is not a "Motoss" class object')
	##TODO: will this cause prolems for inherited class objects?
}

mu.test.type<- function(types){
	if( any(types != types[1]) ){
		message(' Notice:You are comparing methods for different error types. \n These should not be compared!	\n Output will be generated nevertheless. \n')
	}
}


mu.test.rates<-function(rates){
	if( any(rates != rates[1]) ){
		message(' Notice:You are comparing methods with different error rates. \n These should not be compared!	\n Output will be generated nevertheless. \n')
	}
}

mu.test.same.data<- function(pvals){
	pvals.different<- any( apply(pvals,1, function(x) any(x!=x[1])))
	if(pvals.different) stop('Different data was used for suppied procedures.')		
}


mu.test.name<- function(hyp.names){
	names.different<- any( apply(hyp.names,1, function(x) any(x!=x[1])))
	if(names.different) message('Notice: Hypotheses have different names. Can they be compared?')		
}


#-------- Create comparison list for Mutoss objects--------------#


compareMutoss<-function(...){
	objects<-list(...)
	
	classes<- sapply(objects, function(x) class(x) 	)#getting object classes
	mu.test.class(classes) #testing for compatible object classes
	
	types<-	sapply(objects, function(x) x@errorControl@type)#getting error control types
	mu.test.type(types) #testing for compatible error control types
	
	rates<-sapply(objects, function(x) x@errorControl@alpha)#extracting error rates
	
	pi.nulls<- as.numeric(lapply(objects, function(x) x@pi0))#extracting pi0 estimates
	
	pvalues<- sapply(objects, function(x) x@pValues )# getting adjusted pvals
	mu.test.same.data(pvalues)
	m<- nrow(pvalues)
	
	raw.hyp.names<- lapply(objects, function(x) x@hypNames )# getting hypothesis names
	if(all(sapply(raw.hyp.names, function(x) identical(x, character(0))))){
		hyp.names<- paste('hyp', 1:m, sep='') 
	}	
	else if(all(sapply(raw.hyp.names, function(x) length(x)==m))) {
		mu.test.name(raw.hyp.names)
		hyp.names<- raw.hyp.names[,1]
	}
	
	##TODO: [JR] Deal with mising hyp names only in a subset of objects
	
	# Preparing Raw Pvalues
	raw.pvals<- pvalues[,1]
	pval.order<- order(raw.pvals)
	pval.ranks<- rank(raw.pvals)
	raw.pvals.frame<-data.frame(
			pValue=raw.pvals,
			order=pval.order,
			ranks=pval.ranks)
	row.names(raw.pvals.frame)<-hyp.names 
	
	#Preparing adjusted pvalues
	adj.pvals<- lapply(objects, function(x) x@adjPValues )
	method.names<- unlist(lapply(adj.pvals, function(x) attributes(x)[1]))
	adj.pvals.frame<- data.frame(adj.pvals)
	colnames(adj.pvals.frame)<- method.names
	row.names(adj.pvals.frame)<- hyp.names
	
	#Preparing critical values
	critical<- lapply(objects, function(x) x@criticalValues )
	method.names<- unlist(lapply(critical, function(x) attributes(x)[1]))
	critical.frame<- data.frame(critical)
	colnames(critical.frame)<- method.names
	row.names(critical.frame)<- hyp.names	
	
	#Preparing decisions
	rejections<- lapply(objects, function(x) x@rejected )
	method.names<- unlist(lapply(rejections, function(x) attributes(x)[1]))
	rejections.frame<- data.frame(rejections)
	colnames(rejections.frame)<- method.names
	row.names(rejections.frame)<- hyp.names
	
	##TODO: [JR] Add groud truth to comparison method
	
	comparing<- list(
			types=types,
			rates=rates,
			pi.nulls=pi.nulls,
			raw.pValues=raw.pvals.frame,
			adjusted.pvals=adj.pvals.frame,
			criticalValue=critical.frame,
			rejections=rejections.frame
	)
	return(comparing)		
	
}
#For testing purposes
#source('~/workspace/mutoss/src/BasicFunctions/DummyBigObjects.R')
#test<- list(mu.test.obj.1, mu.test.obj.2)

#-------------- Comparison of adjusted p values -----------------#

mu.compare.adjusted<- function(comparison.list, identify.check=F){
	adjPValues<- comparison.list[['adjusted.pvals']]
	hyp.num<- nrow(adjPValues)
	method.num<- ncol(adjPValues)
	method.names<- factor(colnames(adjPValues))
	method.index<- as.numeric(method.names)
	raw.pValues<-comparison.list[['raw.pValues']]
	
	pvalue.ranks<-raw.pValues$ranks
	
	method.type<-comparison.list[['types']]
	stacked.adjPValues<- unlist(adjPValues, use.names=F)
	x<- rep(pvalue.ranks, method.num)
	method.labels<- rep(method.names, each=hyp.num)
	hyp.labels<- rep(row.names(adjPValues), method.num)
	point.charachters<- rep(method.index, each=hyp.num) #for plotting purposes only
	
	point.size<- hyp.num^(-0.1)
	plot(stacked.adjPValues~x,
			pch=point.charachters,
			ylim=c(0,1),
			cex=point.size,
			xlab='')
	
	the.title<- paste('Adjusted p-values for ',unique(method.type),' controlling procedures')
	title(the.title)			
	
	par(xpd=T)
	
	legend(x=0, y=-0.15,
			horiz=T,
			legend=method.names,
			pch=method.index,
			cex=method.num ^ (-1/4)	)
	par(xpd=F) #reset par to default value	
	
	if(identify.check) {
		identify(stacked.adjPValues~x, labels=hyp.labels )
	}
	##TODO: [JR] Plotting method using colors?	
}

#For testing purposes
#source('~/workspace/mutoss/src/BasicFunctions/DummyBigObjects.R')

#----------- Comparison of critical vales---------- #

mu.compare.critical<- function(comparison.list, identify.check=F){
	method.type<-comparison.list[['types']] #extracting method type
	
	criticalValues<- comparison.list[['criticalValue']]#extracting critical values
	hyp.num<- nrow(criticalValues)
	method.num<- ncol(criticalValues)
	method.names<- factor(colnames(criticalValues))
	method.index<- as.numeric(method.names)
	
	raw.pValues<-comparison.list[['raw.pValues']] #extracting raw palues	
	pvalue.ranks<-raw.pValues$ranks	
	
	stacked.criticalValues<- unlist(criticalValues, use.names=F)
	x<- rep(pvalue.ranks, method.num)
	method.labels<- rep(method.names, each=hyp.num)
	hyp.labels<- rep(row.names(criticalValues), method.num)
	point.charachters<- rep(method.index, each=hyp.num) #for plotting purposes only
	
	point.size<- hyp.num^(-0.1)
	
	plot(stacked.criticalValues~x, 
			pch=point.charachters, 
			ylim=c(0,1),
			xlab='',
			cex=point.size)
	
	the.title<- paste('Critical Values for ',unique(method.type),' controlling procedures')
	title(the.title)			
	
	par(xpd=T)
	legend(
			x=0,
			y=-0.15,
			horiz=T,
			legend=method.names,
			pch=method.index,
			cex=method.num ^ (-1/4)	)
	par(xpd=F) #reset par to default value	
	
	if(identify.check) {
		identify(stacked.criticalValues~x, labels=hyp.labels )
	}
}

#For testing purposes:
#source('~/workspace/mutoss/src/BasicFunctions/DummyBigObjects.R')
#mu.compare.critical(1)
#mu.compare.critical(compare.3, T)

#----- Sumary of comparison-----------#
mu.compare.summary<- function(comparison.list){
	method.type<-comparison.list[['types']] #extracting method type
	error.rates<-comparison.list[['rates']] #extracting error rates
	pi.nulls<-comparison.list[['pi.nulls']] #extracting pi0
	rejections<-comparison.list[['rejections']]
	hyp.num<- nrow(rejections)
	method.num<- ncol(rejections)
	method.names<- factor(colnames(rejections))
	method.index<- as.numeric(method.names)
	
	count.rejections<-apply(rejections, 2, sum)
	
	seperate<- rep('|', method.num)	
	summary<-data.frame(
			method.type, seperate, 
			error.rates, seperate, 
			count.rejections, seperate,
			pi.nulls)
	colnames(summary) <-c(
			'Error Type'," ",
			'Error Rate'," ",
			'Rejections Count', " ",
			'pi_0') 
	
	cat('\n Comparing multipe hypothesis procedures.\n',
			hyp.num, 'hypotheses tested.\n\n')
	
	print(summary)	
}

#For testing purposes :
#source('~/workspace/mutoss/src/BasicFunctions/DummyBigObjects.R')
#mu.compare.summary(1)
#mu.compare.summary(compare.3)

#############
# Fisher 23 #
#############


fisher23.model <- function() {
	return(list(model=list(typ="Fisher 2-by-3")))
}

mutoss.fisher23.model <- function() { return(new(Class="MutossMethod",
					label="Fisher's exact test in (2x3) tables",
					callFunction="fisher23.model",
					output=c("model"),
					info="<h2></h2>
							<p>(Marginal) Fisher's exact test in (2x3) tables</p> 
							<p></p>
							<h3>Reference:</h3>
							<ul>
							<li>Fisher, R. A. (1922). \"<i>On the interpretation of Chi^2 from contingency tables, and the calculation of P.</i>\" Journal of the Royal Statistical Society, 85 (1):87-94.</li>
							</ul>",
					parameters=list(
					)
			)) }


fisher23.marginal <- function(data, model) {
	#m <- dim(data)[3]
	#result <- vector(mode="numeric",length=m)
	result <- apply(data, 3, function(x) {fisher23_fast(x,2.0e-16)$rand_p} )
	return(list(pValues=result))
}


fisher23_fast <- function(obs, epsilon){
# obs = observations = a 2x3 table
	
#build marginals = (n1., n2., n.1, n.2, n.3)
	marginals <- c(sum(obs[1, ]), sum(obs[2, ]), sum(obs[, 1]), sum(obs[, 2]), sum(obs[, 3]))
	
	n <- sum(marginals) / 2
	
	x <- array(data=rep.int(0.0, 2*3*max(marginals)*max(marginals)), c(2, 3, max(marginals)*max(marginals)))
	
#build log nominator statistic
	log_nom <- sum(log(gamma(marginals+1))) - log(gamma(n+1))
	
#build log denominator statistic
	log_denom <- sum(log(gamma(obs+1)))
	
	
#compute probability of observed table
	prob_table <- exp(log_nom - log_denom)
	
	
	nonrand_p <- 0.0
	rand_count <- 0
	
	dim1 <- min(marginals[1], marginals[3])
	
#traverse all possible tables with given marginals
	counter <- 0
	
	for (k in 0:dim1)
	{
		for (l in max(0,marginals[1]-marginals[5]-k):min(marginals[1]-k, marginals[4]))
		{
			counter <- counter+1
			x[1, 1, counter] <- k
			x[1, 2, counter] <- l
			x[1, 3, counter] <- marginals[1] - x[1, 1, counter] - x[1, 2, counter]
			x[2, 1, counter] <- marginals[3] - x[1, 1, counter]
			x[2, 2, counter] <- marginals[4] - x[1, 2, counter]
			x[2, 3, counter] <- marginals[5] - x[1, 3, counter]
		}
	}
	
	log_denom_iter <- rep.int(0.0, times=counter)
	for (k in 1:counter)
	{
		log_denom_iter[k] <- sum(log(gamma(x[, , k]+1)))
	}
	prob_lauf <- exp(log_nom - log_denom_iter)
	nonrand_p <- sum(prob_lauf[prob_lauf <= prob_table])        
	rand_count <- sum(abs(exp(prob_lauf) - exp(prob_table)) < epsilon)        
	
	u <- runif(1)
	rand_p <- max(0.0, nonrand_p - u*rand_count*prob_table)
	
	return(list(nonrand_p=nonrand_p, rand_p=rand_p, prob_table=prob_table))
}


#############
# Fisher 22 #
#############


fisher22.model <- function() {
	return(list(model=list(typ="Fisher 2-by-2")))
}


mutoss.fisher22.model <- function() { return(new(Class="MutossMethod",
					label="Fisher's exact test in (2x2) tables",
					callFunction="fisher22.model",
					output=c("model"),
					info="<h2></h2>
							<p>(Marginal) Fisher's exact test in (2x2) tables</p> 
							<p></p>
							<h3>Reference:</h3>
							<ul>
							<li>Fisher, R. A. (1922). \"<i>On the interpretation of Chi^2 from contingency tables, and the calculation of P.</i>\" Journal of the Royal Statistical Society, 85 (1):87-94.</li>
							</ul>",
					parameters=list(
					)
			)) }


fisher22.marginal <- function(data, model) {
	#m <- dim(data)[3]
	#result <- vector(mode="numeric",length=m)
	result <- apply(data, 3, function(x) {fisher22_fast(x,2.0e-16)$rand_p} )
	return(list(pValues=result))
}


fisher22_fast <- function(obs, epsilon){
# obs = observations = a 2x2 table
	
#build marginals = (n1., n2., n.1, n.2)
	marginals <- c(sum(obs[1, ]), sum(obs[2, ]), sum(obs[, 1]), sum(obs[, 2]))
	
	n <- sum(obs)
	
	x <- array(data=rep.int(0.0, 2*2*max(marginals)*max(marginals)), c(2, 2, max(marginals)*max(marginals)))
	
#build log nominator statistic
	log_nom <- sum(log(gamma(marginals+1))) - log(gamma(n+1))
	
#build log denominator statistic
	log_denom <- sum(log(gamma(obs+1)))
	
	
#compute probability of observed table
	prob_table <- exp(log_nom - log_denom)
	
	
	nonrand_p <- 0.0
	rand_count <- 0
	
	dim1 <- min(marginals[1], marginals[3])
	
#traverse all possible tables with given marginals
	counter <- 0
	
	for (k in (marginals[1]-marginals[4]):dim1)
	{
		counter <- counter+1
		x[1, 1, counter] <- k
		x[1, 2, counter] <- marginals[1] - k
		x[2, 1, counter] <- marginals[3] - k
		x[2, 2, counter] <- marginals[2] - x[2, 1, counter]
	}
	
	log_denom_iter <- rep.int(0.0, times=counter)
	for (k in 1:counter)
	{
		log_denom_iter[k] <- sum(log(gamma(x[, , k]+1)))
	}
	prob_lauf <- exp(log_nom - log_denom_iter)
	nonrand_p <- sum(prob_lauf[prob_lauf <= prob_table])        
	rand_count <- sum(abs(exp(prob_lauf) - exp(prob_table)) < epsilon)        
	
	u <- runif(1)
	rand_p <- max(0.0, nonrand_p - u*rand_count*prob_table)
	
	return(list(nonrand_p=nonrand_p, rand_p=rand_p, prob_table=prob_table))
}

# 
# 
# Author: FrankKonietschke
###############################################################################



gao<-function(formula, data, alpha = 0.05, control = NULL , silent = FALSE){
	
	dat <- model.frame(formula, data)
	if (ncol(dat) != 2) {
		stop("Specify one response and only one class variable in the formula !")
	}
	if (is.numeric(dat[, 1]) == FALSE) {
		stop("Response variable must be numeric !")
	}
	response <- dat[, 1]
	group <- as.factor(dat[, 2])
	fl <- levels(group)
	a <- nlevels(group)
	N <- length(response)
	n <- aggregate(response,list(group),FUN="length")$x
	
	if (any(n <= 2)) {
		warn <- paste("The factor level", fl[n <= 2], "has got less than two observations!")
		stop(warn)
	}
	if (is.null(control)) {
		cont <- 1
	
	}

	if(! is.null(control)){
   if (!any(fl == control)) {
		stop("The dataset doesn't contain this control group!")
	}
	cont <- which(fl == control)
}
	
	
	C<-contrMat(1:a,"Dunnett",base=cont)
	
	
	
	
# ------------- Compute the pseudo-ranks------------------ #
	
	
	#browser()
	rx <- c()
	
	for (i in 1:N){
		
		help <- expand.grid(response[i],response)
		
		help1 <- (help[,1]>help[,2])+1/2*(help[,1]== help[,2])
		help2 <- data.frame(h1=help1,h2=group)
		samples2 <- split(help2$h1, help2$h2)
		
		pseudo <- sapply(1:a, function(arg) {
					
					mean(samples2[[arg]])
				})
		
		rx[i] <-N*mean(pseudo)
	}
	new.data <-data.frame(res=rx,group=group)
	
	
# ------------------ Point estimators ---------------------#
	
	pd <- 1/N*aggregate(new.data$res,list(group), FUN="mean")$x
	
	Cpd <- C%*%pd
	
# ------------ Compute the variance estimators ----------- #
	
	v1 <- 1/N^2*aggregate(new.data$res,list(group),FUN="var")$x
	lambda <- N/n
	v11 <-c(v1*lambda)
	v2 <- diag(v1*lambda)
	
	Cv <- C%*%v2%*%t(C)
	
	
# ------------------ Test Statistics ----------------------#
	
	T <-sqrt(N)*Cpd / sqrt(c(diag(Cv))) 
	

# ------------------ Degrees of freedom--------------------#
	
	
	ncont <-which((1:a)!= cont)
	
	numerator <- c(diag(Cv))^2
	denu1<-v1[cont]^2/(n[cont]^2*(n[cont]-1))
	denu2 <- v1[ncont]^2 /(n[ncont]^2*(n[ncont]-1))
	denu <- N^2*(denu1 + denu2)
	
	df <- numerator / denu
	
	
	
#-------------------------p-Values ------------------------#
	
	pv<- c()
	
	for (h in 1:(a-1)){
		pv[h]<- min(2*pt(T[h],df[h]),2-2*pt(T[h],df[h]))
	}
	
	adj.p <- p.adjust(pv,"hochberg")
	Rejected <- (adj.p<=alpha)
	
#------------------- Build the output ---------------------# 
	vj <- which((1:a) != cont)
	vi <- rep(cont, a - 1)
	cmpid <- sapply(1:(a-1), function(arg) {
				i <- vi[arg]
				j <- vj[arg]
				paste("F", "(", fl[j], ")", "-","F","(" ,fl[i],")", sep = "")
			})

	
	result <- data.frame(Comparison=cmpid, Estimator = Cpd, df=df, Statistic = T, P.Raw=pv,P.Adj=adj.p,Rejected = Rejected )
	rownames(result)<-1:(a-1)
	
	
	output = list(Info=data.frame(Sample=fl, Size=n, Single.Effects=pd),
			Analysis=result)
	if (! silent)
	{
		cat("#----Xin Gao's (2008) Non-Parametric Multiple Test Procedure","\n") 
		cat("#----Type of Adjustment: Hochberg", "\n")
		cat("#----Level of significance", "=", alpha ,"\n")
		cat("#----The procedure compares if the distribution functions F() are equal. The FWER is strongly controlled", "\n")
		print(result)
	}
	return(output)
	
}


gao.wrapper <- function(model, data,  alpha, control) {
	control <- NULL
	result <- gao(formula=formula(model), 
			data=data, 
			alpha = alpha,control)
	
	
	

	
	pvalues <- result$Analysis$P.Adj
	estimates <- result$Analysis$Estimator
	confint <- cbind(estimates, rep(NA, length(estimates)),rep(NA,length(estimates)))
	rownames(confint)<-result$Analysis$Comparison
	rejected1 <- result$Analysis$Rejected

	
	return(list(adjPValues=pvalues,rejected=rejected1,confIntervals= confint,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
}


mutoss.gao <- function() { return(new(Class="MutossMethod",
					label="Nonparametric Multiple contrast tests",
					errorControl="FWER",
					callFunction="gao.wrapper",
					output=c("adjPValues", "rejected","confIntervals","errorControl"),
					info="<h2>Nonparametric multiple contrast tests</h2>
							<p> This function computes Xin Gao's nonparametric multiple test procedures in an unbalanced one way layout. <p> 
							<p></p>
							<h3>Reference:</h3>
							<ul>
							<li>Gao, X. et al. \"<i>Nonparametric multiple comparison procedures for unbalanced one-way factorial designs.</i>\" Journal of Statistical Planning and Inference, 77, 2574-2591, 2008.</li>
							</ul>",
					parameters=list(model=list(type="ANY"),
							hypotheses=list(type="ANY"),
							alpha=list(type="numeric")
							
							
					
					)
			)) }

# Collection of elementary functions calculating an estimate
# of the number m0 resp. the proportion pi0 of true null hypotheses 
# in a finite family of hypotheses. 
# 
# Author: MarselScheer and WiebkeWerft
###############################################################################



storey_pi0_est = function(pValues, lambda) 
{
	pi0 = (sum(pValues > lambda) + 1) / (1 - lambda) / length(pValues)	
	return(list(pi0 = pi0, lambda = lambda))
}

mutoss.storey_pi0_est <- function() { return(new(Class="MutossMethod",
					label="Storey-Taylor-Siegmund (2004) Procedure",
					callFunction="storey_pi0_est",
					output=c("pi0", "lambda"),
					info="<h2>Storey-Taylor-Siegmund procedure</h2>\n\n\
							<p>The Storey-Taylor-Siegmund procedure for estimating pi0 is applied to pValues.\
							The formula is equivalent to that in Schweder and Spjotvoll (1982),\
							page 497, except the additional '+1' in the nominator that\
							introduces a conservative bias which is proven to be sufficiently large\
							for FDR control in finite families of hypotheses if the estimation\
							is used for adjusting the nominal level of a linear step-up test.</p>\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Storey, J. D., Taylor, J. E. and Siegmund, D. (2004). \"<i> Plots of P-values to evaluate many tests simultaneously. </i>\" Biometrika 69, 3, 493-502. </li>\n\
							<li>Huang, Y. and Hsu, J. (2007). \"<i> Strong control, conservative point estimation and simultaneous conservative consistency of false discovery rates: a unified approach.</i>\" JRSS B 66, 1, 187-205.</li>
							</ul>",
					parameters=list(pValues=list(type="numeric"), lambda=list(type="numeric"))
			)) }


ABH_pi0_est <- function(pValues)
{
	m <- length(pValues)
	index <- order(pValues)
	spval <- pValues[index]
	m0.m <- rep(0, m)
	for (k in 1:m) {
		m0.m[k] <- (m + 1 - k)/(1 - spval[k])
	}
	idx <- which(diff(m0.m, na.rm = TRUE) > 0)
	if (length(idx) == 0)
		grab <- 2
	else grab <- min(idx, na.rm = TRUE)+1
	pi0.ABH <- (ceiling(min(m0.m[grab], m))/m)
	return(list(pi0 = pi0.ABH))
}



mutoss.ABH_pi0_est <- function() { return(new(Class="MutossMethod",
					label="Hochberg-Benjamini (1990) lowest slope line method",
					callFunction="ABH_pi0_est",
					output=c("pi0"),
					info="<h2>Hochberg-Benjamini lowest slope line method</h2>\n\n\
							<p>The Lowest Slope Line (LSL) method of Hochberg and Benjamini for estimating pi0 is applied to pValues.\
							This method for estimating pi0 is motivated by the graphical approach proposed\
							by Schweder and Spjotvoll (1982), as developed and presented in Hochberg and Benjamini (1990).</p>\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Hochberg, Y. and Benjamini, Y. (1990). \"<i> More powerful procedures for multiple significance testing. </i>\" Statistics in Medicine 9, 811-818. </li>\n\
							<li>Schweder, T. and Spjotvoll, E. (1982). \"<i> Plots of P-values to evaluate many tests simultaneously.</i>\" Biometrika 69, 3, 493-502.</li>
							</ul>",
					parameters=list(pValues=list(type="numeric"))
			)) }


TSBKY_pi0_est <- function(pValues, alpha)
{	
	m <- length(pValues)
	adjp <- p.adjust(pValues,"BH")
	pi0.TSBKY <- ((m - sum(adjp < alpha/(1 + alpha), na.rm = TRUE)) / m)
	return(list(pi0=pi0.TSBKY))
}


mutoss.TSBKY_pi0_est <- function() { return(new(Class="MutossMethod",
					label="Benjamini, Krieger and Yekutieli (2006) two-step estimation method",
					callFunction="TSBKY_pi0_est",
					output=c("pi0"),
					info="<h2>Hochberg-Benjamini lowest slope line method</h2>\n\n\
							<p>The two-step estimation method of Benjamini, Krieger and Yekutieli for estimating pi0 is applied to pValues.
							It consists of the following two steps:</p>\
							<p>Step 1. Use the linear step-up procedure at level alpha' =alpha/(1+alpha). Let r1 be the number of\
							rejected hypotheses. If r1=0 do not reject any hypothesis and stop; if r1=m reject all m\
							hypotheses and stop; otherwise continue.</p>\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Benjamini, Y., Krieger, A. and Yekutieli, D. (2006). \"<i> Adaptive linear step-up procedures that control the false discovery rate. </i>\" Biometrika 93, 3, page 495. </li>\n\
							</ul>",
					parameters=list(pValues=list(type="numeric"),alpha=list(type="numeric"))
			)) }
# TODO: GB (MS) This will help will look terrible!
BR_pi0_est <- function(pValues, alpha, lambda=1, truncate = TRUE)
{	
	
	if ( lambda <= 0 || lambda >= 1/alpha) {
		stop('BR_pi0_est() : lambda should belong to (0, 1/alpha)')
	}
	
	m <- length(pValues)
	stage1 <- indepBR( pValues, alpha, lambda, silent = TRUE)
	pi0 <- ( m + 1 - sum(stage1$rejected) ) / ( m * ( 1 - lambda*alpha ) )
	
	if (truncate) {
		pi0 = min(1,pi0)
	}
	
	return(list(pi0=pi0))
}


mutoss.BR_pi0_est <- function() { return(new(Class="MutossMethod",
					label="Blanchard-Roquain (2009) estimation method",
					callFunction="BR_pi0_est",
					output=c("pi0"),
					info="<h2> Blanchard-Roquain estimation under independence </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Blanchard, G. and Roquain, E. \"<i> Adaptive False Discovery Rate Control under Independence and Dependence.</i>\" 
							Journal of Machine Learning Research 10:2837-2871, 2009. . </li>\n\
							</ul>\
							<p>The proportion of true\
							 nulls is estimated using the Blanchard-Roquain 1-stage procedure with parameter lambda,\
                             via the formula\n\
 
				 estimated pi<sub>0</sub> = ( m - R(alpha,lambda)  + 1) / ( m*( 1 - lambda * alpha ) )\n\
 
				 where R(alpha,lambda) is the number of hypotheses rejected by the BR 1-stage procedure,
				 alpha is FDR level control for this procedure and lambda a 
 				 parameter belonging to (0, 1/alpha) with default value 1. Independence of p-values is assumed.
 				 This estimate may in some cases be larger than 1; it is truncated to 1 if the parameter truncated=TRUE.
                 The estimate is used in the Blanchard-Roquain 2-stage step-up (with the non-truncated version)</p>",
					parameters=list(pValues=list(type="numeric"),alpha=list(type="numeric"),
									lambda=list(type="numeric", default=1), truncate=list(type="logical", default=TRUE))
			)) }

# Student - Newman - Keuls Test
# 
# Author: FrankKonietschke
###############################################################################
snk <- function(formula,data,alpha, MSE=NULL, df=NULL, silent = FALSE){
	
	dat <- model.frame(formula, data)
	if (ncol(dat) != 2) {
		stop("Specify one response and only one class variable in the formula")
	}
	if (is.numeric(dat[, 1]) == FALSE) {
		stop("Response variable must be numeric")
	}
	response <- dat[, 1]
	group <- as.factor(dat[, 2])
		fl <- levels(group)
		a <-nlevels(group)
		N <- length(response)
		samples <- split(response,group)
		n <- sapply(samples,"length")
		mm <- sapply(samples,"mean")
		vv <- sapply(samples,"var")
		if (is.null(MSE)){
		MSE <- sum((n-1)*vv)/(N-a)
	     }
		if (is.null(df)){
		df <- N-a
			}
	
		nc <- a*(a-1)/2
		order.h1 <- data.frame(Sample=fl, Size=n, Means=mm,Variance=vv)
		
		ordered <- order.h1[order(order.h1$Means,decreasing=FALSE), ]
		rownames(ordered) <- 1:a
		
		
		
  #---------------- Compute helping indices ----------#
	
	i <- 1:(a-1)
	
	h1 <- list()
	for(s in 1:(a-1)){
		
		h1[[s]]<- i[1:s]
		
	}
	vi <- unlist(h1)
	
	j <- a:2
	h2 <-list()
	
	for (s in 1:(a-1)){
		h2[[s]] <- j[s:1]
	}
	
	vj <- unlist(h2)
	
	h3 <- list()
	h4 <- list()
	
	for (s in 1:(a-1)){
		h3[[s]] <- rep(j[s],s)
		h4[[s]] <- rep(i[s],s)
	}
	Nmean <- unlist(h3)
	Step  <- unlist(h4)
	
	
	#--------Compute the Mean Differences---------#
	
	mean.difference <- sapply(1:nc,function(arg){
				i <- vi[arg]
				j <- vj[arg]
				(ordered$Means[j]-ordered$Means[i])
				
			})
	mean.difference <- round(mean.difference, 4)
	# ------- Compute the test statistics --------#
	
	T <- sapply(1:nc,function(arg){
				i<-vi[arg]
				j<-vj[arg]
				(ordered$Means[j]-ordered$Means[i])/sqrt(MSE/2*(1/ordered$Size[i] + 1/ordered$Size[j]))
				
			})


	T <- round(T, 4)
	
	# ----- Compute now the critical value -----#
	
	quantiles <- qtukey(1-alpha,Nmean,df)
	pvalues <- ptukey(T,Nmean,df,lower.tail=FALSE)
	
	#---- Calculate the rejected Hypotheses ------# 
	
	Rejected1 <- (pvalues<alpha)
	
	#------------ Names for the Output -----------# 
	
	names.ordered <- sapply(1:nc, function(arg){
				i <- vi[arg]
				j <- vj[arg]
				paste(ordered$Sample[j], "-", ordered$Sample[i], sep="")
			})
	
	# ------ Compute now the rejected statistics-----#
	
	for (s in 1:nc){
		if (Rejected1[s]==FALSE){
			Under1 <- (vj[s]>=vj)
			Under2 <- (vi[s]<=vi)
			Under3 <- Under1 * Under2
			Under4 <- which(Under3==1)
		
			Rejected1[Under4] <- FALSE
			
		}
	}
	
	
	#-----Prepare the pValues for the Output----#
	
	Out1 <- (pvalues < alpha)
	Out2 <- (Rejected1 == FALSE)
	Out3 <- Out1 * Out2
	Out4 <- (Out3 == 1)
	pvalues <- round(pvalues,4)
	pvalues[Out4] <- paste(">",alpha)
	
	variances.output <- data.frame(Overall=MSE, df=df)
	Comparison <- data.frame(Comparison=names.ordered,Diff=mean.difference, Statistic=T, Adj.P=pvalues, Rejected=Rejected1, Layer = Step)
	

      if (! silent)
	{
		cat("#----Student-Newman-Keuls (1927; 1939; 1952) rejective Multiple Test Procedure  \n\n")
		cat("#----Attention: The SNK test controls the FWER only in the WEAK sense  \n\n")	
	}
	
		 	
	result <- list(Ordered.Means = ordered, Variances=variances.output,
			SNK = Comparison)
	
	return(result)	
	
	
			}
			
			
snk.wrapper <- function(model, data,  alpha, silent=FALSE) {
				
				result <- snk(formula=formula(model), 
						data, 
						alpha = alpha)
						difference <- result$SNK$Diff#, result$SNK$Statistic, result$SNK$Layer)
						diffm<-cbind(difference,rep(NA,length(difference)),rep(NA,length(difference)))
						diffm<-matrix(diffm,nrow=length(difference))
				rownames(diffm)<-result$SNK$Comparison
				return(list(adjPValues=result$SNK$Adj.P,rejected=result$SNK$Rejected,statistics=result$SNK$Statistic,
								confIntervals= diffm,errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
			}
			
			
mutoss.snk <- function() { return(new(Class="MutossMethod",
								label="Student-Newman-Keuls Test",
								errorControl="FWER.weak",
								callFunction="snk.wrapper",
								output=c("adjPValues", "rejected", "confIntervals", "errorControl"),
								info="<h2>Student - Newman - Keuls rejective test procedure.\
                                       The procedure controls the FWER in the WEAK sense. </h2>\n\n\
										<p> The Newman-Keuls procedure is based on a stepwise or \
                                           layer approach to significance testing. Sample means are \
                                           ordered from the smallest to the largest. The largest \
                                           difference, which involves means that are r = p steps apart, \
                                           is tested first at alpha level of significance; if significant, \
                                           means that are r = p - 1 steps apart are tested at \alpha level \
                                           of significance and so on.\ 
										</p>\n\
										<h3>Reference:</h3>\
										<ul>\
										<li>Keuls M (1952).  \"<i> The use of the studentized range in \
                                            connection with an analysis of variance
										 </i>\" Euphytica 1 37, 112-122. </li>\n\
										</ul>",
								parameters=list(
										data=list(type="data.frame"),
										model=list(type="ANY"),
										alpha=list(type="numeric")
								)
						)) }
			
			
	
	
		
			
			

			

# Here all implemented concrete SUD-Procedures can be found.  
# 
# Author: MarselScheer and WerftWiebke
###############################################################################


#++++++++++++++++++++++++++++   OutputFkt    +++++++++++++++++++++
printRejected = function(rejected, pValues = NULL, adjPValues = NULL) 
{
	cat("Number of hyp.:\t", length(rejected), "\n")
	cat("Number of rej.:\t", sum(rejected), "\n")
	idx <- which(rejected)
	
	if (length(idx) != 0)
	{
		output <- data.frame(rejected = idx)
		if (!is.null(pValues))
		{
			output <- data.frame(output, pValues[idx])
			names(output)[length(names(output))] <- "pValues"
		}
		
		if (!is.null(adjPValues))
		{
			output <- data.frame(output, adjPValues[idx])
			names(output)[length(names(output))] <- "adjPValues"
		}
		
		
		if (!is.null(pValues))
			# sorting by pValues
			output 	<- output[order(output$pValue), ]
		else
		{
			if (!is.null(adjPValues))
				# no pValues availible, sorting by adjPValues
				output 	<- output[order(output$adjPValues), ]			
		}
		
		rownames(output) <- 1:length(idx)
		
		print(output)
	}
}
#----------------------------   OutputFkt    ---------------------

#Rom_simpleImplementation <- function(pValues, alpha) 
#{
#	# ROM, D. M. (1990). A sequentially rejective test procedure based 
#	# on a modified Bonferroni inequality. Biometrika 77, 663-665.
#	
#	# Formula for the critical values is taken from
#	# FINNER, H. and ROTERS, M. (2002). Multiple hypotheses testing and 
#	# expected type I errors. Ann. Statist. 30, 220-238.
#	# Notice: 	The smallest critical value in this paper is alpha_1!!	
#	#			Thus the critical values are calculated in this manner, and
#	#			at the end the order is reversed.
#	
#	# ++++++ Calculating critical values
#	
#	# TODO: Perhaps there are computational problems if too many hypotheses are tested.
#	len 		<- length(pValues)
#	criticalValues 	<- rep(0, times=len)
#	criticalValues[1]	<- alpha
#	
#	# TODO: !! firstSum_k[14:len] is constant for example if alpha=5%, len=200
#	# firstSum_k := SUMME(alpha^i, i=1..(k-1))	
#	firstSum_k <- cumsum(c(0, sapply(1:(len-1), function(i) alpha^i)))
#	
#	criticalValues[2] <- 1/2 * (firstSum_k[2] - 0)
#	secondSummand <- function(i) choose(k,i) * criticalValues[i+1]^(k-i)
#	for (k in 3:len)
#	{		
#		# TODO: secondSum can be calculated faster!
#		secondSum 	<- sum(sapply(1:(k-2), secondSummand))
#		criticalValues[k] 	<- 1 / k * (firstSum_k[k] - secondSum)  
#		#cat("1 ", secondSum, "\n")
#	}	 
#	criticalValues <- criticalValues[len:1]
#
#	# ------ Calculating critical Values
#
#	SU(pValues, criticalValues)
#
#}


# TODO: MS !! Discussion about big n !!
rom <- function(pValues, alpha, silent = FALSE) 
{
	# 
	# Remark: The critical values calculated by this procedure were
	#			compared with the critical values calculated by Rom
	#			himself in his paper and they are the same.
	
	
	# Formula for the critical values is taken from
	# FINNER, H. and ROTERS, M. (2002). Multiple hypotheses testing and 
	# expected type I errors. Ann. Statist. 30, 220-238.
	# Notice: 	The smallest critical value in this paper is alpha_1!!	
	#			Thus the critical values are calculated in this manner, and
	#			at the end the order is reversed.
	
	# ++++++ Calculating critical Values
	
	# TODO: MS perhaps there are computational problems if too many hypotheses are tested.
	len 		<- length(pValues)
	criticalValues 	<- rep(0, times=len)
	criticalValues[1]	<- alpha
	
	# TODO: MS !! firstSum_k[14:len] is constant for example if alpha=5%, len=200
	# firstSum_k 	:= SUMME(alpha^i, i=1..(k-1))
	firstSum_k 	<- cumsum(c(0, sapply(1:(len-1), function(i) alpha^i))) 
	
	criticalValues[2] <- 1/2 * (firstSum_k[2] - 0)
	
	# SUMME(binomial(k,i) * alpha_{i+1}^{k-i}, i=1..(k-2))
	# = SUMME(aki, i=1..(k-2))
	# = SUMME(binomial(k-1, i) * k / (k-i) * a(k-1)i * alpha_{i+1}, i=1..(k-2))		
	# for k = 3 and i = 1
	# aki and binKoef actually has 2 dimensions, the k-dimension and the i-dimension. 
	# But in this code we will only work with the i-dimension. In every step of 
	# the for-loop aki[i] will be updated.
	aki 	<- rep(0, times = (len-2))	
	binKoef <- rep(1, times = (len-2))
	
	binKoef[1] 	<- choose(3,1)
	aki[1] 		<- criticalValues[2]^(3-1)
	
	for (k in 3:len)
	{						
		secondSummand 		<- sum(binKoef[1:(k-2)] * aki[1:(k-2)])
		criticalValues[k] 			<- 1/k * (firstSum_k[k] - secondSummand)
		
		# updating the vectors for the next step
		binKoef[k-1]		<- (k+1) * k / 2 #choose(k+1, k-1)
		binKoef[1:(k-2)]	<- binKoef[1:(k-2)] * (k+1) / (k:3)	
		
		aki[k-1]			<- criticalValues[k]^2
		aki[1:(k-2)]		<- aki[1:(k-2)] * criticalValues[2:(k-1)]
	}	 
	criticalValues <- criticalValues[len:1]
	# ------ Calculating critical Values
	
	rejected <- SU(pValues, criticalValues)
	
	if (! silent)
	{
		cat("\n\n\t\tRom's (1990) step-up procedure\n\n")
		printRejected(rejected, pValues, NULL)
	}
	
	# TODO: MS calculating adjustedPValues for ROM numerically
	return(list(rejected = rejected, criticalValues = criticalValues,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha))
	)
}


mutoss.rom <- function() { return(new(Class="MutossMethod",
					label="Rom's (1990) step-up",
					errorControl="FWER",
					callFunction="rom",
					output=c("rejected", "criticalValues","errorControl"),
					info="<h2>Rom's step-up procedure</h2>\n\n\
							<p>Rom's step-up-procedure is applied to pValues. The procedure 
                               controls the FWER in the strong sense if the pValues are
 							   stochastically independent.</p>
  
                           <p> This function calculates the critical values by the formula given
                               in Finner, H. and Roters, M. (2002) based on the joint distribution
								of order statistics. After that a step-up test
                               is performed to reject hypotheses associated with pValues.</p>
  
                           <p> Since the formula for the critical values is recursive,
                               the calculation of adjusted pValues is far from obvious and is
                               not implemented here.</p>
							<h3>Reference:</h3>\
							<ul>\
							<li>Rom, D. M. \"<i> A sequentially rejective test procedure based on a modified Bonferroni inequality. </i>\" Biometrika 77, 663-665. </li>\n\
							</ul>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }

#-------------------- Holm's Step-down--------------------#

holm <- function(pValues, alpha, silent = FALSE) 
{
	m <- length(pValues)
	criticalValues <- sapply(1:m, function(i) alpha/(m-i+1))
	adjPValues <- p.adjust(pValues, "holm")
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\t\tHolm's (1979) step-down Procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	
	return(list(adjPValues = adjPValues,
			rejected = rejected, criticalValues=criticalValues,
			errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha))
	)
}

mutoss.holm <- function() { return(new(Class="MutossMethod",
					label="Holm's (1979) step-down",
					errorControl="FWER",
					callFunction="holm",
					output=c("adjPValues", "rejected", "criticalValues","errorControl"),
					info="<h2>Holm's step-down-procedure</h2>\n\n\
							<p> Holm's step-down-procedure is applied to pValues. It controls
								the FWER in the strong sense under arbitrary dependency.</p>\n\
							<p>	Holm's procedure uses the same critical values as the Hochberg's procedure, namely  <i>c(i)=&alpha;/(m-i+1)</i>, 
								but is a step-down version while Hochberg's method is a step-up version of the Bonferroni test.
								Holm's method is based on the Bonferroni inequality and is valid regardless of the joint
								distribution of the test statistics, whereas Hochberg's method relies on the assumption that 
								Simes' inequality holds for the joint null distribution of the test statistics. If this assumption is met, Hochberg's
								step-up procedure is more powerful than Holm's step-down procedure.</p>\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Holm, S. (1979). \"<i> A simple sequentially rejective multiple test procedure. </i>\" Scand. J. Statist. Vol. 6, 65-70. </li>\n\
							<li>Huang, Y. and Hsu, J. (2007). \"<i> Hochberg's step-up method: cutting corners off Holm's step-down method. </i>\" Biometrika, 94(4):965-975.</li>
							</ul>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }


# TODO: MS Probably jointCDF.unif should probably be moved to some math.R or so.
# TODO: MS !! jointCDF.unif: There are numerical issues because the of accuracy of doublePrecison
# TODO: MS !! How to communicate numerical issues to the user. 
jointCDF.orderedUnif = function(vec) 
{
	# vec is not ordered. Thus the probability must be 0
	if (!all(order(vec) == 1:length(vec)))
	{		
		print("ORDER!")
		print(vec)
		print(order(vec))
		vecName <- deparse(substitute(vec))
		warning(paste("jointCDF.unif(): The variable", vecName, "is not ordered. Thus the probability is 0!"))
		return(0)
	}
	
	if (min(vec) <= 0) return(0)
	vec[ vec > 1 ] <- 1
	
	if (100 < length(vec))
		warning("Length of the argument is longer than 100. Calculated value may not be useable!")
	
	# By Bolshev's recursion
	# 	P(U_{1:n} <= vec[1], ..., U_{n:n} <= vec[n])
	# 	= Fn(vec[1], ..., vec[n])
	# 	= 1 - sum( binom(n, j) * Fj(vec[1], ..., vec[j]) * (1-vec[j+1])^(n-j), j=0..n-1)
	# with F0 = 1.
	# The variable Fj[k+1] used in this function will correspond to
	# Fk(vec[1], ..., vec[k]) for all k = 0 .. n.
	# So Fj[1] is F0 = 1, Fj[2] is F1(vec[1]) and so on. 
	Fj <- rep(0, times = length(vec) + 1)	
	
	Fj[1] <- 1 # F0
	# consider k; 
	# 	Fj[k+1]; 
	#	Fk(vec[1], ..., vec[k]) 
	#	= 1 - sum( binom(k, s) * Fs(vec[1], ..., vec[s]) * (1-vec[s+1])^(n-s) , s=0..k-1 )
	#	= 1 - sum( choose(k,s) * Fj[s+1] * (1 - vec[s+1])^(k-s), s=0..k-1 )
	
	summand <- function(s) choose(k,s) * Fj[s+1] * (1 - vec[s+1])^(k-s)
	
	
	for(k in 1:length(vec))
		Fj[k+1] <- 1 - sum( sapply(0:(k-1), summand))
	
	return(Fj[length(vec)+1])
}


calculateBetaAdjustment = function(n, startIDX_SUD, alpha, silent = FALSE, initialBeta = 1, maxBinarySteps = 50, tolerance = 0.0001) 
{
	#+++++++++++++++++++++++++++   Subfunctions   +++++++++++++++++++++++++
	#probability mass function
	pmf <- function(criticalValues, startIDX_SUD, n, n0, j) 
	{# Calculates P_{n,n0}(V = j) for a set of critical Values.
		# Formulas are from Finner, Gontscharuk, Dickhaus: FDR controlling step-up-down tests
		# 	related to the asmptotically optimal rejection curve. (to appear)
		if (n0 < j) return(0)
		
		n1 <- n - n0
		
		if (startIDX_SUD <= n1)
		{
			if (j == 0)
				return(	choose(n0,j)
								* 1 
								* (1-criticalValues[n1 + j + 1])^(n0-j)
				)
			return(	choose(n0,j)
							* jointCDF.orderedUnif(criticalValues[(n1 + 1):(n1 + j)]) 
							* (1-criticalValues[n1 + j + 1])^(n0-j)
			)
		}
		
		if ((n1 < startIDX_SUD) && (j < startIDX_SUD - n1 - 1))
		{			
			if (n1 + j == 0) # <=> n1 == 0 and j == 0
				return(choose(n0,j)
								* jointCDF.orderedUnif(c(rep(1 - criticalValues[startIDX_SUD], times = (n - startIDX_SUD + 1)), 1 - criticalValues[(startIDX_SUD - 1):(n1 + j + 1)]))
								* 1			
				)
			return(choose(n0,j)
							* jointCDF.orderedUnif(c(rep(1 - criticalValues[startIDX_SUD], times = (n - startIDX_SUD + 1)), 1 - criticalValues[(startIDX_SUD - 1):(n1 + j + 1)]))
							* criticalValues[n1 + j]^j
			)
		}
		
		if ((n1 < startIDX_SUD) && (j == startIDX_SUD - n1 - 1))
		{
			if (n1 + j == 0) # <=> n1 == 0 and j == 0
				return(choose(n0,j)
								* jointCDF.orderedUnif(rep(1 - criticalValues[startIDX_SUD], times = (n - startIDX_SUD + 1)))
								* 1
				)			
			return(choose(n0,j)
							* jointCDF.orderedUnif(rep(1 - criticalValues[startIDX_SUD], times = (n - startIDX_SUD + 1)))
							* criticalValues[n1 + j]^j
			)
		}
		
		
		if ((n1 < startIDX_SUD) && (j == startIDX_SUD - n1))
			return(choose(n0,j)
							* jointCDF.orderedUnif(rep(criticalValues[startIDX_SUD], times=j))
							* (1 - criticalValues[n1 + j + 1])^(n0 - j)
			)
		
		if ((n1 < startIDX_SUD) && (startIDX_SUD - n1 < j))
			return(choose(n0,j)
							* jointCDF.orderedUnif(c(rep(criticalValues[startIDX_SUD], times=(startIDX_SUD - n1)), criticalValues[(startIDX_SUD + 1):(n1 + j)]))
							* (1 - criticalValues[n1 + j + 1])^(n0 - j)
			)		
	}
	calculateMaximumUpperFDRBound <- function(criticalValues, n, startIDX_SUD) 
	{
		# Formulas are from Finner, Gontscharuk, Dickhaus: FDR controlling step-up-down tests
		# 	related to the asmptotically optimal rejection curve. (to appear)
		
		# Calculating the probability mass function of V under a DU(n_0, n) model
		# 	n 	= Number of hypotheses
		# 	n0 	= Number of true hypotheses
		# 	pm[j+1,n0+1] = P_{n,n0}(V = j)
		
		pm <- sapply( 1:n,
				function(n0) 
				{
					sapply(0:n, # actually we only had to go to n0, but in this way pm will be a matrix 
							function(j) 
							{
								pmf(criticalValues, startIDX_SUD, n, n0, j)								
							}
					)
				}
		)
		# now the special case n0 = 0, then P_{n,n0}(V=0) = 1
		pm <- cbind(c(1, rep(0, times = n)), pm)
		
		# just for plausibility, gonna look if P_{n,n0}(V in {0, ..., n}) = 1 for every n0 = 1, ..., n
		rng <- range(colSums(pm))
		if (rng[2]-rng[1] > 0.01)
			warning("Maximum upper bounds of FDR probably not accurate!")
		
		# Calculating b(n,n0|startIDX_SUD) for every n0
		# which is a uppper bound for the FDR according to Finner, Gontscharuk, Dickhaus.
		
		bn <- sapply(1:n, 
				function(n0) 
				{
					n1 <- n - n0 
					n0 * sum(sapply(1:n0, function(j) criticalValues[n1 + j]/(n1 + j) * pm[j, n0]))
				}
		)
		bn
		
		return(max(bn))		
	}
	SearchInitialBetaInterval <- function(n, startIDX_SUD, alpha, initialBeta) 
	{
		# searches two beta's: beta1 and beta2 such that the beta2-adjusted AORC
		# controls the FDR and the beta1-adjusted AORC not and beta2 + step = beta1
		
		criticalValues <- sapply(1:n, function(i) i * alpha / (n + initialBeta - i * (1 - alpha)))
		UpperFDRBound <- calculateMaximumUpperFDRBound(criticalValues, n, startIDX_SUD)					
		
		# startBeta controls the FDR, thus beta must be reduced! 
		if (UpperFDRBound < alpha)
		{
			FDRControlOfInitialBeta <- TRUE
			step <- -1
		}
		else
		{
			FDRControlOfInitialBeta <- FALSE
			step <- 1
		}
		
		beta <- initialBeta + step
		intervalFound <- FALSE
		while (!intervalFound) 
		{			
			criticalValues <- sapply(1:n, function(i) i * alpha / (n + beta - i * (1 - alpha)))
			UpperFDRBound <- calculateMaximumUpperFDRBound(criticalValues, n, startIDX_SUD)
			
			# if we have FDRControl by the initialBeta but not for beta, then we are done!
			# Also if we have not control of the FDR by the initalBeta but for beta, then we are done!
			if (xor(UpperFDRBound < alpha, FDRControlOfInitialBeta))
				intervalFound <- TRUE
			else
				# initialBeta and beta both control the FDR or both do not control the FDR
				beta <- beta + step
		}
		
		if (FDRControlOfInitialBeta)
			return(c(beta, beta + 1))
		
		return(c(beta - 1, beta))
	}
	#---------------------------   Subfunctions   -------------------------
	
	if (!silent)
		cat("Searching initial interval to start the bisection approach.\n")
	
	betaInt <- SearchInitialBetaInterval(n, startIDX_SUD, alpha, initialBeta)
	
	beta 			<- betaInt[2]
	step 			<- (betaInt[2] - betaInt[1]) / 2
	numberOfSteps 	<- 0
	
	if (!silent) 
		cat("Starting the bisection approach.\n")
	
	lastFeasibleBeta 			<- beta
	lastFeasibleUpperFDRBound 	<- -Inf 
	while(numberOfSteps < maxBinarySteps)
	{		
		numberOfSteps <- numberOfSteps + 1
		criticalValues <- sapply(1:n, function(i) i * alpha / (n + beta - i * (1 - alpha)))
		UpperFDRBound <- calculateMaximumUpperFDRBound(criticalValues, n, startIDX_SUD)
		
		if (!silent)
			cat("Step ", numberOfSteps, ": beta =", beta, " => Upper FDR bound =", UpperFDRBound, "\n")
		
		if (alpha - tolerance <= UpperFDRBound && UpperFDRBound <= alpha)
		{
			if (!silent)
			{
				cat("\nUpper FDR bound element in [alpha - tolerance, alpha]\n")
				cat("Returned beta =", beta, " => Upper FDR bound =", UpperFDRBound, "\n")
			}
			return(beta)
		}
		
		if (UpperFDRBound > alpha)
			beta <- beta + step
		else
		{	
			lastFeasibleBeta 			<- min(beta, lastFeasibleBeta)
			lastFeasibleUpperFDRBound 	<- max(UpperFDRBound, lastFeasibleUpperFDRBound) 
			beta <- beta - step
		}
		
		step <- step / 2			
	}	
	
	
	if(!silent)
		cat("\nReturned beta =", lastFeasibleBeta, " => Upper FDR bound =", lastFeasibleUpperFDRBound, "\n" )
	
	return(lastFeasibleBeta)
}

#------------------------ AORC---------------------#

aorc <- function(pValues, alpha, startIDX_SUD = length(pValues), betaAdjustment, silent = FALSE) 
{	
	len <- length(pValues)
	if (missing(betaAdjustment))
	{
		if (!silent)
			cat("Using calculateBetaAdjustment() to set the missing parameter betaAdjustment.\n")
		betaAdjustment = calculateBetaAdjustment(len, startIDX_SUD, alpha, silent)
	}
	
	criticalValues <- sapply(1:len, function(i) i * alpha / (len + betaAdjustment - i * (1 - alpha))) 
	rejected <- SUD(pValues, criticalValues, startIDX_SUD)
	
	if (! silent)
	{
		cat("\n\n\t\tAsymptotically optimal rejection curve (2009)\n\n")
		printRejected(rejected, pValues)
	}
	
	return(list(rejected = rejected, criticalValues = criticalValues,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}

mutoss.aorc <- function() { return(new(Class="MutossMethod",
					label="Asymptotically optimal rejection curve (2009)",
					errorControl="FDR",
					callFunction="aorc",
					output=c("criticalValues", "rejected", "errorControl"),
					info="<h2>Step-up-down procedure based on the asymptotically optimal rejection curve</h2>\n\n\
							<p> The graph of the function f(t) = t / (t * (1 - alpha) + alpha) is called the asymptotically \
							optimal rejection curve. Denote by finv(t) the inverse of f(t). Using the \ 
							critical values finv(i/n) for i = 1, ..., n yields asymptotic FDR control. \
							To ensure finite FDR control it is possible to adjust f(t) by a factor. The \
							function calculateBetaAdjustment() calculates a beta such that (1 + beta / n) * f(t) \
							can be used to control the FDR for a given finite sample size. If beta is not provided,
							calculateBetaAdjustment() will be called automatically.\ 
							</p>\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Finner, H., Dickhaus, T. & Roters, M. \"<i> On the false discovery rate and an asymptotically \
							optimal rejection curve. </i>\" The Annals of Statistics 37, 596-618. </li>\n\
							</ul>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"), 
							startIDX_SUD=list(type="integer", label="Start Index for Step-Up-Down", optional=TRUE),
							betaAdjustment=list(type="numeric", label="Adjustment factor beta_n", optional=TRUE))
			)) }



#aorc( runif(30), 0.05, 5)

#----------------- Banjamini Liu----------------#

BL <- function(pValues, alpha, silent=FALSE) {
	m <- length(pValues)
	criticalValues <- sapply(1:m, function(i) 1-(1-min(1, (m*alpha)/(m-i+1)))^(1/(m-i+1)))
	rejected <- SD(pValues, criticalValues)
	index <- order(pValues) 	# index for sorting pValues
	rindex <- order(index) 		# reversed index to obtain the original order 
	spval <- pValues[index]
	adjPValues <- vector(mode="numeric",length=m)
	adjPValues[1] <- min(1 - (1 - spval[1])^m, 1)
	for (i in 2:m) adjPValues[i] <- max(adjPValues[i - 1], ifelse((alpha*m)/(m-i+1)<=1, ((m-i+1)/m)*(1 - (1 - spval[i])^(m - i + 1)), 0))#(0)?! 
	adjPValues <- adjPValues[rindex] # obtain the original order
	#rejected <- (adjustedPValues <= alpha) # either this or SUD leads to rejected 
	if (! silent)
	{
		cat("\n\n\t\tBenjamini-Liu's (1999) step-down procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, criticalValues=criticalValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}
mutoss.BL <- function() { return(new(Class="MutossMethod",
					label="Benjamini-Liu (1999) step-down",
					errorControl="FDR",
					callFunction="BL",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("Independent test statistics."),
					info="<h2>Benjamini-Liu (1999) step-down </h2>\n\n
							<h3>Reference:</h3>
							<ul>
							<li>Bejamini, Y. and Liu, W. (1999). \"<i> A step-down multiple hypotheses testing procedure that controls the false discovery rate under independence . </i>\" Journal of Statistical Planning and Inference Vol. 82(1-2): 163-170. </li>\n
							</ul>
							<p>Benjamini-Liu's step-down procedure is applied to pValues. 
							The procedure controls the FDR if the corresponding test statistics are stochastically independent. 
							In Benjamini and Liu (1999) a large simulation study concerning the power of the two procedures suggested that the BL step-down procedure is more powerfull then the Linear Step-Up (BH) when the number of hypotheses is small. 
							This is also the case when most hypotheses are far from the null. The BL step-down method calculates critical values according to Benjamin and Liu (1999), i.e.
							<i>c<sub>i</sub> = 1 - (1 - min(1, m*&alpha;/(m-i+1)))<sup>(1/(m-i+1))</sup></i> for <i>i = 1,...,m</i>, 
							where <i>m</i> is the number of hypotheses tested. 
							Then, let <i>k</i> be the smallest <i>i</i> for which <i>P<sub>(i)</sub> > c<sub>i</sub></i> and reject associated hypotheses <i>H<sub>(1)</sub>,...,H<sub>(k-1)</sub></i>.",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }

#-------------------- BH Linear Step Up--------------------#

BH <- function(pValues, alpha, silent=FALSE) {
	m <- length(pValues)
	criticalValues <- sapply(1:m, function(i) (i*alpha)/m)
	adjPValues <- p.adjust(pValues, "BH")
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\t\tBenjamini-Hochberg's (1995) step-up procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(
					adjPValues=adjPValues, 
					criticalValues=criticalValues,
					rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}
mutoss.BH <- function() { return(new(Class="MutossMethod",
					label="Benjamini-Hochberg (1995) step-up",
					errorControl="FDR",
					callFunction="BH",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("independence or positive regression dependency"),
					info="<h2>Benjamini-Hochberg (1995) Linear Step-Up Procedure </h2>\n\n\
							<p>The Benjamini-Hochberg (BH) linear step-up procedure controls the FDR if the test statistics are stochastically independent or satisfy positive regression dependency  (see Benjamini and Yekutieli 2001 for details).
							In their seminal paper, Benjamini and Hochberg (1995) suggest the False Discovery Rate (FDR) as an alternative error criterion to the Family-Wise-Error-Rate and show that for <i>0<=m<sub>0</sub><=m</i> independent pValues corresponding to true null hypotheses 
							and for any joint distribution of the <i>m<sub>1</sub>=m-m<sub>0</sub></i> p-values corresponding to the non-null hypotheses the FDR is controlled at level <i>(m<sub>0</sub>/m)*&alpha;</i>.  
							Benjamini and Yekutieli show (2001) that this procedure controls the FDR in a much more general setting i.e. when the PRDS condition is satisfied. 
							#####Benjamini et al. (2006) improved by adaptive procedures which use an estimate of <i>m<sub>0</sub></i> and apply the BH method at level <i>&alpha;'=&alpha*m/m<sub>0</sub></i>, to fully exhaust the desired level <i>&alpha;</i>.</p>\n
							<h3>References:</h3>\
							<ul>\
							<li>Benjamini, Y. and Hochberg, Y. (1995). \"<i> Controlling the false discovery rate: A practical and powerful approach to mulitple testing. </i>\" Journal of the Royal Statistical Society, Series B, 57:289-300. </li>\n\
							<li>Benjamini, Y. and Yekutieli, D. (2001). \"<i> The control of the false discovery rate in multiple testing under dependency. </i>\" Annals of Statistics, 29(4):1165-1188. </li>\n\
							<li>Benjamini, Y., Krieger, A. and Yekutieli, D. \"<i> Adaptive linear step-up procedures that control the false
							discovery rate. </i>\" Biometrika, 93(3):491-507, 2006. </li>\n\
							</ul>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }


BY <- function(pValues, alpha, silent=FALSE) {
	m <- length(pValues)
	a <- sum(1/(1:m))
	criticalValues <- sapply(1:m, function(i) (i*alpha)/(a*m))
	#rejected <- SU(pValues, criticalValues)
	adjPValues <- p.adjust(pValues, "BY")
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\t\tBenjamini-Yekutieli's (2001) step-up procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, criticalValues=criticalValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}
mutoss.BY <- function() { return(new(Class="MutossMethod",
					label="Benjamini-Yekutieli (2001) step-up",
					errorControl="FDR",
					callFunction="BY",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("any dependency structure"),
					info="<h2>Benjamini-Yekutieli (2001) step-up procedure </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Bejamini, Y. and Yekutieli, D. (2001). \"<i> The control of the false discovery rate in multiple testing under dependency. </i>\" Annals of Statistics, 29(4):1165-1188. </li>\n\
							</ul>
							<p>The Benjamini-Yekutieli step-up procedure is applied to pValues. 
							The procedure ensures FDR control for any dependency structure. 
							The critical values of the Benjamini-Yekutieli (BY) procedure are calculated by 
							replacing the <i>&alpha;</i> of the Benjamini-Hochberg procedure by <i>&alpha;/(&sum;1/i)</i>, i.e.
							<i>c(i)=i*&alpha;/m*(&sum;1/i)</i> for <i>i=1,...,m</i>. For large number <i>m</i> of hypotheses the critical values of the BY procedure and the 
							BH procedure differ by a factor <i>log(m)</i>. Benjamini and Yekutieli (2001) showed that this step-up procedure controls
							the FDR at level <i>&alpha;*m/m<sub>0</sub></i> for any test statistics dependency structure.</p>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }

hochberg <- function(pValues, alpha, silent=FALSE) {
	m <- length(pValues)
	criticalValues <- sapply(1:m, function(i) alpha/(m-i+1))
	#rejected <- SU(pValues, criticalValues)
	adjPValues <- p.adjust(pValues, "hochberg")
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\t\tHochberg's (1988) step-up procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, criticalValues=criticalValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
}
mutoss.hochberg <- function() { return(new(Class="MutossMethod",
					label="Hochberg (1988) step-up",
					errorControl="FWER",
					callFunction="hochberg",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("independent tests"),
					info="<h2>Hochberg (1988) step-up procedure </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Hochberg, Y. (1988). <i> A sharper Bonferroni procedure for multiple tests of significance. </i> Biometrika, 75:800-802. </li>\n\
							<li>Huang, Y. and Hsu, J. (2007). <i> Hochberg's step-up method: cutting corners off Holm's step-down method. </i>Biometrika, 94(4):965-975.</li>\n							
							</ul>
							<p>The Hochberg step-up procedure is based on marginal p-values. It controls the FWER in the strong 
							sense under joint null distributions of the test statistics that satisfy Simes' inequality.
							The Hochberg procedure is more powerful than Holm's (1979) procedure, but the test statistics need to be
							independent or have a distribution with multivariate total positivity of order two or a scale mixture
							thereof for its validity (Sarkar, 1998). 
							Both procedures use the same set of critical values <i>c(i)=&alpha;/(m-i+1)</i>. Whereas Holm's procedure is a step-down 
							version of the Bonferroni test, and Hochberg's is a step-up version of the Bonferroni test.
							Note that Holm's method is based on the Bonferroni inequality and is valid regardless of the joint
							distribution of the test statistics.</p>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }


adaptiveBH <- function(pValues, alpha, silent=FALSE) {
	m <- length(pValues)
	pi0.ABH <- ABH_pi0_est(pValues)$pi0
	criticalValues <- sapply(1:m, function(i) (i*alpha)/(m*pi0.ABH))
	adjPValues <- p.adjust(pValues,"BH")*pi0.ABH
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\t\tBenjamini-Hochberg (2000) adaptive step-up procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, criticalValues=criticalValues, rejected=rejected, pi0=pi0.ABH,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}
mutoss.adaptiveBH <- function() { return(new(Class="MutossMethod",
					label="Benjamini-Hochberg (2000) adaptive step-up",
					errorControl="FDR",
					callFunction="adaptiveBH",
					output=c("adjPValues", "criticalValues", "rejected", "pi0", "errorControl"),
					assumptions=c("independence or positive regression dependency"),
					info="<h2>Benjamini-Hochberg (2000) adaptive linear step-up procedure </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Benjamini, Y. and Hochberg, Y. (2000). <i> On the Adaptive Control of the False Discovery Rate
							in Multiple Testing With Independent Statistics. </i> Journal of Educational and Behavioral Statistics, 25(1): 60-83. </li>\n\
							<li>Benjamini, Y. and Hochberg, Y. (1995). <i> Controlling the false discovery rate: A practical and powerful approach to mulitple testing.
							</i>Journal of the Royal Statistical Society, Series B, 57:289-300. </li>\n\
							<li> Hochberg, Y. and Benjamini, Y. (1990). <i>More powerful procedures for multiple significance testing. </i>
 							Statistics in Medicine 9, 811-818.</li>\n
							</ul>
							<p>The adaptive Benjamini-Hochberg step-up procedure is applied to pValues.
								It controls the FDR at level alpha for independent or positive regression dependent test statistics.
								In the adaptive Benjamini-Hochberg step-up procedure the number of true null hypotheses is estimated first as in Hochberg and
								Benjamini (1990), and this estimate is used in the procedure of Benjamini and
								Hochberg (1995) with alpha'=alpha*m/m0. The method for estimating m<sub>0</sub> is motivated by 
							the graphical approach proposed by Schweder and Spjotvoll (1982), 
							as developed and presented in Hochberg and Benjamini (1990).</p>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }


#---------------------- Adaptive STS-----------------#

adaptiveSTS <- function(pValues, alpha, lambda=0.5, silent=FALSE) {
	m <- length(pValues)
	adjP <- p.adjust(pValues,"BH")
	pi0 <- storey_pi0_est(pValues, lambda)$pi0
	criticalValues <- sapply(1:m, function(i) (i*alpha)/(m*pi0))
	adjPValues <- adjP*min(pi0, 1)
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\t\tStorey-Taylor-Siegmund (2004) adaptive step-up procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, criticalValues=criticalValues, rejected=rejected, pi0=pi0,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}
mutoss.adaptiveSTS <- function() { return(new(Class="MutossMethod",
					label="Storey-Taylor-Siegmund (2004) adaptive step-up",
					errorControl="FDR",
					callFunction="adaptiveSTS",
					output=c("adjPValues", "criticalValues", "rejected", "pi0", "errorControl"),
					assumptions=c("test independence or positive regression dependency"),
					info="<h2>Storey-Taylor-Siegmund (2004) adaptive step-up procedure</h2>\n\n
							
							<p> The adaptive STS method uses a conservative estimate of <i>pi0</i> which is plugged in a linear step-up procedure. The estimation of <i>pi0</i> requires a parameter <i>&lambda;</i> which is set to <i>0.5</i> by default.
							Note that the estimated <i>pi0</i> is truncated at 1 as suggested by the author, so the implemetation of the procedure is not entirely supported by the proof in the reference. 
							</p>\n
							<h3>Reference:</h3>
							<ul>
							<li>Storey, J.D., Taylor, J.E. and Siegmund, D. \"<i> Strong control, conservative point estimation and
							simultaneous conservative consistency of false discovery rates: a unified approach.</i>\" Journal of the Royal Statistical Society, B 66(1):187-205, 2004. </li>\n
							</ul>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"), lambda=list(type="numeric", default = 0.5))
			)) }

#---------------------------- Sidack Step Down--------------------------------#
SidakSD <- function(pValues, alpha, silent=FALSE) {
	m <- length(pValues)
	criticalValues <- sapply(1:m, function(i) 1-(1-alpha)^(1/(m-i+1)))
	#rejected <- SD(pValues, criticalValues)
	tmp <- mt.rawp2adjp(pValues, "SidakSD")
	adjPValues <- tmp$adjp[order(tmp$index),2]
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\tSidak-like (1987) step-down procedure\n\n")
		printRejected(rejected, pValues, NULL)
	}
	return(list(adjPValues=adjPValues, criticalValues=criticalValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
}
mutoss.SidakSD <- function() { return(new(Class="MutossMethod",
					label="Sidak-like (1987) step-down",
					errorControl="FWER",
					callFunction="SidakSD",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("test independence","positive orthant dependent test statistics"),
					info="<h2>Sidak-like (1987) step-down procedure </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Hollander, B.S. and Covenhaver, M.D. \"<i> An Improved Sequentially Rejective Bonferroni Test Procedure.</i>\" Biometrics, 43(2):417-423, 1987. </li>\n\
							</ul>
							<p> The Sidak-like step-down procedure is an improvement over Holm's (1979) step-down procedure. The improvement is analogous to the Sidak's correction over the original Bonferroni procedure. This Sidak-like step-down procedure assumes positive orthant dependent test statistics. </p>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }

#----------------------------------Blanchard Roquain 2008 ----------------------#

indepBR <- function(pValues, alpha, lambda=1, silent = FALSE) 
{	
	if ( lambda <= 0 || lambda >= 1/alpha) {
		stop('indepBR() : lambda should belong to (0, 1/alpha)')
	}
	
	len <- length(pValues)
	
	criticalValues <- sapply( 1:len, function(i) alpha * min( i * ( 1 - lambda * alpha) / (len - i + 1) , lambda ) )
	rejected <- SU(pValues, criticalValues)
	
	if (! silent)
	{
		cat("\n\n\t\t Blanchard-Roquain 1-stage step-up under independence (2009)\n\n")
		printRejected(rejected, pValues)
	}
	
	return(list(rejected = rejected, criticalValues = criticalValues,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}
mutoss.indepBR <- function() { return(new(Class="MutossMethod",
					label="Blanchard-Roquain adaptive step-up (2009)",
					errorControl="FDR",
					callFunction="indepBR",
					output=c("criticalValues", "rejected", "errorControl"),
					assumptions=c("p-value independence"),
					info="<h2> Blanchard-Roquain (2009) 1-stage adaptive step-up </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Blanchard, G. and Roquain, E. \"<i> Adaptive False Discovery Rate Control under Independence and Dependence.</i>\" 
							Journal of Machine Learning Research 10:2837-2871, 2009. . </li>\n\
							</ul>
							<p>This is a step-up procedure with critical values\n\ 
						 C<sub>i</sub> = alpha * min( i * ( 1 - lambda * alpha) / (m - i + 1) , lambda )\n\
						 where alpha is the level at which FDR should be controlled and lambda an \ 
							arbitrary parameter belonging to (0, 1/alpha) with default value 1. \
					This procedure controls FDR at the desired level when the p-values are independent.</p>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"), lambda=list(type="numeric", default=1))
			)) }


#----------------------------Blanchard Roquain 2009---------------------#
twostageBR <- function(pValues, alpha, lambda=1, silent = FALSE) 
{	
	if ( lambda <= 0 || lambda >= 1/alpha) {
			stop('twostageBR() : lambda should belong to (0, 1/alpha)')
		}
	
	m <- length(pValues)
	stage1 <- indepBR( pValues, alpha, lambda, silent = TRUE)
	pi0inv <- ( 1 - lambda*alpha )*m / ( m + 1 - sum(stage1$rejected) )
	
	BHadjPValues <- p.adjust(pValues,"BH")
	
	rejected <- ( BHadjPValues <= alpha*pi0inv )
	if (! silent)
	{
		cat("\n\n\t\tBlanchard-Roquain (2009) 2-stage step-up Procedure\n\n")
		printRejected(rejected, pValues)
	}
	return(list(rejected=rejected, errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
	
}
mutoss.twostageBR <- function() { return(new(Class="MutossMethod",
					label="Blanchard-Roquain 2-stage adaptive step-up (2009)",
					errorControl="FDR",
					callFunction="twostageBR",
					output=c("rejected", "errorControl"),
					assumptions=c("p-value independence"),
					info="<h2> Blanchard-Roquain 2-stage step-up under independence </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Blanchard, G. and Roquain, E. \"<i> Adaptive False Discovery Rate Control under Independence and Dependence.</i>\" 
							Journal of Machine Learning Research 10:2837-2871, 2009. . </li>\n\
							</ul>\
							<p>This is an adaptive linear step-up procedure where the proportion of true\
							 nulls is estimated using the Blanchard-Roquain 1-stage procedure with parameter lambda,\
                             via the formula\n\
 
				 estimated pi<sub>0</sub> = ( m - R(alpha,lambda)  + 1) / ( m*( 1 - lambda * alpha ) )\n\
 
				 where R(alpha,lambda) is the number of hypotheses rejected by the BR 1-stage procedure,
				 alpha is the level at which FDR should be controlled and lambda an 
 				 arbitrary parameter belonging to (0, 1/alpha) with default value 1.
 				 This procedure controls FDR at the desired level when the p-values are independent.</p>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"), lambda=list(type="numeric",default=1))
			)) }

# 
# 
# Author: FrankKonietschke
###############################################################################


ranktruncated <- function(pValues, K, silent = FALSE){
	
	L <- length(pValues)
	
	if (K > L){
		warn1 <- paste("K must be smaller than L")
		stop(warn1)
	}
	
#-----Compute the test statistic-----#
	
	index <- order(pValues)
	rindex <- order(index)
	spval <- pValues[index]
	
	w <- prod(spval[1:K])
	
	
#---Compute the used pvalues--------#
	
	w.pvalues <- pValues[rindex]
	
	p.used <- data.frame(Position=index[1:K], pValue=spval[1:K])
	
	
	
	
	
#--Compute the function awt as in the paper-----#

awt <- function(w,t,K){
	if (w<=t^K){
		s <- c(0:(K-1))
		num1 <- K*log(t)-log(w)
		num2 <- w * sum(num1^s/factorial(s))
	}
	if (w > t^K) {
		num2<- t^K
	}
	return(num2)
}

# ----- Compute now the exact distribution-----#
	
	fac1 <- choose(L, K+1)*(K+1)
	t <- seq(0.001,0.999,0.001)
	terg <- c()
	
	for (i in 1:length(t)){
		terg[i] <- (1-t[i])^(L-K-1)*awt(w,t[i],K)
}

#--------------Compute the p-Value -------------#
distribution <- fac1*mean(terg)

#-Regarding the error of numerical integration-#
	
p1 <- (distribution>1)
distribution[p1] <- 1

#--------------Prepare the output---------------#

	if (! silent)
	{
		cat("#----Rank Truncated Product of P-Values (Dubridge and Koeleman; 2003)   \n\n")
			
	}
result <- data.frame(Statistic = w, p.Value=distribution)
return(list(Used.pValue=p.used, RTP=result))
}

setClass("ErrorControl",
		representation  = representation(
				type           = "character", # Type of error rate controlled for (FWER, FWER.weak, FDR, FDX, gFWER, perComparison (?))
				alpha          = "numeric",   # Error rate of the procedure
				k              = "numeric",   # Additional parameter for  generalized FWE control
				q              = "numeric"    # Additional paramter for FDX control
		)
)

##TODO: add slots for "model": formula, link, family
##TODO: write some header, it is the main object that comes out after four weeks of hard work!!

setClass("Mutoss",
		representation    = representation(
				data			= "ANY",            # Raw data used in model
				model			= "ANY",            # link function,error family and design 
				description     = "character",
				statistic       = "numeric",      	# For Z, T or F statistics (maybe different slots?)
				hypotheses		= "ANY",
				hypNames        = "character",    	# Identifiers for the hypotheses tested
				criticalValues	= "numeric",	    # Procedure-specific critical values
				pValues         = "numeric",      	# Raw p-values. Either imported or calculated with data and model 
				adjPValues      = "numeric",      	# Procedure-specific adjusted p-values
				errorControl    = "ErrorControl", 	# Details of the multiplicity control procedure used.				  
				rejected        = "logical",      	# Logical vector of the output of a procedure at a given error rate
				qValues         = "numeric",      	# Storey's estimates of the supremum of the pFDR
				locFDR          = "numeric",      	# Efron's local fdr estimates (by which method?)
				pi0             = "numeric",      	# Estimate of the proportion of null hypotheses (by which method?)
				confIntervals   = "matrix",       	# Confidence intervals for selected parameters (of which kind? selected how?)
				commandHistory  = "character"
		)
)

############################################
# mutoss.plotCI

mutoss.plotCI<-function(mat){
	diff<-max(mat[,1])-min(mat[,1])
	if(any(is.na(mat[,2])))	mat[,2]<-min(mat[,1])-diff*2
	if(any(is.na(mat[,3])))	mat[,3]<-max(mat[,1])+diff*2
	k<-nrow(mat)
	plotCI(1:k,mat[,1],abs(mat[,3]-mat[,1]),abs(mat[,2]-mat[,1]),lwd=2,col="red",scol="blue",
			main="CI plot",xaxt="n",xlab="Parameters",
			ylab="Values")
	axis(1, at=c(1:k), labels=rownames(mat)) 
}

requireLibrary <- function(package) {
	if(!require(package, character.only=TRUE)) {
		answer <- readline(paste("Package ",package," is required - should we install it?",sep=""))
		if (substr(answer, 1, 1) %in% c("y","Y")) {
			install.packages(package)
			require(package, character.only=TRUE)
		} else {
			stop(paste("Required package",package,"should not be installed"))
		}
	}
}

mutoss.apply <- function(mutossObj, f, label = deparse(substitute(f)), recordHistory = TRUE, ...) {
  
  
	params <- list()	
	for (param in names(formals(f))) {					#runs over all parameters of f
		if (param %in% slotNames(mutossObj)) {			#checks if parameter name corresponds to a mutoss slot	
			paramTail <- list(slot(mutossObj, param))	#extracts values from appropriate slot
			names(paramTail) <- param 					#attaches names for parameter values
			params <- c(params, paramTail)				#concatenates parameters with and without data from objects
		}
	}	
	result <- eval(as.call(c(f,params,...)))			#evaluates function call with extracted parameters	
	for (param in names(result)) {						
		if (param %in% slotNames(mutossObj)) {
                        value <- result[param][[1]]								
			attr(value, "method.name") <- label			#attaches attributes to 
			slot(mutossObj, param) <- value				#writes result in corresponding mutoss slots
		}
	}

        if ( recordHistory ) {
        
        mutossObj@commandHistory = c( mutossObj@commandHistory,
          paste(format( match.call( call = sys.call( sys.parent(1) ))), collapse='')) #write command into history
      }

	return(mutossObj)
}

hommel <- function(pValues, alpha,silent=FALSE) {
	m <- length(pValues)
	adjPValues <- p.adjust(pValues, "hommel")
	rejected <- adjPValues<=alpha
	if (! silent)
	{
		cat("\n\n\t\tHommel's (1988) step-up Procedure\n\n")
		printRejected(rejected, pValues, NULL)
	}
	return(list(adjPValues=adjPValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))
}
mutoss.hommel <- function() { return(new(Class="MutossMethod",
					label="Hommel (1988) adjustment",
					errorControl="FWER",
					callFunction="hommel",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("any dependency structure"),
					info="<h2>Hommel (1988) adjustment </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Hommel, Gerhard. \"<i> A stagewise rejective multiple test procedure based on a modified Bonferroni test. </i>\" Biometrika 75, pp. 383-386, 1988. </li>\n\
							</ul>
							<p>The method is applied to pValues. It controls \
							the FWER in the strong sense when the hypothesis tests are independent \
							or when they are non-negatively associated. \
							The method base upon the closure principle to assure the FWER alpha and \ 
							the critical Values of this procedure are given by alpha/n, \
							alpha/(n-1), ..., alpha/2, alpha/1.</p>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }

oracleBH <- function(pValues, alpha, pi0, silent=FALSE) {
	m <- length(pValues)
	adjPValues <- p.adjust(pValues,"BH")*pi0
	rejected <- (adjPValues <= alpha)
	criticalValues <- sapply(1:m, function(i) (i*alpha)/(pi0*m))
	if (! silent)
	{
		cat("\n\n\t\tBenjamini-Hochberg's (1995) oracle linear-step-up Procedure\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, criticalValues=criticalValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}
mutoss.oracleBH <- function() { return(new(Class="MutossMethod",
					label="Benjamini-Hochberg (1995) oracle linear-step-up",
					errorControl="FDR",
					callFunction="oracleBH",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("test independence or positive regression dependency"),
					info="<h2>Benjamini-Hochberg (1995) oracle linear step-up Procedure </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Bejamini, Yoav and Hochberg, Josef. \"<i> Controlling the false discovery rate: a practical and powerful approach to multiple testing.
							</i>\" J. Roy. Statist. Soc. Ser. B 57 289-300, 1995. </li>\n\
							</ul>
							<p>Knowledge of the number of true null hypotheses (m0) can be very useful to improve upon the performance of the FDR controlling procedure. \
							For the oracle linear step-up procedure we assume that m0 were given to us by an `oracle', the linear step-up procedure with q0 = q*m/m0 \
							would control the FDR at precisely the desired level q in the independent and continuous case, and \
							would then be more powerful in rejecting hypotheses for which the alternative holds.</p>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"), pi0=list(type="numeric"))
			)) }

Qvalue <- function(pValues, lambda=seq(0,.90,.05), pi0.method="smoother", fdr.level=NULL, robust=FALSE, smooth.df=3, smooth.log.pi0=FALSE, silent=FALSE) {
	requireLibrary("qvalue")
  qvalue <- get("qvalue", envir=asNamespace("qvalue"))
  out <- qvalue(pValues, lambda=lambda, pi0.method=pi0.method, fdr.level=fdr.level, robust=robust, smooth.df=smooth.df, smooth.log.pi0=smooth.log.pi0)
	qValues<-out$qvalues
	pi0<-out$pi0
	if (! silent)
	{
		cat("\n\n\t\tStorey's (2001) q-value Procedure\n\n")
		cat("Number of hyp.:\t", length(pValues), "\n")
		cat("Estimate of the prop. of null hypotheses:\t", pi0, "\n")
	}
	return(list(qValues=qValues, pi0=pi0, errorControl = new(Class='ErrorControl', type="pFDR")))
}
mutoss.Qvalue <- function() { return(new(Class="MutossMethod",
					label="Storey's (2001) q-value Procedure",
					errorControl="pFDR",
					callFunction="Qvalue",
					output=c("qValues", "pi0", "errorControl"),
					info="<h2>Storey (2001) qvalue Procedure </h2>\n\n\
							<h3>Reference:</h3>\
							<ul>\
							<li>Storey, John \"<i> The Positive False Discovery Rate: A Baysian Interpretation and the Q-Value.
							</i>\" The Annals of Statistics 2001, Vol. 31, No. 6, 2013-2035, 2001. </li>\n\
							</ul>
							<p>The Qvalue procedure estimates the q-values for a given set of p-values. The q-value of a test measures the \
							proportion of false positive incurred when that particular test is called sigificant. \
							It gives the scientist a hypothesis testing error measure for each observed statistic with respect to the pFDR. \
							Note: If no options are selected, then the method used to estimate pi0 is the smoother method desribed in Storey and Tibshirani (2003). \
							The bootstrap method is described in Storey, Taylor and Siegmund (2004). </p>\n",
					parameters=list(pValues=list(type="numeric"), lambda=list(type="numeric",optional=TRUE,label="Tuning parameter lambda"), pi0.method=list(type="character",optional=TRUE,choices=c("smoother","bootstrap"),label="Tuning parameter for the estimation of pi_0"),
							fdr.level=list(type="numeric",optional=TRUE,label="Level at which to control the FDR"),robust=list(type="logical",optional=TRUE,label="Robust estimate"),
							smooth.df=list(type="integer",optional=TRUE,label="Number of degrees-of-freedom"),smooth.log.pi0=list(type="logical",optional=TRUE))
			)) }






BlaRoq<-function(pValues, alpha, pii, silent=FALSE){
	k <- length(pValues)
	
	if (missing(pii)) {
		pii = sapply( 1:k, function(i) exp(-i/(0.15*k)) ) # a default choice different from BY, exponential decreasing prior
	}
	
	if (any(pii<0)) {
		stop("BlaRoq(): Prior pii can only have positive elements")
	}
	
	if ( length(pii) != k) {
		stop("BlaRoq(): Prior pii must have the same length as pValues")
	}
	
	pii <- pii / sum(pii)
	
	precriticalValues <- cumsum(sapply(1:k, function(i) (i*pii[i])/k))
	# The following code is inspired from  p.adjust	
	i <- k:1
	o <- order(pValues, decreasing = TRUE)
	ro <- order(o)
	adjPValues <- pmin(1, cummin( pValues[o] / precriticalValues[i] ))[ro]
	
	rejected <- (adjPValues <= alpha)
	if (! silent)
	{
		cat("\n\n\t\t Blanchard-Roquain/Sarkar (2008) step-up for arbitrary dependence\n\n")
		printRejected(rejected, pValues, adjPValues)
	}
	return(list(adjPValues=adjPValues, criticalValues=alpha*precriticalValues, rejected=rejected,
					errorControl = new(Class='ErrorControl',type="FDR",alpha=alpha)))
}


mutoss.BlaRoq <- function() { return(new(Class="MutossMethod",
					label="Blanchard-Roquain/Sarkar (2008) step-up",
					errorControl="FDR",
					callFunction="BlaRoq",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					info="<h2>Blanchard-Roquain (2008) step-up Procedure for arbitrary dependent p-Values.</h2>\n\
							(Also proposed independently by Sarkar (2008))\n\
							<h3>References:</h3>\
							<ul>\
							<li>Blanchard, G. and Roquain, E. (2008).\"<i> Two simple sufficient conditions for FDR control.\
							</i>\" Electronic Journal of Statistics, 2:963-992. </li>\n\
							<li>Sarkar, S. K. (2008) \"<i>On methods controlling the false discovery rate.</i>\"\ 
							Sankhya, Series A, 70:135-168</li>\n\
							</ul>\
							<p>A generalization of the Benjamini-Yekutieli procedure, taking as an additional parameter
							a distribution pi on [1..k] (k is the number of hypotheses)
							representing prior belief on the number of hypotheses that will be rejected.</p>
							<p>The procedure is a step-up with critical values C_i defined as alpha/k times
							the sum for j in [1..i] of j*pi[j]. For any fixed prior pii, the FDR is controlled at
							level alpha for arbitrary dependence structure of the p-Values. The particular case of the
							Benjamini-Yekutieli step-up is recovered by taking pii[i] proportional to 1/i.
							If pii is missing, a default prior distribution proportional to exp( -i/(0.15*k) ) is taken.
							It should perform better than the BY procedure if more than about 5% to 10% of hypotheses are rejected,
                            and worse otherwise.
							</p>
							
							<p>Note: the procedure automatically normalizes the prior pii to sum to one if this is not the case.
							</p>\n",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"), pii=list(type="RObject",label="Prior pi",optional=TRUE))
			)) }

# Simultaneous confidence intervals for linear contrasts
# 
# Author: FrankKonietschke
###############################################################################







multcomp.wrapper <- function(model, hypotheses, alternative, rhs=0, alpha, factorC ) {
	
	type<-""
	
	if (any(factorC== c("Tukey", "Dunnett", "Sequen",
			"Williams", "Changepoint", "AVE", "McDermott", "Marcus", "UmbrellaWilliams"))) {
		eval(parse(text=paste("type <- mcp(",factorC,"=hypotheses)")))
	} else {		
		eval(parse(text=paste("type <- mcp(",factorC,"=hypotheses)")))
	}	

	glhtObj <- glht(model, linfct = type, rhs=rhs, alternative=alternative)
	summaryGLHT <- summary(glhtObj)
	pvalues <- summaryGLHT$test$pvalues
	estimates <- summaryGLHT$test$coefficients
	confint <- confint(glhtObj,level=(1-alpha))$confint
	
	
	
	rejected1 <- (pvalues < alpha)
	confi <- cbind(confint)
	print(cbind(confi,pvalues))
	return(list(adjPValues=pvalues,rejected=rejected1,confIntervals= confi,
					errorControl = new(Class='ErrorControl',type="FWER",alpha=alpha)))

	

}


mutoss.multcomp<- function() { return(new(Class="MutossMethod",
					label="Multiple Contrast Tests",
					errorControl="FWER",
					callFunction="multcomp.wrapper",
					output=c("adjPValues", "rejected","confIntervals","errorControl"),
					info="<h2>Parametric multiple contrast tests and simultaneous confidence intervals</h2>
							<p>With this function, it is possible to compute simultaneous tests and confidence intervals for general linear hypotheses in parametric models<p> 
							<p></p>
							<h3>Reference:</h3>
							<ul>
							<li>Frank Bretz, Alan Genz and Ludwig A. Hothorn \"<i>On the numerical availability of multiple comparison procedures.</i>\" Biometrical Journal, 43(5), 645-656. , 2001.</li>
													</ul>",
					parameters=list(model=list(type="ANY"),
							hypotheses=list(type="ANY"),
							alpha=list(type="numeric"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							factorC=list(type="character", label="Factor for Comparison", fromR="FactorVar")
						
					)
			)) }

# 
# Author: JonathanRosenblatt
###############################################################################



#---------- Service Functions---------#

reject<- function(sorted, criticals){
	m<- length(sorted)
	stopifnot( length(criticals) == m )
	indicators<-  sorted<criticals # Marking p-values below the critical values
	
	if(!any(indicators)) {
		return(list(cutoff=0,cut.index=0))
	}
	
	cut.index<- max((1:m)[indicators])
	
	cutoff<- sorted[cut.index] #The highest rejected p-value
	
	return( list(cutoff=cutoff,cut.index=cut.index) )
}

bh.adjust<- function(sorted, m, m0, constant=1){
	adjusted<- rep(NA,m)
	temp.min<- sorted[m]
	min.ind<- rep(0,m)    
	for (i in m:1) {
		temp<- min(m0*sorted[i]*constant / i, 1)
		if  ( temp <= temp.min  ) {
			temp.min <- temp
			min.ind[i]<- 1
		}
		adjusted[i]<- temp.min
	}
	return(adjusted)
}




linearStepUp<- function(sorted, q, m, adjust=FALSE, m0=m, pi0, constant=1){
	if(missing(m0) & !missing(pi0)) {
		m0=pi0*m
	}
	else{
		
		criticals<- (1:m)*q/(m0*constant)
		cutoff<- reject(sorted,criticals)
		rejected<-  sorted<=cutoff$cutoff
		
		adjusted=rep(NA,m)
		
		if(adjust) {
			adjusted<-bh.adjust(sorted,m=m,m0=m0,constant=constant)
		}
		
		multiple.pvals<- data.frame(
				original.pvals=sorted,
				criticals=criticals,
				rejected=rejected,
				adjusted.pvals=adjusted)
		output<- list(Cutoff=cutoff,Pvals=multiple.pvals)
		return(output)
	}
	
}

#---------------Two Stage------------------#
solve.q<- function(sorted, m, j, r){
	##TODO: [JR] correct problem: when all pvalues are rejected a might be negative
	a<- sorted*(m-r)/(1:m)
	#stopifnot(a>0 , j>=1 , j<=m , r>=0 , r<=m) 
	adjusted<- ifelse(a>0.5, 1 , a/(1-a) )
	temp.min<- adjusted[m]
	for(i in m:j){
		if(adjusted[i]<=temp.min) temp.min<- adjusted[i]
		else adjusted[i]<- temp.min
	}
	return(adjusted)
}# Close solve.q function

two.stage.adjust<- function(sorted, r=0, patience=4, m){
	adjusted<- rep(0,m)  
	
	# Adjusting sorted p-values
	adjusted.q<- solve.q(sorted=sorted,m=m,j=1,r=0)
	checking<- adjusted.q
	
	#Has the procedure rejected everything at the first stage?
	if(sum(linearStepUp(sorted,adjusted.q[1]/(1+adjusted.q[1]),m=m)$Pvals[['rejected']])==m){
		adjusted.q<- rep(adjusted.q[1],m)
		return(adjusted.q)
	}
	
	else{
		for (j in 1:m) { 
			delta.r<- 1
			delta.q<- 1
			new.q<- adjusted.q[j]
			r.new<- sum(linearStepUp(sorted,new.q/(1+new.q),m=m)$Pvals[['rejected']])
			counter<- 0
			max.q<- 0
			
			while(delta.r>0 & delta.q>0){
				old.q<- new.q
				r.old<- r.new
				new.q<- solve.q(sorted=sorted,m=m,j=j,r=r.old)[j]
				r.new<- sum(linearStepUp(sorted,new.q/(1+new.q),m=m)$Pvals[['rejected']])
				delta.r<- abs(r.new-r.old)  
				delta.q<- abs(new.q-old.q)
				counter<- counter+1
				if(counter>patience & max.q!=new.q) max.q<- max(max.q,new.q)
				else if(counter>patience & max.q==new.q ) break
			} #Close interations inside q[j]
			
			adjusted.q[j]<- min(new.q,1)
			adjusted.q[min(j+1,m)]<- adjusted.q[j]
			stopifnot(any(adjusted.q[j]<=checking[j]))
		}#Close looping over j.    
		
		temp.min<- adjusted.q[m]
		for(i in m:1){
			if(adjusted.q[i]<=temp.min) temp.min<- adjusted.q[i]
			else adjusted.q[i]<- temp.min
		}
		return(adjusted.q)      
	}#Close 'else' clause
	
}# Close two.stage.adjust


two.stage<- function(pValues, alpha){
	ranks<- rank(pValues)
	sorted<-sort(pValues)
	m<- length(sorted)
	
	#Stage I- estimating m0
	q1<- alpha/(1+alpha) 
	stage.one<- linearStepUp(sorted, q1, adjust=TRUE, m=m)
	r<- sum(stage.one$Pvals[['rejected']]) #count rejection
	if (r==0) {  #if nothing is rejected, return the results of the linear step up
		stage.one$Pvals[['adjusted.pvals']]<- 1
		return(stage.one)  
	}
	else if (r==m) {
		stage.one$Pvals[['adjusted.pvals']]<- stage.one$Pvals[['adjusted.pvals']][1]
		return(stage.one)  
	}
	
	#Stage II- updating alpha using m0
	else {
		m0<- m-r
		output<- linearStepUp(sorted=sorted,q=q1,m0=m0,m=m)
		output$Pvals[['adjusted.pvals']]<- two.stage.adjust(sorted, alpha, m=m)
		output<-output$Pvals[ranks,]
		output.2<- list(
				criticalValues=output$criticals,
				rejected=output$rejected,
				adjPValues=output$adjusted.pvals,
				errorControl=new(Class='ErrorControl',type="FDR",alpha=alpha),
				pi0= m0/m
		)
		return(output.2)
	}
}

#pvals<- runif(100,0,0.1)
#two.stage(pvals,0.1)

mutoss.two.stage<- function() { return(new(Class="MutossMethod",
					label="B.K.Y. (2006) Two-Stage Step-Up",
					errorControl="FDR",
					callFunction="two.stage",
					output=c("adjPValues", "criticalValues", "rejected", "pi0", "errorControl"),
					assumptions=c("Independent test statistics"),
					info="<h2>Benjamini-Krieger-Yekutieli (2006) Two-Stage Step-Up Procedure</h2>\n\n							
							<p>A p-value procedure which controls the FDR at level <i>&alpha;</i> for independent test statistics, in which case it is more powerful then non adaptive procedures such as the Linear Step-Up (BH). On the other hand, when this is not the case, no error control is guaranteed.
							The linear step-up procedure is used in he first stage to estimate the number of true null hypotheses (mo) which is plugged in a linear step-up
							procedure at the second stage. 
							<h3>Reference:</h3>
							<ul>
							<li>Benjamini, Y., Krieger, A. and Yekutieli, D. \"<i> Adaptive linear step-up procedures that control the false
							discovery rate. </i>\" Biometrika, 93(3):491-507, 2006. </li>\n
							</ul>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }

#---------------------Multistage Step-Down-------------------#

multiple.down.adjust<- function(sorted, m){
	adjusted<- rep(NA,m)
	temp.max<- sorted[1]
	max.ind<- rep(0,m)
	
	for (i in 1:m) {
		temp<-  min(sorted[i]*(m+1-i)/(i*(1-sorted[i])),1)
		if  ( temp >= temp.max  ) {
			temp.max <- temp
			max.ind[i] <- 1
		}
		adjusted[i]<- temp.max
	}
	
	return(adjusted)
}





multiple.down=function(pValues, alpha){
	sorted<- sort(pValues)
	ranks<- rank(pValues)
	m<- length(pValues)
	
	if(alpha>0.5) warning('FDR is not controlled when q>0.5')
	
	criticals<- sapply(1:m,function(i) alpha*i/(m-i*(1-alpha)+1))
	indicators<-  sorted<criticals # Marking p-values below the critical values
	
	if(!indicators[1]) cutoff<-list(cutoff=0,cut.index=0)
	else if(all(indicators)) cutoff<- list(cutoff=sorted[m],cut.index=m)
	else{ 
		cut.index<- min((1:m)[!indicators])-1
		cutoff<- list(cutoff=sorted[cut.index],cut.index=cut.index) 
	}
	
	rejected<-  sorted<=cutoff$cutoff
	
	adjusted<-multiple.down.adjust(sorted,m)  
	
	output<- data.frame(
			criticals=criticals,
			rejected=rejected,
			adjusted.pvals=adjusted)
	output<- output[ranks,]
	output.2<-list(
			criticalValues=output$criticals,
			rejected=output$rejected,
			adjPValues=output$adjusted.pvals,
			errorControl=new(Class='ErrorControl',type="FDR",alpha=alpha)
	)
	return(output.2)	
}

mutoss.multiple.down <- function() { return(new(Class="MutossMethod",
					label="B.K.Y. (2006) Multi-Stage Step-down",
					errorControl="FDR",
					callFunction="multiple.down",
					output=c("adjPValues", "criticalValues", "rejected", "errorControl"),
					assumptions=c("Independent test statistics"),
					info="<h2>Benjamini-Krieger-Yekutieli (2006) multi-stage step-down procedure</h2>\n\n\
							<p>A non-linear step-down p-value procedure which control the FDR for independent test statistics and enjoys more power then other non-adaptive procedure such as the linear step-up (BH).
							For the case of non-independent test statistics, non-adaptive procedures such as the linear step-up (BH) or the all-purpose conservative Benjamini-Yekutieli (2001) are recommended.</p>\n
							<h3>Reference:</h3>
							<ul>
							<li>Benjamini, Y., Krieger, A. and Yekutieli, D. \"<i> Adaptive linear step-up procedures that control the false
							discovery rate. </i>\" Biometrika, 93(3):491-507, 2006. </li>\n
							</ul>",
					parameters=list(pValues=list(type="numeric"), alpha=list(type="numeric"))
			)) }

# TODO: Add comment
# 
# Author: wiebke
###############################################################################


#### one sample model ####

onesamp.model <- function() {
	return(list(model=list(typ="onesamp")))
}

mutoss.onesamp.model <- function() { return(new(Class="MutossMethod",
					label="One-sample test",
					callFunction="onesamp.model",
					output=c("model"),
					info="<h2>One sample model</h2>
							<p>The input for this one sample model is a data matrix whose columns represent the samples and the rows represent the multiple endpoints. 
							E.g. for genomics this would be a gene matrix, where each row gives the expression for a single gene.</p> 
							<p> For the next steps you have the following choices:</p>
							<p> Either marginal hypotheses tests (if robust Wilcoxon otherwise t-test) could be performed on each row of the 
							data matrix to obtain raw p-values which then need to be adjusted for multiplicity to control a chosen error rate.</p>
							<p> Or resampling based methods could be performed based on Dudoit and van der Laan (2007) to obtain adjusted p-values which control the 
							FWER. Afterwards it is possible to use augmentation procedures to get adjusted p-values for control of FDR, FDX or gFWER.</p> 
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>
							</ul>",
					parameters=list(
					)
			)) }

onesamp.marginal <- function(data, robust, alternative, psi0) {
	result <- NULL
	if (robust) {
		result <- apply(data, 1, function(x) {wilcox.test(x, alternative=alternative, mu=psi0)$p.value} )
	} else {
		result <- apply(data, 1, function(x) {t.test(x ,alternative=alternative, mu=psi0)$p.value} )
	}
	return(list(pValues=result))
}

mutoss.onesamp.marginal.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "onesamp")
}


mutoss.onesamp.marginal <- function() { return(new(Class="MutossMethod",
					label="One-sample test",
					callFunction="onesamp.marginal",
					output=c("pValues"),
					info="<h2>Marginal one sample test.</h2>
							<p>The robust version uses the Wilcoxon-Mann-Whitney test, otherwise a t-test will be performed.</p> 
							<h3>Reference:</h3>
							<ul>
							<li>Wilcoxon, F. (1945). <i>Individual Comparisons by Ranking Methods.</i> Biometrics Bulletin 1:80-83.</li>\n\
							<li>Mann, H. and Whitney, D. (1947). <i>On a test of whether one of two random variables is stochastically larger 
							than the other.</i> Annals of Mathematical Statistics 18:50-60</li>\n\
							<li>Student (1908).<i>The probable error of a mean.</i> Biometrika, 6(1):1-25.</li>\n 
							</ul>",
					parameters=list(
							data=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							psi0=list(type="numeric", label="Hypothesized null value", default=0)
					)
			)) }



#### two sample model ####


twosamp.model <- function(classlabel) {
	classlabel <- as.vector(classlabel)
	return(list(model=list(typ="twosamp", classlabel=classlabel)))
}

mutoss.twosamp.model <- function() { return(new(Class="MutossMethod",
					label="Two sample test",
					callFunction="twosamp.model",
					output=c("model"),
					info="<h2>Two sample model</h2>
							<p>The input for this one sample model is a data matrix whose columns represent the samples and the rows represent the multiple endpoints. 
							E.g. for genomics this would be a gene matrix, where each row gives the expression for a single gene.</p> 
							<p> Furthermore, a classlabel needs to be provided to distinguish the two sample groups.
							<p> For the next steps you have the following choices:</p>
							<p> Either marginal hypotheses tests (if robust Wilcoxon otherwise t-test) could be performed on each row of the 
							data matrix to obtain raw p-values which then need to be adjusted for multiplicity to control a chosen error rate.</p>
							<p> Or resampling based methods could be performed based on Dudoit and van der Laan (2007) to obtain adjusted p-values which control the 
							FWER. Afterwards it is possible to use augmentation procedures to get adjusted p-values for control of FDR, FDX or gFWER.</p> 
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>
							</ul>",
					parameters=list(
							classlabel=list(type="RObject", label="classlabel")
					)
			)) }

twosamp.marginal <- function(data, model, robust, alternative, psi0, equalvar) {
	label <- as.numeric(as.factor(model$classlabel))
	result <- NULL
	if (robust) {
		result <- apply(data, 1, function(x) {wilcox.test(x=x[ ,label==1], y=x[label==2], alternative=alternative, mu=psi0)$p.value} )
	} else {
		result <- apply(data, 1, function(x) {t.test(x=x[ ,label==1], y=x[label==2], alternative=alternative, mu=psi0, equal.var=equalvar)$p.value} )
	}
	return(list(pValues=result))
}

mutoss.twosamp.marginal.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "twosamp")
}

mutoss.twosamp.marginal <- function() { return(new(Class="MutossMethod",
					label="Two sample test",
					callFunction="twosamp.marginal",
					output=c("pValues"),
					info="<h2></h2>
							<p>The robust version uses the Wilcoxon-Mann-Whitney test, otherwise a two-sample t-test will be performed.</p> 
							<h3>Reference:</h3>
							<ul>
							<li>Wilcoxon, F. (1945). <i>Individual Comparisons by Ranking Methods.</i> Biometrics Bulletin 1:80-83.</li>\n\
							<li>Mann, H. and Whitney, D. (1947). <i>On a test of whether one of two random variables is stochastically larger 
							than the other.</i> Annals of Mathematical Statistics 18:50-60</li>\n
							</ul>",
					parameters=list(
							data=list(type="ANY"),
							model=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							psi0=list(type="numeric", label="Hypothesized null value", default=0),
							equalvar=list(type="logical", label="Equal variance")
					)
			)) }


### paired sample model ###

paired.model <- function(classlabel) {
	classlabel <- as.vector(classlabel)
	return(list(model=list(typ="pairedsamp", classlabel=classlabel)))
}

mutoss.paired.model <- function() { return(new(Class="MutossMethod",
					label="Paired sample test",
					callFunction="paired.model",
					output=c("model"),
					info="<h2>Paired sample test</h2>
							<p>The robust version uses the Wilcoxon signed rank test, otherwise a paired t-test will be performed.</p> 
							<p>The input for this paired sample model is a data matrix whose columns represent the samples and the rows represent the multiple endpoints. 
							E.g. for genomics this would be a gene matrix, where each row gives the expression for a single gene.</p> 
							<p> Furthermore, a classlabel needs to be provided to distinguish the two paired groups. The arrangement of group indices does not matter, as long
							as the columns are arranged in the same corresponding order between groups. For example, if group 1 is code as 0 and group 2 is 
							coded as 1, for 3 pairs of data, it does not matter if the classlabel is coded as (0,0,0,1,1,1) or (1,1,1,0,0,0) or (0,1,0,1,0,1)
							or (1,0,1,0,1,0), the paired differences between groups will be calculated as group2 - group1. 
							</p>
							<p> For the next steps you have the following choices:</p>
							<p> Either marginal hypotheses tests (if robust Wilcoxon otherwise t-test) could be performed on each row of the 
							data matrix to obtain raw p-values which then need to be adjusted for multiplicity to control a chosen error rate.</p>
							<p> Or resampling based methods could be performed based on Dudoit and van der Laan (2007) to obtain adjusted p-values which control the 
							FWER. Afterwards it is possible to use augmentation procedures to get adjusted p-values for control of FDR, FDX or gFWER.</p> 
							
							
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>
							</ul>",
					parameters=list(
							classlabel=list(type="RObject", label="classlabel")
					)
			)) }

paired.marginal <- function(data, model, robust, alternative, psi0, equalvar) {
	label <- as.numeric(as.factor(model$classlabel))
	result <- NULL
	if (robust) {
		result <- apply(data, 1, function(x) {wilcox.test(x=x[label==1], y=x[label==2], alternative=alternative, mu=psi0, paired=TRUE, var.equal=equalvar)$p.value} )
	} else {
		result <- apply(data, 1, function(x) {t.test(x=x[label==1], y=x[label==2], alternative=alternative, mu=psi0, paired=TRUE, var.equal=equalvar)$p.value} )
	}
	return(list(pValues=result))
}

mutoss.paired.marginal <- function() { return(new(Class="MutossMethod",
					label="Paired sample test",
					callFunction="paired.marginal",
					output=c("pValues"),
					info="<h2></h2>
							<p>The robust version uses the Wilcoxon test, otherwise a paired t-test will be performed.</p> 
							<p>A vector of classlabels needs to be provided to distinguish the two paired groups. The arrangement of group indices does not matter, as long
							as the columns are arranged in the same corresponding order between groups. For example, if group 1 is code as 0 and group 2 is 
							coded as 1, for 3 pairs of data, it does not matter if the classlabel is coded as (0,0,0,1,1,1) or (1,1,1,0,0,0) or (0,1,0,1,0,1)
							or (1,0,1,0,1,0), the paired differences between groups will be calculated as group2 - group1. </p>
							<p>You could either choose a valid R object to load as classlabels or you could provide it manually by inserting e.g. c(0,1,0,1,0,1) or rep(c(0,1), each=5) or rep(c(0,1), 5). </p>",
					parameters=list(
							data=list(type="ANY"),
							model=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							psi0=list(type="numeric", label="Hypothesized null value", default=0),
							equalvar=list(type="logical", label="Equal variance")
					)
			)) }

mutoss.paired.marginal.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "pairedsamp")
}

### f test model ###


ftest.model <- function(classlabel) {
	classlabel <- as.vector(classlabel)
	return(list(model=list(typ="ftest", classlabel=classlabel)))
}

mutoss.ftest.model <- function() { return(new(Class="MutossMethod",
					label="F test",
					callFunction="ftest.model",
					output=c("model"),
					info="<h2>F test</h2>
							<p>The input for this F test model is a data matrix whose columns represent the samples and the rows represent the multiple endpoints. 
							E.g. for genomics this would be a gene matrix, where each row gives the expression for a single gene.</p> 
							<p> Furthermore, a classlabel needs to be provided to distinguish k sample groups.</p>
							<p> For the next steps you have the following choices:</p>
							<p> Either marginal hypotheses tests (if robust Kruskal-Wallis test, otherwise F-test) could be performed on each row of the 
							data matrix to obtain raw p-values which then need to be adjusted for multiplicity to control a chosen error rate.</p>
							<p> Or resampling based methods could be performed based on Dudoit and van der Laan (2007) to obtain adjusted p-values which control the 
							FWER. Afterwards it is possible to use augmentation procedures to get adjusted p-values for control of FDR, FDX or gFWER.</p> 
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>
							</ul>",
					parameters=list(
							classlabel=list(type="RObject", label="classlabel")
					)
			)) }

ftest.marginal <- function(data, model, robust) {
	label <- as.numeric(as.factor(model$classlabel))
	result <- NULL
	if (robust) {
		result <- apply(data, 1, function(x) {kruskal.test(x=x, g=label)$p.value} )
	} else {
		result <- apply(data, 1, function(x) { out=x
					anova(lm( out ~ label ))$'Pr(>F)'[1]} )	
	}	
	return(list(pValues=result))
}

mutoss.ftest.marginal <- function() { return(new(Class="MutossMethod",
					label="F test",
					callFunction="ftest.marginal",
					output=c("pValues"),
					info="<h2></h2>
							<p>Robust = Kruskal-Wallis test. Otherwise F-test.</p> 
							<p></p>
							<h3>Reference:</h3>
							<ul>
							<li>Kruskal, W.H. und Wallis, W.A. (1952). <i>Use of ranks in one-criterion variance analysis.</i> JASA, 47:583-621</li>
							</ul>",
					parameters=list(
							data=list(type="ANY"),
							model=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic")
					)
			)) }

mutoss.ftest.marginal.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "ftest")
}

#######################################################################
########### here are the multtest methods - marginal siblings can be found in marginal.R ######################


### one sample model with multtest ###

onesamp.multtest <- function(data, alternative, robust, psi0, alpha, nulldist, B=1000, method, seed=12345) {
	result <- MTP(X=data, W = NULL, Y = NULL, Z = NULL, Z.incl = NULL, Z.test = NULL, 
			na.rm = TRUE, test = "t.onesamp", robust = robust, 
			standardize = TRUE, alternative = alternative, psi0 = psi0, 
			typeone = "fwer", k = 0, q = 0.1, fdr.method = "restricted", 
			alpha = alpha, smooth.null = FALSE, nulldist = nulldist, 
			B = B, ic.quant.trans = FALSE, MVN.method = "mvrnorm", 
			penalty = 1e-06, method = method, get.cr = FALSE, get.cutoff = FALSE, 
			get.adjp = TRUE, keep.nulldist = FALSE, keep.rawdist = FALSE, 
			seed = seed, cluster = 1, type = NULL, dispatch = NULL, marg.null = NULL, 
			marg.par = NULL, keep.margpar = TRUE, ncp = NULL, perm.mat = NULL, 
			keep.index = FALSE, keep.label = FALSE)
	return(list(adjPValues=result@adjp, rejected=as.vector(result@reject)))
}

mutoss.onesamp.multtest.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "onesamp")
}

mutoss.onesamp.multtest <- function() { return(new(Class="MutossMethod",
					label="Resampling-based one sample test",
					errorControl="FWER",
					callFunction="onesamp.multtest",
					output=c("adjPValues", "rejected"),
					info="<h2>Resampling-based one sample test</h2>
							<p>There are different choices of resampling methods available for estimating the joint test statistics null distribution:\n\
							<ul>
							<li>\"boot.cs\": non-parametric bootstrap with centering and scaling</li>\n\
							<li>\"boot.ctr\": centered-only bootstrap distribution</li>\n\
							<li>\"boot.qt\": quantile transformed bootstrap distribution. the default marginal t-distribution with n-1 degree of freedom is used.</li>\n\
							<li>\"perm\": permutation distribution (refering to the Westfall and Young procedures)</li>\n\
							<li>\"ic\": under GUI construction (available at (library(multtest)) </li>\n
							</ul>
							</p> 
							<p>There are four adjustment methods to control the FWER:
							<ul>
							<li>\"sd.minP\": step-down common-quantile procedure based on the minima of unadjusted p-values</li>
							<li>\"sd.maxT\": step-down common-cut-off procedure based on the maxima of test statistics</li>
							<li>\"ss.minP\": single-step common-quantile procedure</li>
							<li>\"ss.maxT\": single-step common-cut-off procedure</li>
							</ul>
							</p>
							<p> The default number of bootstrap iterations (or number of permutations if resampling method is \"perm\") is 1000. This can be reduced to increase the speed
							of computations, at a cost to precision. However, it is recommended to use a large number of resampling iterations, e.g. 10,000. </p>
							<p> The robust version uses the Wilcoxon-Mann-Whitney test, otherwise a t-test will be performed.</p>
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>\n\
							<li>Westfall, P.H. and Young, S.S. (1993). <i>Resampling-Based Multiple Testing. Examples and Methods for p-value adjustment. </i>Wiley Series in Probability and Mathematical Statistics. </li>\n   
							</ul>",
					parameters=list(
							data=list(type="ANY"),
							model=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							psi0=list(type="numeric", label="Hypothesized null value", default=0),
							alpha=list(type="numeric"),
							nulldist=list(type="character", label="Resampling Method", choices=c("boot.cs", "boot.ctr", "boot.qt", "perm")),
							B=list(type="numeric", label="Number of Resampling Iterations", default=1000),
							method=list(type="character", label="Adjustment Method", choices=c("sd.minP","sd.maxT","ss.minP","ss.maxT"),
									seed=list(type="ANY", default=12345)
							)
					))) }


### paired sample model with multtest ###


paired.multtest <- function(data, model, alternative, robust, psi0, alpha, nulldist, B=1000, method, seed=12345) {
	result <- MTP(X=data, W = NULL, Y = model$classlabel, Z = NULL, Z.incl = NULL, Z.test = NULL, 
			na.rm = TRUE, test = "t.pair", robust = robust, 
			standardize = TRUE, alternative = alternative, psi0 = psi0, 
			typeone = "fwer", k = 0, q = 0.1, fdr.method = "restricted", 
			alpha = alpha, smooth.null = FALSE, nulldist = nulldist, 
			B = B, ic.quant.trans = FALSE, MVN.method = "mvrnorm", 
			penalty = 1e-06, method = method, get.cr = FALSE, get.cutoff = FALSE, 
			get.adjp = TRUE, keep.nulldist = FALSE, keep.rawdist = FALSE, 
			seed = seed, cluster = 1, type = NULL, dispatch = NULL, marg.null = NULL, 
			marg.par = NULL, keep.margpar = TRUE, ncp = NULL, perm.mat = NULL, 
			keep.index = FALSE, keep.label = FALSE)
	return(list(adjPValues=result@adjp, rejected=as.vector(result@reject)))
}

mutoss.paired.multtest.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "pairedsamp")
}

mutoss.paired.multtest <- function() { return(new(Class="MutossMethod",
					label="Resampling-based paired sample test",
					errorControl="FWER",
					callFunction="paired.multtest",
					output=c("adjPValues", "rejected"),
					info="<h2>Resampling-based paired test</h2>
							<p>There are different choices of resampling methods available for estimating the joint test statistics null distribution:\n\
							<ul>
							<li>\"boot.cs\": non-parametric bootstrap with centering and scaling</li>\n\
							<li>\"boot.ctr\": centered-only bootstrap distribution</li>\n\
							<li>\"boot.qt\": quantile transformed bootstrap distribution. the default marginal t-distribution with n-1 degree 
							of freedom is used, where n is the number of pairs.</li>\n\
							<li>\"perm\": permutation distribution (refering to the Westfall and Young procedures)</li>\n\
							<li>\"ic\": under GUI construction (available at (library(multtest)) </li>\n
							</ul>
							</p> 
							<p>There are four adjustment methods to control the FWER:
							<ul>
							<li>\"sd.minP\": step-down common-quantile procedure based on the minima of unadjusted p-values</li>
							<li>\"sd.maxT\": step-down common-cut-off procedure based on the maxima of test statistics</li>
							<li>\"ss.minP\": single-step common-quantile procedure</li>
							<li>\"ss.maxT\": single-step common-cut-off procedure</li>
							</ul>
							</p>
							<p> The default number of bootstrap iterations (or number of permutations if resampling method is \"perm\") is 1000. This can be reduced to increase the speed
							of computations, at a cost to precision. However, it is recommended to use a large number of resampling iterations, e.g. 10,000. </p>
							<p> The robust version uses the Wilcoxon-Mann-Whitney test, otherwise a t-test will be performed.</p>
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>\n\
							<li>Westfall, P.H. and Young, S.S. (1993). <i>Resampling-Based Multiple Testing. Examples and Methods for p-value adjustment. </i>Wiley Series in Probability and Mathematical Statistics. </li>\n   
							</ul>",
					parameters=list(
							data=list(type="ANY"),
							model=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							psi0=list(type="numeric", label="Hypothesized null value", default=0),
							alpha=list(type="numeric"),
							nulldist=list(type="character", label="Resampling Method", choices=c("boot.cs", "boot.ctr", "boot.qt", "perm")),
							B=list(type="numeric", label="Number of Resampling Iterations", default=1000),
							method=list(type="character", label="Adjustment Method", choices=c("sd.minP","sd.maxT","ss.minP","ss.maxT"),
									seed=list(type="ANY", default=12345))
					)
			)) }

			
### two sample model with multtest ####			
			
twosamp.multtest <- function(data, model, alternative, robust, psi0, equalvar, alpha, nulldist, B=1000, method, seed=12345) {
	if (equalvar) {
		result <- MTP(X=data, W = NULL, Y = model$classlabel, Z = NULL, Z.incl = NULL, Z.test = NULL, 
				na.rm = TRUE, test = "t.twosamp.equalvar", robust = robust, 
				standardize = TRUE, alternative = alternative, psi0 = psi0, 
				typeone = "fwer", k = 0, q = 0.1, fdr.method = "restricted", 
				alpha = alpha, smooth.null = FALSE, nulldist = nulldist, 
				B = B, ic.quant.trans = FALSE, MVN.method = "mvrnorm", 
				penalty = 1e-06, method = method, get.cr = FALSE, get.cutoff = FALSE, 
				get.adjp = TRUE, keep.nulldist = FALSE, keep.rawdist = FALSE, 
				seed = seed, cluster = 1, type = NULL, dispatch = NULL, marg.null = NULL, 
				marg.par = NULL, keep.margpar = TRUE, ncp = NULL, perm.mat = NULL, 
				keep.index = FALSE, keep.label = FALSE)
	}
	else {
		result <- MTP(X=data, W = NULL, Y = model$classlabel, Z = NULL, Z.incl = NULL, Z.test = NULL, 
				na.rm = TRUE, test = "t.twosamp.unequalvar", robust = robust, 
				standardize = TRUE, alternative = alternative, psi0 = psi0, 
				typeone = "fwer", k = 0, q = 0.1, fdr.method = "restricted", 
				alpha = alpha, smooth.null = FALSE, nulldist = nulldist, 
				B = B, ic.quant.trans = FALSE, MVN.method = "mvrnorm", 
				penalty = 1e-06, method = method, get.cr = FALSE, get.cutoff = FALSE, 
				get.adjp = TRUE, keep.nulldist = FALSE, keep.rawdist = FALSE, 
				seed = seed, cluster = 1, type = NULL, dispatch = NULL, marg.null = NULL, 
				marg.par = NULL, keep.margpar = TRUE, ncp = NULL, perm.mat = NULL, 
				keep.index = FALSE, keep.label = FALSE)
	}
	return(list(adjPValues=result@adjp, rejected=as.vector(result@reject)))
}

mutoss.twosamp.multtest.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "twosamp")
}

mutoss.twosamp.multtest <- function() { return(new(Class="MutossMethod",
					label="Resampling-based two sample test",
					errorControl="FWER",
					callFunction="twosamp.multtest",
					output=c("adjPValues", "rejected"),
					info="<h2>Resampling-based two sample test</h2>
							<p>There are different choices of resampling methods available for estimating the joint test statistics null distribution:\n\
							<ul>
							<li>\"boot.cs\": non-parametric bootstrap with centering and scaling</li>\n\
							<li>\"boot.ctr\": centered-only bootstrap distribution</li>\n\
							<li>\"boot.qt\": quantile transformed bootstrap distribution. the default marginal t-distribution with n-1 degree of freedom is used.</li>\n\
							<li>\"perm\": permutation distribution (refering to the Westfall and Young procedures)</li>\n\
							<li>\"ic\": under GUI construction (available at (library(multtest)) </li>\n
							</ul>
							</p> 
							<p>There are four adjustment methods to control the FWER:
							<ul>
							<li>\"sd.minP\": step-down common-quantile procedure based on the minima of unadjusted p-values</li>
							<li>\"sd.maxT\": step-down common-cut-off procedure based on the maxima of test statistics</li>
							<li>\"ss.minP\": single-step common-quantile procedure</li>
							<li>\"ss.maxT\": single-step common-cut-off procedure</li>
							</ul>
							</p>
							<p> The default number of bootstrap iterations (or number of permutations if resampling method is \"perm\") is 1000. This can be reduced to increase the speed
							of computations, at a cost to precision. However, it is recommended to use a large number of resampling iterations, e.g. 10,000. </p>
							<p> The robust version uses the Wilcoxon-Mann-Whitney test, otherwise a t-test will be performed.</p>
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>\n\
							<li>Westfall, P.H. and Young, S.S. (1993). <i>Resampling-Based Multiple Testing. Examples and Methods for p-value adjustment. </i>Wiley Series in Probability and Mathematical Statistics. </li>\n   
							</ul>",
					parameters=list(
							data=list(type="ANY"),
							model=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic"),
							alternative=list(type="character", label="Alternative", choices=c("two.sided", "less", "greater")),
							psi0=list(type="numeric", label="Hypothesized null value", default=0),
							equalvar=list(type="logical", label="Equal variance"),
							alpha=list(type="numeric"),
							nulldist=list(type="character", label="Resampling Method", choices=c("boot.cs", "boot.ctr", "boot.qt", "perm")),
							B=list(type="numeric", label="Number of Resampling Iterations", default=1000),
							method=list(type="character", label="Adjustment Method", choices=c("sd.minP","sd.maxT","ss.minP","ss.maxT"),
									seed=list(type="ANY", default=12345))
					)
			)) }



##################### F test #############################################
###########################################################################


ftest.multtest <- function(data, model, robust, alpha, nulldist, B=1000, method, seed=12345) {
	result <- MTP(X=data, W = NULL, Y = model$classlabel, Z = NULL, Z.incl = NULL, Z.test = NULL, 
			na.rm = TRUE, test = "f", robust = robust, 
			standardize = TRUE, typeone = "fwer", 
			alpha = alpha, smooth.null = FALSE, nulldist = nulldist, 
			B = B, ic.quant.trans = FALSE, MVN.method = "mvrnorm", 
			penalty = 1e-06, method = method, get.cr = FALSE, get.cutoff = FALSE, 
			get.adjp = TRUE, keep.nulldist = FALSE, keep.rawdist = FALSE, 
			seed = seed, cluster = 1, type = NULL, dispatch = NULL, marg.null = NULL, 
			marg.par = NULL, keep.margpar = TRUE, ncp = NULL, perm.mat = NULL, 
			keep.index = FALSE, keep.label = FALSE)
	return(list(adjPValues=result@adjp, rejected=as.vector(result@reject)))
}


mutoss.ftest.multtest <- function() { return(new(Class="MutossMethod",
					label="Resampling-based F test",
					errorControl="FWER",
					callFunction="ftest.multtest",
					output=c("adjPValues", "rejected"),
					info="<h2>Resampling-based F test</h2>
							<p>There are different choices of resampling methods available for estimating the joint test statistics null distribution:\n\
							<ul>
							<li>\"boot.cs\": non-parametric bootstrap with centering and scaling</li>\n\
							<li>\"boot.ctr\": centered-only bootstrap distribution</li>\n\
							<li>\"boot.qt\": quantile transformed bootstrap distribution. the default marginal F-distribution with df1=k-1, df2=n-k for k gruops is used.</li>\n\
							<li>\"perm\": permutation distribution (refering to the Westfall and Young procedures)</li>\n\
							<li>\"ic\": under GUI construction (available at (library(multtest)) </li>\n
							</ul>
							</p> 
							<p>There are four adjustment methods to control the FWER:
							<ul>
							<li>\"sd.minP\": step-down common-quantile procedure based on the minima of unadjusted p-values</li>
							<li>\"sd.maxT\": step-down common-cut-off procedure based on the maxima of test statistics</li>
							<li>\"ss.minP\": single-step common-quantile procedure</li>
							<li>\"ss.maxT\": single-step common-cut-off procedure</li>
							</ul>
							</p>
							<p> The default number of bootstrap iterations (or number of permutations if resampling method is \"perm\") is 1000. This can be reduced to increase the speed
							of computations, at a cost to precision. However, it is recommended to use a large number of resampling iterations, e.g. 10,000. </p>
							<p> The robust version uses the Kruskal-Wallis test, otherwise a F test will be performed.</p>
							
							<h3>Reference:</h3>
							<ul>
							<li>Dudoit, S. and van der Laan, M.J. (2007). <i>Mulitple Testing Procedures and Applications to Genomics.</i> Springer Series in Statistics.</li>\n\
							<li>Westfall, P.H. and Young, S.S. (1993). <i>Resampling-Based Multiple Testing. Examples and Methods for p-value adjustment. </i>Wiley Series in Probability and Mathematical Statistics. </li>\n   
							</ul>",
					parameters=list(
							data=list(type="ANY"),
							model=list(type="ANY"),
							robust=list(type="logical", label="Robust statistic"),
							alpha=list(type="numeric"),
							nulldist=list(type="character", label="Resampling Method", choices=c("boot.cs", "boot.ctr", "boot.qt", "perm")),
							B=list(type="numeric", label="Number of Resampling Iterations", default=1000),
							method=list(type="character", label="Adjustment Method", choices=c("sd.minP","sd.maxT","ss.minP","ss.maxT"),
									seed=list(type="ANY", default=12345)
							)))
	)}

mutoss.ftest.multtest.model <- function(model) {
	return("typ" %in% names(model) && model$typ == "ftest")
}

# Generates a list of lists. Every of these lists, denoted for now
# by L, can be evaluated as a function call by eval(as.call(L))
# listFunAndParameter MUST have the following form:
# list(
#		funName = "UnifRandom",					# Description/Label of the function to be used
#		fun		= runif,					# A real function, not only the name
#		n		= 1						# "n" is integer   
# )
# Example for the argument:
# Arguments of runif are: n, min, max.
# listFunAndParameter = list(funName="UnifRandomVariable", fun=runif, n=2, min=c(1:2), max=c(1.1, 2.1))
# sapply(generateFunctionStack(listFunAndParameter), function(fc) eval(as.call(fc)))
# is same as
# runif(2, 1, 1.1); runif(2, 2, 1.1); runif(2, 1, 2.1); runif(2, 2, 2.1)
# Of course the second and third call do not make sense.
generateFunctionStack <- function(listFunAndParameter) 
{
	# special case: the procedure does not need any further parameter
	if (length(listFunAndParameter) == 2) 
		return(list(noParameters=list(fun=listFunAndParameter$fun)))
	
	# listFunAndParameter[[3]] is the first real parameter. The first 2 are the function to
	# to be called and a description.
	outerPar <- 1:length(listFunAndParameter[[3]])
	
	# Actually I want to build the outerproduct of the parameters,
	# but instead of this I use index numbers indicating the position
	# of the used parameter. If  n=c("a", "b"), n0=c(1:5), alpha=c(0.1, 0.2)
	# then 2, 3, 2 stands for n="b", n0=3, alpha=0.2
	for(par in listFunAndParameter[-3:-1])
		outerPar <- outer(outerPar, 1:length(par), paste)
	
	# special case of only ONE parameter
	if (length(listFunAndParameter) == 3)
		outerPar <- outer(outerPar, "", paste)		
	
	# build now for every parameter constellation
	# a list that can be casted into a function call. 
	fcStack <- list()
	for (parIDX in outerPar)
	{				
		idx <- as.numeric(unlist(strsplit(parIDX, " ")))
		
		parameter <- list()
		for(i in 1:length(listFunAndParameter[-2:-1]))
			# listFunAndParameter[-2:-1][[i]] is the i-th parameter in the list.
			# from the 1st parameter we want the idx[1]-th entry from the 2nd parameter
			# we want the idx[2]-th entry and so on.
			parameter <- c(parameter, listFunAndParameter[-2:-1][[i]][idx[i]])
		
		stackPosName <- paste(listFunAndParameter$funName, parIDX)
		fcStack[[stackPosName]] <- c(listFunAndParameter$fun, parameter)
		names(fcStack[[stackPosName]]) <- names(listFunAndParameter[-1])
	}	
	return(fcStack)
}


gatherParameters <- function(simObject) 
{
	#+++++++++++++++++++	Subfunctions	+++++++++++++++++++++++
	# extract from resultVecotr ( = simObject$results ) all values of the
	# parameter with the name paraName.
	
	getParamWithName <- function(resultVector, paraName)
	{	
		unlist( 
				lapply(resultVector,
						function(mts)
						{
							val <- mts$parameters[[paraName]]
							if (is.null(val)) 
								return("")
							
							val				
						}
				)
		)
	}	
	#-------------------	Subfunctions	-----------------------
	
	# gathering all parameters used in the simObject$results
	parNames <- unique(unlist(lapply(simObject$results, function(obj) names(obj$parameters))))
	
	return(data.frame(sapply(parNames, function(pN) getParamWithName(simObject$results, pN))))
	
#	# calling data.frame(sapply(parNames, function(pN) getParamWithName(simObject$results, pN)))
#	# is not good, because numeric parameters can be converted to characters and then to factors
#	# and it is possible that the original order is lost. For example the order of c(64, 128)
#	# will be 128 < 64.

#	ret = data.frame(getParamWithName(simObject$results, parNames[1]))
#
#	for (pN in parNames[2:length(parNames)]) # {"funName", "method"} subset of parNames; that is length(parName) >= 2 
#		ret = data.frame(ret, factor(getParamWithName(simObject$results, pN)))
#	
#	names(ret) <- parNames	
#	return(ret)
}


gatherStatistics <- function(simObject, listOfStatisticFunctions, listOfAvgFunctions) 
{
	#+++++++++++++++++++++++++++	Subfunctions	++++++++++++++++++++++++++
	# calculates the intersection of all elements given in aList
	listIntersect <- function(aList)
	{
		nn <- length(aList)		
		if(nn == 1)
			return(aList[[1]])
		
		intersect(aList[[1]], listIntersect(aList[-1]))
	}
	# actually the whole work is done by this subfunction.
	gatherStatisticsOneAvgFun <- function(simObject, listOfStatisticFunctions, avgFun, avgFunName = deparse(substitute(avgFun)))
	{
		# extract the parameter constellations form the obejct returned by simulation()
		paraNameDF <- gatherParameters(simObject)
		unqParaNameDF <- unique(paraNameDF)
		rownames(unqParaNameDF) <- 1:length(rownames(unqParaNameDF))
		
		# this will be the data.frame containing the parameter constellation and the calculated (averaged) statistics
		statDF <- data.frame()
		data.set.numbers <- sapply(simObject$results, function(res) res$data.set.number)
		for (i in rownames(unqParaNameDF))
		{
			# search which objects in simObject$results belong to 
			# parameter configuration in unqParaNameDF[i, ]
			idxs <- listIntersect(
					lapply(names(paraNameDF), 
							function(pN) which(unqParaNameDF[i, pN] == paraNameDF[ , pN])
					)
			)
			
			# applying any given statistic to the objects with the same 
			# parameter constellation			
			if (missing(avgFun))
			{ # no avgFun, thus the resulting data.frame will have one row for every simObject$results
				tmp <- sapply(listOfStatisticFunctions,	
						function(fun) sapply(idxs, function(idx) fun(simObject$data[[data.set.numbers[idx]]], simObject$results[[idx]]))
				)
				statDF <- rbind(statDF, cbind(paraNameDF[idxs,], tmp))
			}else
			{ 	# avgFun supplied, thus the resulting data.frame will have only one row for
				# every parameter constellation

				statDF <- rbind(statDF, 
						sapply(listOfStatisticFunctions,
								function(fun) avgFun(sapply(idxs, function(idx)
														fun(simObject$data[[data.set.numbers[idx]]],
																simObject$results[[idx]])
											)
									)
						)
				)
			}
		}
		
		if (missing(avgFun))
		{
			# number the rows consecutively
			rownames(statDF) <- 1:length(rownames(statDF))
			
			return(
					list(
							statisticDF = statDF, 
							name.parameters = names(paraNameDF), 
							name.statistics = names(listOfStatisticFunctions), 
							name.avgFun = ""
					)
			)
		}
		
		# label the columns of the resulting data.frame
		names(statDF) <- paste(names(listOfStatisticFunctions), avgFunName, sep=".")
		statDF <- cbind(unqParaNameDF, statDF)		
		
		list(
				statisticDF = statDF, 
				name.parameters = names(paraNameDF), 
				name.statistics = paste(names(listOfStatisticFunctions), avgFunName, sep="."), 
				name.avgFun = avgFunName
		)
	}
	#---------------------------	Subfunctions	--------------------------	
	
	# if no average function is given 
	# the resulting data.frame will have one row 
	# for every object in simObject$results
	if (missing(listOfAvgFunctions))
		return(gatherStatisticsOneAvgFun(simObject, listOfStatisticFunctions))
	
	# the average function is a function, pass this directly to 
	# gatherStatisticsOneAvgFun
	if (is.function(listOfAvgFunctions))
	{		
		return(gatherStatisticsOneAvgFun(
						simObject, 
						listOfStatisticFunctions, 
						listOfAvgFunctions, 
						deparse(substitute(listOfAvgFunctions))
				)
		)
	}
	
	# call gatherStatisticsOneAvgFun for every function in 
	# listOfAvgFunctions 
	if (length(listOfAvgFunctions) > 0)
	{		
		if (sum(names(listOfAvgFunctions) != "") != length(listOfAvgFunctions))
			warning("The functions in listOfAvgFunctions should have a name!")
		
		tmp <- list()
		# cnt is needed to determine the name of "fun"
		cnt <- 0
		for (fun in listOfAvgFunctions)
		{
			cnt <- cnt + 1
			tmp[[cnt]] <- gatherStatisticsOneAvgFun(
					simObject,
					listOfStatisticFunctions,
					fun,
					names(listOfAvgFunctions)[cnt]
			)
		}
		
		
		# We have gathered many statistics, now join the information 
		ret <- tmp[[1]]
		ret$statisticDF <- ret$statisticDF[ret$name.parameters]
		
		
		for (i in seq(along.with = listOfAvgFunctions))
		{
			ret$statisticDF <- cbind(ret$statisticDF, tmp[[i]]$statisticDF[tmp[[i]]$name.statistics])
			ret$name.statistics <- c(ret$name.statistics, tmp[[i]]$name.statistics)
			ret$name.avgFun <- c(ret$name.avgFun, tmp[[i]]$name.avgFun)
		}
		
		ret$name.statistics <- unique(ret$name.statistics)
		ret$name.avgFun <- unique(ret$name.avgFun)
		
		return(ret)
	}	
}


simulation <- function(replications, DataGen, listOfProcedures, discardProcInput=FALSE) 
{	
	paraNameDataGen <- names(DataGen)
	if (length(paraNameDataGen) != length(unique(paraNameDataGen)))
	{
		cat("Parameter of data generating function:\n\t", paraNameDataGen, "\n")
		stop("Parameternames of the data generating function are not unique")
	}
	
	# check if parameter of the procedures are unique
	nameProblems <- FALSE
	for( i in seq(along.with = listOfProcedures) )
	{
		paraNameProc <- names(listOfProcedures[[i]])
		if (length(paraNameProc) != length(unique(paraNameProc)))
		{
			nameProblems <- TRUE
			cat("Parameters of procedure", listOfProcedures[[i]]$funName, "are not unique:\n\t", paraNameProc, "\n")
		}
	}
	if (nameProblems) stop("Parameters of some procedures are not unique.\n")
	
	# no intersection between parameters of the data generating function and the
	# procedures are allowed.
	nameProblems <- FALSE	
	for( i in seq(along.with = listOfProcedures) )
	{
		paraNameProc <- names(listOfProcedures[[i]])
		equalNames = sort(intersect(paraNameDataGen, paraNameProc))
		
		if (length(equalNames)!= 2 || !all(equalNames == c("fun", "funName")))
		{
			nameProblems <- TRUE			
			cat("Common names of the data generating function and multiple test procedure", listOfProcedures[[i]]$funName, "are:\n\t", equalNames, "\n")
		}		
	}
	if (nameProblems) stop("The only common name of data generating function and multiple test procedure should be 'fun' and 'funName'.\n")
	
	# TODO: MS print progress of the simulation on the console!
	
	# generating all data generating functions
	dataGenStack 	<- generateFunctionStack(DataGen)
	
	# a bunch of stacks full of procedures
	# for example for every method (bonferroni and holm) there
	# is a stack for bonferroni with the different parameter configurations
	# and a stack for holm with the different parameter configurations
	procedureStacks	<- lapply(listOfProcedures, function(procs) generateFunctionStack(procs))
	names(procedureStacks) 	<- sapply(listOfProcedures, function(procs) procs$funName)
	
	ret = list()
	
	# cnt is used as an identifier. So every list with
	# the same $data.set.number is based on the same generated data
	cnt <- 0
	for( dataGenCall in dataGenStack )
	{	
		# This is probably the right place for gridComputation
		
		# It calls ONE time
		# dataGenCall. Every procedure in procedureStacks is applied
		# to this one "dataSet". returns $data and $results
		genOneDataSetAndApplyProcedures <- function(dummy) 
		{
			# generating data
			data <- eval(as.call(dataGenCall))
			
			# cnt is a global variable that has to be increased                     
			# each time a new "dataSet" is generated.
			assign("cnt", get("cnt", envir=sys.frame(-2)) + 1, envir = sys.frame(-2))
			
			# every procedure will be applied to the generated Dataset and the results
			# will be stored in the following list
			procs.results <- vector("list", sum(sapply(procedureStacks, function(stack) length(stack))))
			procs.results.idx <- 0
			for (pS in seq(along.with=procedureStacks))
			{
				# procedureStacks consists of stacks, go through one by one
				# this means applying every procedure to the given dataset
				procStack <- procedureStacks[[pS]]
				for (proc in procStack)
				{
					procs.results.idx <- procs.results.idx + 1
					result <- list()
					
					# every dataset get a unique number
					result$data.set.number <- get("cnt", envir  = sys.frame(-2))
					
					# saving the parameter constellation of the used data generating function
					paramDataGen <- c(DataGen$funName, dataGenCall[-1])
					names(paramDataGen)[1]	<- "funName"

					# saving the parameter constallation of the used procedure
					paramProc <- names(procedureStacks)[pS]
					
#					if (length(proc) == 1) 
#						#proc uses only the parameter from the output of the data generating function
#						paramProc <- c(paramProc)#, dummy="")
#					else
					if(length(proc)>1)
						paramProc <- c(paramProc, proc[-1])
					
					names(paramProc)[1]	<- "method"
					
					# saving the parameter constallation of the data generating function and the procedure 	
					result$parameters <- c(paramDataGen, paramProc)
					

					# next step is to assign the inputdata for the procedure parameters that was generated by
					# the data generating function. But at first I check if this will overwrite
					# other parameters already specified for the procedure.
					inter <- intersect(names(data$procInput), names(proc))
					if (length(inter) != 0)
						warning("\n\n\tSome of the parameter of one procedure are already specified,\n\t",
								"and the data generating function now provides new values for these",
								"parameters :\n\n\t",
								"Affected procedure : ", listOfProcedures[[pS]]$funName, "\n\t",
								"Affected parameters: ", paste(inter, collapse=" "), "\n")
					
					# assign inputdata generated by the data generating function to the procedure parameters.
					for(paraInputName in names(data$procInput))
						proc[[paraInputName]] <- data$procInput[[paraInputName]]
					
					# calling the procedure
					procOutput 	<- eval(as.call(proc))
					
					# writing the output of the procedure into the result
					for(name in names(procOutput))
						result[[name]] <-  procOutput[[name]]
					
					# append the new result
					procs.results[[procs.results.idx]] <- result
				}
			}

			if (discardProcInput)
				data$procInput = NULL		
			
			# return the used dataset, the output of the procedure with the used
			# parameter constellation
			return(list(data=data, results=procs.results))
		}
		ret <- c(ret, lapply(1:replications, genOneDataSetAndApplyProcedures))
	}
	

	# I want to have $data for data and $results for the output of
	# the procedures and parameter constellations. 	
	only.results <- sapply(ret, function(obj) obj$results)	
	dim(only.results) <- NULL
	return(
			list(data=lapply(ret, function(obj) obj$data),
					results=only.results
			)
	)	
}

 
\documentclass[a4paper]{article}
\usepackage[OT1]{fontenc}
\usepackage{Sweave}
\usepackage{amssymb}
%\usepackage{hyperref}

\newcommand{\sT}{simTool}
\newcommand{\pv}{$p$-value }
\newcommand{\pvs}{$p$-values }
\newcommand{\mts}{MutossSimObject}
\newcommand{\tw}[1]{\texttt{#1}}

\begin{document}

% \VignetteIndexEntry{MuToss Simulation Tool Guide}

\title{\sT}
\author{MarselScheer}



\maketitle

\tableofcontents

<<echo=FALSE>>=
require(mutoss)
@

\section{Note}
This SweaveFile is part of $\mu$Toss package. But because some parts of 
this document are computational intensiv the extension of the Sweavefile
is ".actuallyRnw".


\section{Introduction}
\label{introduction}
This document will give an introduction to the use of \sT. 
We will start with a very simple application then raise the
degree of complexity in a few steps and in the end reproduce 
some of the results from Benjamini, Krieger, Yekutieli (2006).
Basically for a simulation we need the following things
\begin{enumerate}
\item a function that generates the data (for example \pvs)
\item some procedures that evaluates the generated data (for 
	example the bonferroni correction)
\item statistics we want to calculate. For example the the false 
discovery proportion (FDP).
\end{enumerate}
If we are speak for example of $1000$ replications, we mean that these 3 steps
were repeated $1000$ times. That means, after $1000$ replications there have been
$1000$ data sets generated. Every procedure was applied to these $1000$ data sets.
If we have specified for example $3$ procedures then every of these procedures
will give us an output, this means $3000$ "output objects". And every specified
statistic is applied to these $3000$ "output objects".

\subsection{Motivation}
\subsubsection{Example 1}
Suppose we want to compare some characteristics of the two procedures \tw{BH} and \tw{holm}. 
Denote by $V_n(BH)$ and $V_n(holm)$ the number of true hypotheses rejected. For example
we are interested in the distribution of $V_n(BH)$ and $V_n(holm)$ and also in $E[V_n(BH)]$
and $E[V_n(holm)]$. Lets have a look on the arguments of theses procedures:
<<>>=
args(BH)
args(holm)
@
Both have the argument \tw{alpha}, which stands for the niveau 
of the corresponding error measure. 
\tw{BH} controls the false discovery rate (FDR) and \tw{holm} 
controls the family-wise error rate (FWER). One question may be how 
must \tw{alpha} be set in \tw{holm} such that for fixed \tw{alpha} = 0.1 in 
\tw{BH} we have $E[V_n(BH)] \approx E[V_n(holm)]$?
And perhaps we are also interested in how a dependence structure affects the 
distribution of $V_n(BH)$ and $V_n(holm)$.

\subsubsection{Example 2}
Suppose you have developed a new procedure controlling some error measure and you
want to compare this new procedure with some other already established procedures. 
Then a simulation study will consist of creating data and gathering statistics for 
different sample sizes, dependence structures and parameter constellations.
$$
$$
Basically it is always the same story. Data is generated by a specific function, 
perhaps this depends on some parameters, and we want to apply different procedures,
again depending on some paramters, to this generated data. Nearly everyone will say,
"don't bother me with the details of programming, just do the simulation 
with the above information and give me a nice \tw{data.frame} which I can analyze."
And this is exactly the purpose of the simTool.


\subsection{Data generating function}
\label{DataGenFun}
For now we only want to use the procedures \texttt{BH} and \texttt{holm}. Again, lets 
have a look on the arguments of these procedures:
<<>>=
args(BH)
args(holm)
@
Not much has to be specified. The parameter \texttt{alpha} is the level at which 
the corresponding error rate should be controlled and \tw{pValues} are the \pvs 
of the hypotheses that should be tested. In general, if we say that we reject a
\pv this means that we reject the corresponding hypotheses.
The following function generates data and will be used throughout the whole
document. The data generating function \textbf{must have} an entry \$procInput.
All in \$procInput will be used as input for the specified procedures.
In our situation we will generate a list with 2 entries.  \$procInput
will only consist of the generated \pvs. And \$groundTruth indicates which \pv 
corresponds to a true or false hypothesis.  
<<>>=
pValFromEquiCorrData <- function(sampleSize, rho, mu, pi0)
{		
	nmbOfFalseHyp <- round(sampleSize * (1-pi0))
	nmbOfTrueHyp <- sampleSize - nmbOfFalseHyp
	muVec <- c(rep(mu, nmbOfFalseHyp), rep(0, nmbOfTrueHyp))
	Y <- sqrt(rho) * rnorm(1) + sqrt(1-rho) * rnorm(sampleSize) + muVec
	return(list(
		procInput=list(pValues = 1 - pnorm(Y)),
		groundTruth = muVec == 0
		)
	)
}
@


\subsubsection{Illustration of generated Data}
We will now generate 1000 $p$-Values independently (\tw{rho} = 0). 
\tw{sampleSize * (1-pi0)} = 700 of them correspond to false hypotheses and 300 to 
true hypotheses.
<<>>=
set.seed(123)
data <- pValFromEquiCorrData(
		sampleSize = 1000, 
		rho = 0, 
		mu = 2, 
		pi0 = 0.3)
@
Lets visualize the different \pvs. 
<<fig=TRUE>>=
local({
pValues = data$procInput$pValues
groundTruth = data$groundTruth
plot(ecdf(pValues), do.points=FALSE, verticals=TRUE, main="ecdf's of generated p-values")
lines(ecdf(pValues[groundTruth]), do.points=FALSE, verticals=TRUE, col=2)
lines(ecdf(pValues[!groundTruth]), do.points=FALSE, verticals=TRUE, col=4)
abline(0,1,lty=2)
legend("right", legend=c("all", "true", "false"), col=c(1,2,4), lty=1, title="p-values")
})
@

\subsection{A very simple example}
Lets directly start with the function call and then analysing what happend. Just for now we
implement a simple version of the data generating function in section \ref{DataGenFun} and
a simple version of the \tw{BH}.
<<>>=
myGen <- function()
{
	pValFromEquiCorrData(sampleSize = 200, rho = 0, mu = 2, pi0=0.5)
}
BH.05 = function(pValues)
{
	BH(pValues=pValues, alpha = 0.05, silent=TRUE)
}
@

<<>>=
set.seed(123)
sim <- simulation(replications = 10, list(funName="myGen", fun=myGen), 
		list(list(funName="BH.simple", fun=BH.05)))
@
The following happend:
\begin{enumerate}
	\item Call \tw{myGen}
	\item Append generated data set to \tw{sim\$data}
	\item Call \tw{BH.05} with the generated \pvs
	\item Add to the results of \tw{BH.05} the parameter constellation used
		by \tw{myGen} and \tw{BH.05} and also the position of the used
		data in \tw{sim\$data}
	\item Append this extended results of \tw{BH.05} to \tw{sim\$results}
	\item repeat this 9 more times    
\end{enumerate}
Since \tw{replications = 10} in \tw{simulation} the object \tw{sim} consists of:
<<>>=
names(sim)
length(sim$data)
length(sim$results)
@
The structure of one object in \tw{sim\$data} coincides with the structure of the return
of \tw{myGen}:
<<>>=
names(myGen())
names(sim$data[[1]])
@
Lets have a look at the additional information added by \tw{simulation} to the object returned
by \tw{BH.05}. First the entries of the pure return value of \tw{BH.05} and then of \tw{sim\$results} 
<<>>=
names(BH.05(runif(100)))
names(sim$results[[1]])
@
\tw{data.set.number} is the number of the used data set in \tw{sim\$data}. Since every generated
data set is used only once we have
<<>>=
sapply(sim$results, function(x) x$data.set.number)
@
 
Thus it is possible
to reproduce any result directly. Let us reproduce the results of the 6th replication
<<>>=
idx <- sim$results[[6]]$data.set.number
pValues <- sim$data[[idx]]$procInput$pValues
all(BH.05(pValues)$adjPValues == sim$results[[6]]$adjPValues)
@
Since our data generating function and procedure did not really have any parameter, there is
not much information in:
<<>>=
sim$results[[1]]$parameters
@
Note, \tw{BH.05} technically has the parameter \tw{pValues} but since this parameter is 
the "input channel" for the output data of the data generating function it is not really 
regarded as parameter that affect the behaviour of the procedure like for example the 
level $\alpha$ for the error measure.  

\subsection{A simple example}
\label{a.simple.example}
Indeed, for the simple example above the simTool is not very helpful. Let us increase the 
complexity of our simulation. As a first step, we want to increase the parameter \tw{rho} 
of the data generating function gradually and investigate $E V_n(BH)$ and $E V^2_n(BH)$.
<<>>=
set.seed(123)
sim <- simulation(replications = 10, 
		list(funName="pVGen", fun=pValFromEquiCorrData, 
				sampleSize = 100, 
				rho = seq(0,1,by=0.2),
				mu = 2,
				pi0 = 0.5), 
		list(list(funName="BH",	fun=BH, 
			alpha = c(0.5, 0.25), 
			silent=TRUE)))	
@
The following happend:
\begin{enumerate}
	\item Call \tw{myGen} with \tw{rho = 0}
	\item Append generated data set to \tw{sim\$data}
	\item Call \tw{BH} with \tw{alpha = 0.5} and the generated \pvs
	\item Add to the results the parameter constellation used
		by \tw{myGen} and \tw{BH} and also the position of the used
		data in \tw{sim\$data}
	\item Append this extended results to \tw{sim\$results}
	\item Call \tw{BH} with \tw{alpha = 0.25} and the \textbf{same} \pvs as in 3.
	\item Add to the results the parameter constellation used
		by \tw{myGen} and \tw{BH} and also the position of the used
		data in \tw{sim\$data}		
	\item Append this extended results of to \tw{sim\$results}
	\item repeat this 9 more times
	\item Call \tw{myGen} with \tw{rho = 0.2}
	\item and so on   
\end{enumerate}
So we have now a bunch of data sets and results.
<<>>=
length(sim$data)
length(sim$results)
@
60 data sets have been generated because we have \tw{rho = 0, 0.2, 0.4, 0.6, 0.8, 1} and for
each \tw{rho} we generated 10 data sets. We have applied \tw{BH} with \tw{alpha = 0.5} to all
60 data sets and \tw{BH} with \tw{alpha = 0.25}, yielding 120 objects in \tw{results}.
We see from the \tw{data.set.number} that after generating a data set it is used two times in series
<<>>=
sapply(sim$results, function(x) x$data.set.number)
@

In order to gather how many true hypotheses have been rejected corresponding to the different
parameter constellations we need a function that is able to calculate it.
<<>>=
NumberOfType1Error <- function(data, result) sum(data$groundTruth * result$rejected)
V2 <- function(data, result) NumberOfType1Error(data,result)^2
#result.all <- gatherStatistics(sim, list(NumOfType1Err = NumberOfType1Error))
@
Lets us calculate for the first result object the number of rejected true hypotheses explicitly.
<<>>=
idx = sim$results[[1]]$data.set.number
data = sim$data[[idx]]
NumberOfType1Error(data, sim$results[[1]])
@
This means for the following parameter constellation
<<>>=
sim$results[[1]]$parameters
@
we one time observed 
<<echo=FALSE>>=
NumberOfType1Error(data, sim$results[[1]])	
@
rejected true null hypotheses. In order to estimate $E V_n(BH)$ and $E V^2_n(BH)$ 
we have to search in \tw{sim\$results} for the other 9 results using the same 
parameter constellation. In order to facilitate this task we provide a function.
<<>>=
result <- gatherStatistics(sim, 
		list(V = NumberOfType1Error, V2 = V2),
		mean)
result
@
As you can see every, parameter constellation has its own row in \tw{\$statisticDF}. 
By the way, again, we see that \pvs is a parameter of \tw{BH} but since it is contained in \tw{\$procInput} it is
not considered as "real parameter".\\
If we are interested in more than one statistics, then we simply provide a list with "average functions"
<<>>=
result <- gatherStatistics(sim, 
		list(V = NumberOfType1Error, V2 = V2), 
		list(mean = mean, sd = function(vec) round(sd(vec), 1)))
result$statisticDF
@
Another possibility is to call \tw{gatherStatistics} without and "average function". Then every result
get his own row in \tw{\$statisticDF} 
<<>>=
result <- gatherStatistics(sim, 
		list(V = NumberOfType1Error, V2 = V2))
head(result$statisticDF)
tail(result$statisticDF)
@


\subsection{Plotting a bit}
<<>>=
if(!is.element("sim.plot", ls()))
{
sim.plot <- simulation(replications = 1000,
	list(funName = "pVGen",
		fun = pValFromEquiCorrData,
		sampleSize = 100,
		rho = seq(0, 1, by = 0.2),
		mu = 0,
		pi0 = 1
	),
	list(
		list(funName = "BH",
			fun = function(pValues, alpha) BH(
				pValues, 
				alpha, 
				silent=TRUE),
			alpha = c(0.5, 0.75)
		)	
	)
)
}
length(sim.plot$data)
length(sim.plot$results)
@
The are already different kind of R functions that take data.frames and generate 
histograms, boxplots and so on from them.
For some plot example we will need the lattice package.
<<>>=
require(lattice)
@
First we calculate V for every MutossSim object and then plot a histogram and a boxplot of V, 
see Figure~\ref{fig:HistV1} (p.~\pageref{fig:HistV1}), Figure~\ref{fig:BWV1} (p.~\pageref{fig:BWV1}).
<<>>=
result.all <- gatherStatistics(
		sim.plot,
		list(V = NumberOfType1Error)
)		
@
<<label=figHistV1,include=FALSE, echo=FALSE>>=
print(
	histogram(~V | rho, 
		data = subset(
			result.all$statisticDF, 
			alpha=="0.5")))
@
\begin{figure}
\begin{center}
<<label=fig1,fig=TRUE,echo=TRUE>>=
<<figHistV1>>
@
\end{center}
\caption{Histogram of the number of rejected true hypotheses for alpha equals 0.5}
\label{fig:HistV1}
\end{figure}
<<label=figBWV1,include=FALSE, echo=FALSE>>=
print(
	bwplot(V ~ rho | alpha, 
		data = subset(result.all$statisticDF)))
@
\begin{figure}
\begin{center}
<<label=fig2,fig=TRUE,echo=TRUE>>=
<<figBWV1>>
@
\end{center}
\caption{Boxplot of the number of rejected true hypotheses}
\label{fig:BWV1}
\end{figure}
Also after the "average process" we again get an data.frame which can be used to generate 
plots.
<<>>=
result <- gatherStatistics(
	sim.plot,
	list(V = NumberOfType1Error),
	list(Ml = function(x) 
			mean(x) - 2 * sd(x)/sqrt(length(x)),
		Mu = function(x) 
			mean(x) + 2 * sd(x)/sqrt(length(x)),
		M = mean, 
		SD = sd)
)
subset(result$statisticDF, alpha=="0.5")[1:3,-1]

<<label=figXYPlot,include=FALSE, echo=FALSE>>=
print(
	xyplot(
		V.Ml + V.M + V.Mu ~ rho | alpha, 
		data = result$statisticDF,
		type = "a",
		auto.key=list(
			space="right", 
			points=FALSE, 
			lines=TRUE)
	)
)
@
\begin{figure}
\begin{center}
<<label=fig3,fig=TRUE,echo=TRUE>>=
<<figXYPlot>>
@
\end{center}
\caption{XYPlot of the mean number of rejected true hypotheses 
with asymptotic 95\% confidence intervalls (pointwise)}
\label{fig:XYPlotV1}
\end{figure}


\subsection{Memory considerations}
Up to now we have kept any information in memory. Somtimes, it is useful not to discard any
generated Information. For example,
if some objects in results appear odd, they can directly be reproduced and of course with all
information availible we can investigate anything like distribution or any statistic we
did not consider before the simulation. The price for this liberty is in general an
extensive memory usage which of course restrict the size of our simulation. On the other
hand if we exactly know that we are only interested in $E V_n$ the mean number of true 
rejected hypotheses, why should we keep the generated \pvs or the index of the rejected
\pvs ?  Thus it would be enough that an obejct in \tw{\$results} contains $V_n$ the number
of rejected true hypotheses. We will know show an example were only the information needed
is kept in memory. The following is basically the "simple example" from section \ref{a.simple.example}
The main issue is to save memory thus we will calculate the $V_n$ right after applying the
procedure to the generated data set. If done so, we can also discard the generated data!
<<>>=
V.BH <- function(pValues, groundTruth, alpha)
{
	out <- BH(pValues, alpha, silent=TRUE)
	V <- sum(out$rejected * groundTruth)
	return(list(V = V))
}
@
As already mentioned anything in \tw{\$procInput} will be used as an input for the procedures.
Since our procedure now has a parameter \tw{groundTruth} our data generated function has to
be adepted. We just move \tw{groundTruth} in \tw{\$procInput}. 
<<>>=
pValFromEquiCorrData2 <- function(sampleSize, rho, mu, pi0)
{		
	nmbOfFalseHyp <- round(sampleSize * (1-pi0))
	nmbOfTrueHyp <- sampleSize - nmbOfFalseHyp
	muVec <- c(rep(mu, nmbOfFalseHyp), rep(0, nmbOfTrueHyp))
	Y <- sqrt(rho) * rnorm(1) + sqrt(1-rho) * rnorm(sampleSize) + muVec
	return(list(
		procInput=list(pValues = 1 - pnorm(Y),
				groundTruth = muVec == 0)
		)
	)
}
@
Obviously, it is unnecessary
to store this generated data, because the number of Type 1 Errors we are interested in 
will be directly calculated.
Setting \tw{discardProcInput = TRUE} in \tw{simulation} removes \tw{\$procInput} from the object
generated by the data generating function after \tw{\$procInput} has been used by all specified
procedures. In our case theses will be the 2 procedures \tw{V.BH} with \tw{alpha = 0.5} and
\tw{alpha = 0.25}. 
<<>>=
set.seed(123)
sim <- simulation(replications = 10, 
		list(funName="pVGen2", fun=pValFromEquiCorrData2, 
				sampleSize = 100, 
				rho = seq(0,1,by=0.2),
				mu = 2,
				pi0 = 0.5), 
		list(list(funName="V.BH", fun=V.BH, alpha = c(0.5, 0.25))),
		discardProcInput=TRUE)
result <- gatherStatistics(sim, 
		list(V = function(data, results) results$V),
		mean)
result	
@
Here the corresponding table from section \ref{a.simple.example}. 
<<>>=
set.seed(123)
sim.simple <- simulation(replications = 10, 
		list(funName="pVGen", fun=pValFromEquiCorrData, 
				sampleSize = 100, 
				rho = seq(0,1,by=0.2),
				mu = 2,
				pi0 = 0.5), 
		list(list(funName="BH", fun=BH, alpha = c(0.5, 0.25), silent=TRUE)))
result.simple <- gatherStatistics(sim.simple, 
		list(V = NumberOfType1Error),
		mean)
print(result.simple)	
@
The same results but the memory usage differ much:
<<>>=
print(s1 <- object.size(sim))
print(s2 <- object.size(sim.simple))	
unclass(s2/s1)		
@


\section{Reproducing some results from BKY (2006)}
Now, we will reproduce figure $1$ from the publication. For this we need to estimate the FDR. 
This will be done be calculating the FDP for every object in \tw{\$results} and then calculating
the empirical mean. In this simulation where we exactly know what we want and thus retain only
the necessary information.
To be able to calculate the FDP right after applying the procedure we need to know which 
pValue belongs to a true or false hypothesis. Thus, like in the foregoing section, 
\tw{\$groundTruth} will be an element of \tw{\$procInput}.
<<>>=
pValFromEquiCorrData2 <- function(sampleSize, rho, mu, pi0)
{		
	nmbOfFalseHyp <- round(sampleSize * (1-pi0))
	nmbOfTrueHyp <- sampleSize - nmbOfFalseHyp
	muVec <- c(rep(mu, nmbOfFalseHyp), rep(0, nmbOfTrueHyp))
	Y <- sqrt(rho) * rnorm(1) + sqrt(1-rho) * rnorm(sampleSize) + muVec
	return(list(
		procInput=list(pValues = 1 - pnorm(Y),
				groundTruth = muVec == 0)
		)
	)
}
@
We also need a function that calculates the FDP.
<<>>=
FDP <- function(pValues, groundTruth, proc=c("BH", "M-S-HLF"))
{
	if( proc == "BH" )
		out <- BH(pValues, alpha = 0.05, silent = TRUE)
	else
		out <- adaptiveSTS(pValues, alpha = 0.05, silent = TRUE)
	
	R <- sum(out$rejected)
	if ( R == 0 ) return(list(FDP = 0))
	
	V <- sum(out$rejected * groundTruth)
	return(list(FDP = V/R))		
}
@
Now, the (partial) reproduction of the results can start!
<<>>=
set.seed(123)
date()
sim <- simulation(replications = 10000, 
		list(funName="pVGen2", fun=pValFromEquiCorrData2, 
				sampleSize = c(16, 32, 64, 128, 256, 512),
				rho = c(0,0.1,0.5),
				mu = 5,
				pi0 = c(0.25, 0.75)), 
		list(list(funName="FDP", fun=FDP, proc=c("BH", "M-S-HLF"))),
		discardProcInput=TRUE)
date()
result <- gatherStatistics(sim, 
		list(FDP = function(data, results) results$FDP),
		mean)
result
@
<<fig=TRUE>>=
print(xyplot(FDP.mean ~ sampleSize | pi0 * rho, data = result$statisticDF, group=proc,
		type = "a",
		auto.key=list(space="right", points=FALSE, lines=TRUE)
	)
)		
@

\end{document}

\name{SidakSD}
\alias{SidakSD}
\title{Sidak-like (1987) step-down procedure}
\usage{SidakSD(pValues, alpha, silent=FALSE)}
\description{The Sidak-like (1987) step-down procedure is applied to pValues
The Sidak-like step-down procedure is an improvement over the Holm's (1979) 
step-down procedure. The improvement is analogous to Sidak's correction 
over the original Bonferroni procedure. This Sidak-like step-down procedure 
assumes positive orthant dependent test statistics.}
\value{A list containing:

\item{adjPValues}{a numeric vector containing the adjusted pValues}

\item{rejected}{a logical vector indicating which hypotheses are rejected}

\item{criticalValues}{a numeric vector containing critical values used in the step-up-down test}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{WerftWiebke}
\references{Hollander, B.S. and Covenhaver, M.D. (1987). An Improved Sequentially Rejective Bonferroni Test Procedure.
Biometrics, 43(2):417-423, 1987.}
\arguments{\item{pValues}{The used raw pValues.}
\item{alpha}{The level at which the FWER shall be controlled.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9,max=1))
result <- SidakSD(p, alpha)
result <- SidakSD(p, alpha, silent=TRUE)}

\name{MutossMethod-class}
\docType{class}
\alias{MutossMethod-class}
\alias{MutossMethod}
\title{Class MutossMethod}
\description{A \code{MutossMethod} object describes a method that is applicable to \code{Mutoss} objects.}
\section{Slots}{  
  \describe{
    \item{\code{label}:}{A character string that contains the label that will be shown in menus.}
    \item{\code{errorControl}:}{One of the following character strings: FWER, FWER.weak, FDR, FDX, gFWER, perComparison.}
    \item{\code{callFunction}:}{A character string that contains the name of the Mutoss-compatible function.}        
    \item{\code{output}:}{A character vector of the \emph{possible} output of the function.}
    \item{\code{info}:}{A character string with info text. Should contain small description, author, reference etc..}
    \item{\code{assumptions}:}{A character vector of assumptions for this method.}
    \item{\code{parameters}:}{A list of optional description of parameters - see MuToss developer handbook.}
    \item{\code{list}:}{For extensions a list where you can put all your miscellaneous stuff.}
  }
}

\name{snk}
\alias{snk}
\alias{snk.wrapper}
\title{Student - Newman - Keuls rejective test procedure.}
\usage{snk(formula, data, alpha, MSE=NULL, df=NULL, silent=FALSE)
snk.wrapper(model, data, alpha, silent=FALSE)}
\description{Student - Newman - Keuls rejective test procedure.
The procedure controls the FWER in the WEAK sense.}
\details{This function computes the Student-Newman-Keuls test for given
data including p samples. The Newman-Keuls procedure is based on a stepwise or 
layer approach to significance testing. Sample means are 
ordered from the smallest to the largest. The largest 
difference, which involves means that are r = p steps apart, 
is tested first at \eqn{\alpha} level of significance; if significant, 
means that are r = p - 1 steps apart are tested at \eqn{\alpha} level 
of significance and so on. The Newman-Keuls procedure provides an
r-mean significance level equal to \eqn{\alpha} for each group of 
r ordered means, that is, the probability of falsely rejecting 
the hypothesis that all means in an ordered group are equal to 
\eqn{\alpha}. It follows that the concept of error rate applies 
neither on an experimentwise nor on a per comparison basis-the
actual error rate falls somewhere between the two. 
The Newman-Keuls procedure, like Tukey's procedure, requires 
equal sample n's. However, in this algorithm, the procedure is 
adapted to unequal sample sized which can lead to still 
conservative test decisions. 

It should be noted that the Newman-Keuls and Tukey procedures 
require the same critical difference for the first comparison 
that is tested. The Tukey procedure uses this critical difference
for all the remaining tests, whereas the Newman-Keuls procedure 
reduces the size of the critical difference, depending on the 
number of steps separating the ordered means. As a result, the 
Newman-Keuls test is more powerful than Tukey's test. Remember,
however, that the Newman-Keuls procedure does not control the 
experimentwise error rate at \eqn{\alpha}.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{statistics}{A numeric vector containing the test-statistics}

\item{confIntervals}{A matrix containing only the estimates}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{Frank Konietschke}
\references{Keuls M (1952). "The use of the studentized range in connection with an analysis of variance". Euphytica 1: 112-122}
\arguments{\item{formula}{Formula defining the statistical model containing the response and the factor levels.}
\item{model}{Model with formula, containing the response and the factor levels}
\item{data}{dataset containing the response and the grouping factor.}
\item{alpha}{The level at which the error should be controlled. By default it is alpha=0.05.}
\item{MSE}{Optional for a given variance of the data.}
\item{df}{Optional for a given degree of freedom.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{x = rnorm(50)
grp = c(rep(1:5,10))
dataframe <- data.frame(x,grp)
result <- snk(x~grp, data=dataframe, alpha=0.05,MSE=NULL, df=NULL, silent = TRUE)
result <- snk(x~grp, data=dataframe,alpha=0.05,MSE=NULL, df=NULL, silent = FALSE)
result <- snk(x~grp, data=dataframe,alpha=0.05,MSE=1, df=Inf, silent = FALSE) # known variance
result <- snk(x~grp, data=dataframe,alpha=0.05,MSE=1, df=1000, silent = FALSE) # known variance}

\name{oracleBH}
\alias{oracleBH}
\title{Bejamini-Hochberg (2000) oracle linear step-up Procedure...}
\usage{oracleBH(pValues, alpha, pi0, silent=FALSE)}
\description{Bejamini-Hochberg (2000) oracle linear step-up Procedure}
\details{Knowledge of the number of true null hypotheses (m0) can be very useful to improve upon the performance of the FDR controlling procedure. 
For the oracle linear step-up procedure we assume that m0 were given to us by an 'oracle', the linear step-up procedure with q0 = q*m/m0
would control the FDR at precisely the desired level q in the independent and continuous case, and
would then be more powerful in rejecting hypotheses for which the alternative holds.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test.}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{HackNiklas}
\arguments{\item{pValues}{pValues to be used}
\item{alpha}{the level at which the FWER should be controlled}
\item{pi0}{miraculousy known number of true null hypotheses}
\item{silent}{Logical, if true any output on the console will be suppressed.}}
\examples{pval <- c(runif(50), runif(50, 0, 0.01))
result <- oracleBH(pValues=pval,alpha=0.05,pi0=0.85)}

\name{gatherParameters}
\alias{gatherParameters}
\title{Extracts the parameters from the simulation()-Object}
\usage{gatherParameters(simObject)}
\description{Basically, it is a helper function for gatherStatistics(). 
It extracts the parameters from the simObject$results and
creates a data.frame from this. Every used parameter gets its own column.
Every row corresponds to one object in simObject$results. If a object A from
simObject$results does not contain a parameter P that another object B does, 
then the row for object A will have "" in the column for the parameter P.}
\value{A data.frame with rows for every object in simObject$results and columns for
the used parameter.}
\author{MarselScheer}
\arguments{\item{simObject}{An object returned by simulation()}}
\examples{#' # this function generates pValues 
myGen <- function(n, n0) {
  list(procInput=list(pValues = c(runif(n-n0, 0, 0.01), 
       runif(n0))), groundTruth = c(rep(FALSE, times=n-n0), rep(TRUE, times=n0)))
}

# need some simulation()-Object I can work with 
sim <- simulation(replications = 3, list(funName="myGen", fun=myGen, n=200, n0=c(50,100)), 
list(list(funName="BH", fun=function(pValues, alpha) BH(pValues, alpha, silent=TRUE), 
  alpha=c(0.25, 0.5)),
list(funName="holm", fun=holm, alpha=c(0.25, 0.5),silent=TRUE)))

gatherParameters(sim)}

\name{requireLibrary}
\alias{requireLibrary}
\title{Tries to load a package.}
\usage{requireLibrary(package)}
\description{Tries to load a package.
If this package does not exist, it will ask the user whether the package should be installed and loaded.
If the user negates, we will raise an error via stop.}
\value{NULL}
\author{MuToss-Coding Team}
\arguments{\item{package}{Package to load}}

\name{BR_pi0_est}
\alias{BR_pi0_est}
\title{Estimate of pi0 using the one-step Blanchard-Roquain procedure}
\usage{BR_pi0_est(pValues, alpha, lambda=1, truncate=TRUE)}
\description{The proportion of true nulls is estimated using the Blanchard-Roquain 1-stage 
procedure with parameter (alpha,lambda) via the formula}
\details{estimated pi_0 = ( m - R(alpha,lambda)  + 1) / ( m*( 1 - lambda * alpha ) )

where R(alpha,lambda) is the number of hypotheses rejected by the BR 1-stage procedure,
alpha is the FDR level for this procedure and lambda a 
parameter belonging to (0, 1/alpha) with default value 1. 
Independence of p-values is assumed.
This estimate may in some cases be larger than 1; it is truncated to 1 if the parameter truncated=TRUE.
The estimate is used in the Blanchard-Roquain 2-stage step-up (using the non-truncated version)}
\value{\item{pi0}{The estimated proportion of true null hypotheses.}}
\author{GillesBlanchard}
\references{Blanchard}
\arguments{\item{pValues}{The raw p-values for the marginal test problems (assumed to be independent)}
\item{alpha}{The FDR significance level for the BR procedure}
\item{lambda}{(default 1) The parameter for the BR procedure, shoud belong to (0, 1/alpha)}
\item{truncate}{(logical, default TRUE) if TRUE, output estimated is truncated to 1}}

\name{multcomp.wrapper}
\alias{multcomp.wrapper}
\title{Simultaneous confidence intervals for arbitrary parametric contrasts in unbalanced one-way layouts.}
\usage{multcomp.wrapper(model, hypotheses, alternative, rhs=0, alpha, factorC)}
\description{Simultaneous confidence intervals for arbitrary parametric contrasts in unbalanced one-way layouts.
The procedure controls the FWER in the strong sense.}
\details{this function, it is possible to compute simultaneous confidence
for arbitrary parametric contrasts in the unbalanced one way layout. Moreover, it computes
p-values. The simultaneous confidence intervals are computed using
multivariate t-distribution.}
\value{A list containing:  
\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{confIntervals}{A matrix containing the estimates and the lower and upper confidence bound}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{MuToss-Coding Team}
\arguments{\item{model}{a fitted model, for example an object returned by lm, glm, or aov etc. It is
assumed that coef and vcov methods are available for model.}
\item{hypotheses}{a specification of the linear hypotheses to be tested.}
\item{alternative}{a character string specifying the alternative hypothesis, must be one of 'two.sided'
(default), 'greater' or 'less'.}
\item{rhs}{an optional numeric vector specifying the right hand side of the hypothesis.}
\item{alpha}{the significance level}
\item{factorC}{character string, specifing the factor variable of interest}
}
\examples{data(warpbreaks)
# Tukey contrast on the levels of the factor 'Tension'

multcomp.wrapper(aov(breaks ~ tension, data = warpbreaks), 
  hypotheses = "Tukey", alternative="two.sided", factorC="tension",alpha=0.05)

# Williams contrast on 'Tension'
multcomp.wrapper(aov(breaks ~ tension, data = warpbreaks), 
  hypotheses= "Williams", alternative="two.sided",alpha=0.05,factorC="tension")

# Userdefined contrast matrix
K <-matrix(c(-1,0,1,-1,1,0, -1,0.5,0.5),ncol=3,nrow=3,byrow=TRUE)
multcomp.wrapper(aov(breaks ~ tension, data = warpbreaks), 
  hypotheses=K, alternative="two.sided",alpha=0.05,factorC="tension")

# Two-way anova
multcomp.wrapper(aov(breaks ~ tension*wool, data = warpbreaks), 
  hypotheses="Tukey", alternative="two.sided",alpha=0.05,factorC="wool")
multcomp.wrapper(aov(breaks ~ tension*wool, data = warpbreaks), 
  hypotheses="Tukey", alternative="two.sided",alpha=0.05,factorC="tension")
multcomp.wrapper(aov(breaks ~ tension*wool, data = warpbreaks), 
  hypotheses=K, alternative="two.sided",alpha=0.05, factorC="tension")
data(iris)
multcomp.wrapper(model=lm(Sepal.Length ~ Species, data=iris), 
  hypotheses="Tukey","two.sided",alpha=0.05, factorC="Species")
K <-matrix(c(-1,0,1,-1,1,0, -1,0.5,0.5),ncol=3,nrow=3,byrow=TRUE)
multcomp.wrapper(model=lm(Sepal.Length ~ Species, data=iris),
  hypotheses=K,"two.sided",alpha=0.05, factorC="Species")}

\name{Mutoss-class}
\docType{class}
\alias{Mutoss-class}
\title{Class Mutoss}
\description{A Mutoss object can store the input for a multiple test procedure and also the output.}
\section{Slots}{
  \describe{
    \item{\code{data}:}{Raw data used in model}
    \item{\code{model}:}{link function,error family and design}
    \item{\code{description}:}{a general description}
    \item{\code{statistic}:}{for Z, T or F statistics}
    \item{\code{hypotheses}:}{of class ANY}
    \item{\code{hypNames}:}{identifiers for the hypotheses tested}
    \item{\code{criticalValues}:}{procedure-specific critical values}
    \item{\code{pValues}:}{raw p-values}
    \item{\code{adjPValues}:}{procedure-specific adjusted p-values}
    \item{\code{errorControl}:}{A Mutoss S4 class of type \code{errorControl}}
    \item{\code{rejected}:}{Logical vector of the output of a procedure at a given error rate}
    \item{\code{qValues}:}{Storey's estimates of the supremum of the pFDR}
    \item{\code{locFDR}:}{Efron's local fdr estimates}
    \item{\code{pi0}:}{Estimate of the proportion of null hypotheses}
    \item{\code{confIntervals}:}{Confidence intervals for selected parameters}
    \item{\code{commandHistory}:}{commandHistory}
  }
}
\author{MuToss-Coding Team}

\name{BL}
\alias{BL}
\title{Benjamini-Liu (1999) step-down procedure}
\usage{BL(pValues, alpha, silent=FALSE)}
\description{Benjamini-Liu's step-down procedure is applied to pValues. 
The procedure controls the FDR if the corresponding test statistics are stochastically independent.}
\details{The Benjamini-Liu (BL) step-down procedure neither dominates nor is dominated by the Benjamini-Hochberg (BH) step-up procedure.
However, in Benjamini and Liu (1999) a large simulation study concerning the power of the two procedures reveals that the BL step-down procedure is more suitable when the number of hypotheses is small. 
Moreover, if most hypotheses are far from the null then the BL step-down procedure is more powerful than the BH step-up method.
The BL step-down method calculates critical values according to Benjamin and Liu (1999), 
i.e., c_i = 1 - (1 - min(1, (m*alpha)/(m-i+1)))^(1/(m-i+1)) for i = 1,...,m, where m is the number of hypotheses tested. 
Then, let k be the smallest i for which P_(i) > c_i and reject the associated hypotheses H_(1),...,H_(k-1).}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues.}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test.} 

\item{rejected}{A logical vector indicating which hypotheses are rejected.}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{Werft Wiebke}
\references{Bejamini, Y. and Liu, W. (1999). A step-down multiple hypotheses testing procedure that controls the false discovery rate under independence. 
Journal of Statistical Planning and Inference Vol. 82(1-2): 163-170.}
\arguments{\item{pValues}{Numeric vector of p-values}
\item{alpha}{The level at which the FDR is to be controlled.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9, max=1))
result <- BL(p, alpha)
result <- BL(p, alpha, silent=TRUE)}

\name{pval2locfdr}
\alias{pval2locfdr}
\title{Strimmer et al.'s fdrtool-based local fdr}
\usage{pval2locfdr(pValues, cutoff)}
\description{The function \code{pval2locfdr} takes a vector of p-values and
estimates for each case the local fdr.}
\value{A list containing:

\item{locfdr}{Numeric vector with local FDR values for each case}
\item{rejected}{Logical vector indicating rejection/retention for each hypothesis when a \code{cutoff} is supplied.}}
\author{JonathanRosenblatt}
\references{Strimmer, K. (2008). fdrtool: a versatile R package for estimating local and tail
area-based false discovery rates. Bioinformatics 24: 1461-1462. \cr
Efron B., Tibshirani R., Storey J. D. and Tusher, V. (2001). Empirical Bayes Analysis of a Microarray Experiment.
Journal of the American Statistical Association 96(456):1151-1160.}
\arguments{\item{pValues}{pValues to be used.}
\item{cutoff}{The local fdr cutoff for rejection. Hypotheses with \code{local fdr} smaller then \code{cutoff} will be rejected.}}
\examples{pvals<- runif(1000)^2
pval2locfdr(pvals)
pval2locfdr(pValues=pvals, cutoff=0.4)}

\name{twosamp.marginal}
\alias{twosamp.marginal}
\title{Marginal two sample test}
\usage{twosamp.marginal(data, model, robust, alternative, psi0, equalvar)}
\description{The robust version uses the Wilcoxon-Mann-Whitney test, otherwise a two-sample t-test will be performed.}
\details{A vector of classlabels needs to be provided to distinguish the two groups.}
\value{\item{pValues}{A numeric vector containing the unadjusted pValues}}
\author{MuToss-Coding Team}
\references{
  Wilcoxon, F. (1945). Individual Comparisons by Ranking Methods. Biometrics Bulletin 1:80-83.
  Mann, H. and Whitney, D. (1947). On a test of whether one of two random variables is stochastically larger than the other. Annals of Mathematical Statistics 18:50-60
  Student (1908). The probable error of a mean. Biometrika, 6(1):1-25.
}
 \arguments{
  \item{data}{the data set}
  \item{model}{the result of a call to \code{twosamp.model(classlabel)}}
  \item{robust}{a logical variable indicating whether a two sample t-test or a Wilcoxon-Mann-Whitney test should be used.}
  \item{alternative}{a character string specifying the alternative hypothesis,
          must be one of \code{two.sided}, \code{greater} or \code{less}}
  \item{psi0}{a numeric that defines the hypothesized null value}
  \item{equalvar}{a logical variable indicating whether to treat the two variances as being equal}
}

\name{mu.test.class}
\alias{mu.test.class}
\title{Test for compatible classes for comparison...}
\usage{mu.test.class(classes)}
\description{Test for compatible classes for comparison}
\details{Internal function of muToss package.
Takes list of classes and tests if they are all muToss.}
\value{True if all classes are muToss. False otherwise.}
\author{MuToss-Coding Team.}
\arguments{\item{classes}{A list of class names.}}

\name{BY}
\alias{BY}
\title{Benjamini-Yekutieli (2001) step-up procedure}
\usage{BY(pValues, alpha, silent=FALSE)}
\description{The Benjamini-Yekutieli step-up procedure is applied to pValues. 
The procedure ensures FDR control for any dependency structure.}
\details{The critical values of the Benjamini-Yekutieli (BY) procedure are calculated by 
replacing the alpha of the Benjamini-Hochberg procedure by alpha/sum(1/1:m)), i.e.,
c(i)=i*alpha/(m*(sum(1/1:m))) for i=1,...,m. For large number m of hypotheses the critical values of the BY procedure and the 
BH procedure differ by a factor log(m). Benjamini and Yekutieli (2001) showed that this step-up procedure controls
the FDR at level alpha*m/m0 for any dependency structure among the test statistics.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test} 

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{WerftWiebke}
\references{Benjamini, Y. and Yekutieli, D. (2001). The control of the false discovery rate in multiple testing under dependency.
Annals of Statistics, 29(4):1165-1188.}
\arguments{\item{pValues}{The used unadjusted pValues.}
\item{alpha}{The level at which the FDR shall be controlled.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9, max=1))
result <- BY(p, alpha)
result <- BY(p, alpha, silent=TRUE)}

\name{paired.marginal}
\alias{paired.marginal}
\title{Marginal paired two sample test}
\usage{paired.marginal(data, model, robust, alternative, psi0, equalvar)}
\description{The robust version uses the Wilcoxon signed rank test, otherwise a paired t-test will be performed.}
\details{A vector of classlabels needs to be provided to distinguish the two paired groups. The arrangement of group indices does not matter, as long
		as the columns are arranged in the same corresponding order between groups. For example, if group 1 is code as 0 and group 2 is 
		coded as 1, for 3 pairs of data, it does not matter if the classlabel is coded as (0,0,0,1,1,1) or (1,1,1,0,0,0) or (0,1,0,1,0,1)
		or (1,0,1,0,1,0), the paired differences between groups will be calculated as group2 - group1.}
\value{
\item{pValues}{A numeric vector containing the unadjusted pValues}
}
\author{MuToss-Coding Team}
\references{
  Wilcoxon, F. (1945). Individual Comparisons by Ranking Methods. Biometrics Bulletin 1:80-83.
} 
 \arguments{
  \item{data}{the data set}
  \item{model}{the result of a call to \code{paired.model(classlabel)}}
  \item{robust}{a logical variable indicating whether a paired t-test or a Wilcoxon signed rank test should be used.}
  \item{alternative}{a character string specifying the alternative hypothesis,
          must be one of \code{two.sided}, \code{greater} or \code{less}}
  \item{psi0}{a numeric that defines the hypothesized null value}
  \item{equalvar}{a logical variable indicating whether to treat the two variances as being equal}
}

\name{two.stage}
\alias{two.stage}
\title{A p-value procedure which controls the FDR for independent test statistics.}
\usage{two.stage(pValues, alpha)}
\description{A p-value procedure which controls the FDR for independent test statistics.}
\details{In the Benjamini-Krieger-Yekutieli two-stage procedure the linear step-up procedure is used in
stage one to estimate m0 which is re-plugged in a linear step-up. 
This procedure is more powerful then non-adaptive procedures, while still controlling the FDR. 
On the other hand, error control is not guaranteed under dependence in which case more conservative procedures should be used (e.g. BH).}
\value{A list containing:
\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test.} 

\item{adjPValues}{A numeric vector containing adjusted p-values.}

\item{pi0}{An estimate of the proportion of true null hypotheses among all hypotheses (pi0=m0/m). }

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{JonathanRosenblatt}
\arguments{\item{pValues}{A numeric vecor of p-values.}
\item{alpha}{The FDR error rate to control.}}
\examples{pvals<- runif(100)^2
two.stage(pvals, 0.1)}

\name{adaptiveSTS}
\alias{adaptiveSTS}
\title{Storey-Taylor-Siegmund (2004) adaptive step-up procedure}
\usage{adaptiveSTS(pValues, alpha, lambda=0.5, silent=FALSE)}
\description{Storey-Taylor-Siegmund's (2004) adaptive step-up procedure}
\details{The adaptive STS procedure uses a conservative estimate of pi0 which is 
plugged in a linear step-up procedure. The estimation of pi0 requires a 
parameter (lambda) which is set to 0.5 by default.
Note that the estimated pi0 is truncated at 1 as suggested by the author, 
so the implemetation of the procedure is not entirely supported by the proof in the reference.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{Werft Wiebke}
\references{Storey, J.D., Taylor, J.E. and Siegmund, D. (2004). Strong control, conservative point estimation and
simultaneous conservative consistency of false discovery rates: a unified approach.
Journal of the Royal Statistical Society, B 66(1):187-205.}
\arguments{\item{pValues}{The used raw pValues.}
\item{alpha}{The level at which the FDR shall be controlled.}
\item{lambda}{The tuning parameter for the estimation procedure (defaults to 0.5)}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9,max=1))
result <- adaptiveSTS(p, alpha, lambda=0.5)
result <- adaptiveSTS(p, alpha, lambda=0.5, silent=TRUE)}

\name{pValuesPlot}
\alias{pValuesPlot}
\title{A function plotting p-values...}
\usage{pValuesPlot(pValues)}
\description{A function plotting p-values}
\author{MarselScheer}
\arguments{\item{pValues}{A numeric containing the pValues to plot.}}

\name{compareMutoss}
\title{Functions for comparing outputs of different procedures.}
\usage{compareMutoss(...)
mu.compare.adjusted(comparison.list, identify.check=F)
mu.compare.critical(comparison.list, identify.check=F)
mu.compare.summary(comparison.list)}
\description{Functions for comparing outputs of different procedures.}
\details{These functions are used to compare the results of different multiple comparisons procedures stored as Mutoss class objects.
\code{compareMutoss} takes as input an arbitrary number of Mutoss objects and arranges them in a simple list objects (non S4).
\code{mu.compare.adjuted}, \code{mu.compare.critical} and \code{mu.compare.summary} take the output of the \code{compareMutoss} and plots or summerize the results textually or graphically.}
\value{\item{compareMutoss}{Returns a list with the following components:\cr
\bold{types}: Character vector of error types corrsponding to each procedure.\cr
\bold{rates}: Numeric vector of error rates used for each procedure.\cr
\bold{pi.nulls}: Numeric vector of estimates of the proportion of true null hypothesis if avilable.\cr
\bold{raw.pValues}: The raw p-values used for each procedure.\cr
\bold{adjusted.pvals}: Data frame with columns holding procedure specific adjusted p values.\cr
\bold{criticalValue}: Data frame with columns holding the critical values corresponding to each procedure and error rate.\cr
\bold{rejections}: Data frame with columns holding logical vectors of rejected hypotheses (TRUE for rejected).\cr}
\item{mu.compare.adjusted}{Creates a plot with the adjusted p-values for each procedure.}
\item{mu.compare.critical}{Creates a plot with the critical values for each procedure and error rate.}
\item{mu.compare.summary}{Creates a short textual summary for comparing results of different procedures.}}
\author{Jonathan Rosenblatt}
\alias{compareMutoss}
\alias{mu.compare.adjusted}
\alias{mu.compare.critical}
\alias{mu.compare.summary}
\arguments{\item{...}{An arbitrary number of Motoss class objects.}
\item{comparison.list}{The output of the \code{compareMutoss} function.}
\item{identify.check}{Logical parameter specifying if hypotheses should be identified on the output plots.}
}
\examples{# TODO: EXAMPLE PROBLEMS
\dontrun{
\dontrun{Creating several Mutoss class objects}
mu.test.obj.1 <- mutoss.apply(new(Class="Mutoss", 
                                  pValues=runif(10)), 
                                  f=bonferroni, 
                                  label="Bonferroni Correction", 
                                  alpha=0.05, 
                                  silent=T)
mu.test.obj.2 <- mutoss.apply(new(Class="Mutoss", 
                                  pValues=runif(10)), 
                                  f=holm, 
                                  label="Holm's step-down-procedure", 
                                  alpha=0.05,
                                  silent=T)
mu.test.obj.3 <- mutoss.apply(new(Class="Mutoss", 
                                  pValues=runif(10)), 
                                  f=aorc, 
                                  label="Asymtotically optimal rejection curve", 
                                  alpha=0.05, 
                                  startIDX_SUD = 1, 
                                  silent=T)
\dontrun{Trying to coercing a non-Mutoss object}
compareMutoss(1)
\dontrun{ Coercing several objects into a list}
compare.1<- compareMutoss(mu.test.obj.1, mu.test.obj.2)
compare.2<- compareMutoss(mu.test.obj.1, mu.test.obj.2, mu.test.obj.3)
\dontrun{Plotting the adjusted pvalues. Identification available.}
mu.compare.adjusted(compare.1, T)
mu.compare.adjusted(compare.2, T)
\dontrun{Plotting the critical values. Identification available.}
mu.compare.critical(compare.1, T)
mu.compare.critical(compare.2, T)
\dontrun{Showing a textual sumary}
mu.compare.summary(compare.1)
mu.compare.summary(compare.2)
}}

\name{mu.test.type}
\alias{mu.test.type}
\title{Tests that different procedures use the same error types.}
\usage{mu.test.type(types)}
\description{Tests that different procedures use the same error types.}
\details{Internal muToss function.}
\value{Returns a notice if error types differ.}
\author{MuToss-Coding Team.}
\arguments{\item{types}{Character vector of error types extracted from muToss objects.}}

\name{fisher23.marginal}
\alias{fisher23.marginal}
\title{Fisher-type (2x3) table association analysis for calculating all marginal p-values...}
\usage{fisher23.marginal(data, model)}
\description{Fisher-type (2x3) table association analysis for calculating all marginal p-values}
\value{A list containing the m marginal p-values}
\author{ThorstenDickhaus}
\arguments{\item{data}{A tensor of dimension (2x3xm) where m is the number of endpoints (genes, etc.)}
\item{model}{A model object indicating that this type of analysis shall be performed}}

\name{rom}
\alias{rom}
\title{Rom's (1990) step-up-procedure.}
\usage{rom(pValues, alpha, silent=FALSE)}
\description{Rom's step-up-procedure is applied to pValues. The procedure 
controls the FWER in the strong sense if the pValues are
stochastically independent.}
\details{This function calculates the critical values by the formula given
in Finner, H. and Roters, M. (2002) based on the joint distribution
of order statistics. After that a step-up test
is performed to reject hypotheses associated with pValues.

Since the formula for the critical values is recursive,
the calculation of adjusted pValues is far from obvious and is
not implemented here.}
\value{A list containing:
\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected.}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test.}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{Marsel Scheer}
\references{Rom, D. M. (1990). A sequentially rejective test 
procedure based on a modified Bonferroni inequality. 
Biometrika 77, 663-665.

Finner, H. and Roters, M. (2002). Multiple hypotheses testing and
expected type I errors. Ann. Statist. 30, 220-238.}
\arguments{\item{pValues}{pValues to be used. They are assumed to be stochastically independent.}
\item{alpha}{the level at which the FWER shall be controlled.}
\item{silent}{if true any output on the console will be suppressed.}}
\examples{r <- c(runif(50), runif(50, 0, 0.01))
result <- rom(r, 0.05)
result <- rom(r, 0.05, silent = TRUE)}

\name{mu.test.same.data}
\alias{mu.test.same.data}
\title{Tests if the same pvalues were used by different procedures.}
\usage{mu.test.same.data(pvals)}
\description{Tests if the same pvalues were used by different procedures.}
\details{Internal muToss function.}
\value{Stops if different data was used by different procedures.}
\author{MuToss-Coding Team}
\arguments{\item{pvals}{Data frame of p-values used by each procedure.}}

\name{mu.test.name}
\alias{mu.test.name}
\title{Tests if all hypotheses have the same names.}
\usage{mu.test.name(hyp.names)}
\description{Tests if all hypotheses have the same names.}
\details{Internal muToss function.}
\value{Gives a notice when hypotheses names in different procedures are found different.}
\author{MuToss-Coding Team}
\arguments{\item{hyp.names}{Character vector of hypotheses names.}}

\name{holm}
\alias{holm}
\title{Holm's (1979) step-down-procedure}
\usage{holm(pValues, alpha, silent=FALSE)}
\description{Holm's step-down-procedure is applied to pValues. It controls
the FWER in the strong sense under arbitrary dependency.}
\details{Holm's procedure uses the same critical values as Hochberg's procedure, namely  c(i)=alpha/(m-i+1), 
but is a step-down version while Hochberg's method is a step-up version of the Bonferroni test.
Holm's method is based on the Bonferroni inequality and is valid regardless of the joint
distribution of the test statistics, whereas Hochberg's method relies on the assumption that 
Simes' inequality holds for the joint null distribution of the test statistics. If this assumption is met, Hochberg's
step-up procedure is more powerful than Holm's step-down procedure.}
\value{A list containing: 

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-down test}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{MarselScheer}
\references{S. Holm (1979). A simple sequentially rejective multiple 
test procedure. Scand. J. Statist. Vol. 6, 65-70. \eqn{n}

Huang, Y. and Hsu, J. (2007). Hochberg's step-up method: cutting corners off Holm's step-down method. Biometrika, 94(4):965-975.}
\arguments{\item{pValues}{pValues to be used. They can have arbitrary dependency structure.}
\item{alpha}{The level at which the FWER shall be controlled}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{r <- c(runif(50), runif(50, 0, 0.01))
result 	<- holm(r, 0.05)
result 	<- holm(r, 0.05, silent = TRUE)}

\name{SD}
\alias{SD}
\title{A general step-down procedure.}
\usage{SD(pValues, criticalValues)}
\description{A general step-down procedure.}
\details{Suppose we have n pValues and they are already sorted.
The procedure starts with comparing pValues[1] with criticalValues[1]. If 
pValues[1] <= criticalValues[1], then the hypothsis associated with pValues[1] is rejected and the algorithm 
carries on with second smallest pValue and criticalValue in the same way. The algorithm stops
rejecting at the first index i for which pValues[i] > criticalValues[i]. 
Thus pValues[j] is rejected if and only if pValues[i] <= criticalValues[i] for all i <= j.}
\value{rejected logical vector indicating if hypotheses are rejected or retained.}
\author{MarselScheer}
\arguments{\item{pValues}{pValues to be used.}
\item{criticalValues}{criticalValues for the step-down procedure}}

\name{hommel}
\alias{hommel}
\title{Hommel's (1988) step-up-procedure}
\usage{hommel(pValues, alpha, silent=FALSE)}
\description{Hommel's step-up-procedure.}
\details{The method is applied to p-values. It controls
the FWER in the strong sense when the hypothesis tests are independent
or when they are non-negatively associated.

The method is based upon the closure principle and the Simes test.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test.}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}
}
\author{HackNiklas}
\references{
G. Hommel (1988). A stagewise rejective multiple test procedure based on a modified Bonferroni test. Biometrika 75, pp. 383-386}
\arguments{
\item{pValues}{pValues to be used. They need a independent structure.}
\item{alpha}{The level at which the FWER should be controlled}
\item{silent}{Logical. If true, any output on the console will be suppressed.}
}
\examples{
pval <- c(runif(50), runif(50, 0, 0.01))
result 	<- hommel(pval, 0.05)
result 	<- hommel(pval, 0.05, silent = TRUE)
}

\name{ABH_pi0_est}
\alias{ABH_pi0_est}
\title{Lowest Slope Line (LSL) method of Hochberg and Benjamini for estimating pi0}
\usage{ABH_pi0_est(pValues)}
\description{The Lowest Slope Line (LSL) method of Hochberg and Benjamini for estimating pi0 is applied to pValues.
This method for estimating pi0 is motivated by the graphical approach proposed
by Schweder and Spjotvoll (1982), as developed and presented in Hochberg and Benjamini (1990).}
\value{\item{pi0.ABH}{The estimated proportion of true null hypotheses.}}
\author{WerftWiebke}
\references{Hochberg, Y. and Benjamini, Y. (1990). More powerful procedures for multiple significance testing. 
Statistics in Medicine 9, 811-818.

Schweder, T. and Spjotvoll, E. (1982). Plots of P-values to evaluate many tests simultaneously. 
Biometrika 69, 3, 493-502.}
\arguments{\item{pValues}{The raw p-values for the marginal test problems}}
\examples{my.pvals <- c(runif(50), runif(50, 0, 0.01))
result <- ABH_pi0_est(my.pvals)}

\name{mutoss.plotCI}
\alias{mutoss.plotCI}
\title{mutoss.plotCI}
\usage{mutoss.plotCI(mat)}
\description{Plots the confidence intervals}
\arguments{\item{mat}{Matrix containing the confidence interval limits. TODO specify the matrix layout.}}

\name{regwq}
\alias{regwq}
\title{REGWQ - Ryan / Einot and Gabriel / Welsch  test procedure...}
\usage{regwq(formula, data, alpha, MSE=NULL, df=NULL, silent=FALSE)}
\description{REGWQ - Ryan / Einot and Gabriel / Welsch  test procedure
This function computes REGWQ test for given
data including p samples. It is based on a stepwise or 
layer approach to significance testing. Sample means are 
ordered from the smallest to the largest.  
The largest 
difference, which involves means that are r = p steps apart, 
is tested first at \eqn{\alpha} level of significance; if significant, 
means that are \eqn{r <p} steps apart are tested at a different \eqn{\alpha} level 
of significance and so on. Compare to the Student-
Newman-Keuls test, the \eqn{\alpha} levels are adjusted for the p-1 different
layers by the formula \eqn{\alpha_p=\alpha}, if p=k or p=k-1,
\eqn{\alpha_p = 1-(1-\alpha)^{p/k}}  otherwise. It might happen that the
quantiles are not descending in p. In this case, they are adapted by
\eqn{c_k = max_{2\leq r \leq k} c_r, k=2,\ldots,p}.
The REGWQ procedure, like Tukey's procedure, requires 
equal sample n's. However, in this algorithm, the procedure is 
adapted to unequal sample sized which can lead to still 
conservative test decisions.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{statistics}{A numeric vector containing the test-statistics}

\item{confIntervals}{A matrix containing only the estimates}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{Frank Konietschke}
\references{Hochberg, Y. & Tamhane, A. C. (1987). Multiple Comparison Procedures,  Wiley.}
\arguments{\item{formula}{Formula defining the statistical model containing the response and the factors}
\item{data}{dataset containing the response and the grouping factor}
\item{alpha}{The level at which the error should be controlled. By default it is alpha=0.05.}
\item{MSE}{Optional for a given variance of the data}
\item{df}{Optional for a given degree of freedom}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{x = rnorm(50)
grp = c(rep(1:5,10))
dataframe <- data.frame(x,grp)
result <- regwq(x~grp, data=dataframe, alpha=0.05,MSE=NULL, df=NULL, silent = TRUE)
result <- regwq(x~grp, data=dataframe, alpha=0.05,MSE=NULL, df=NULL, silent = FALSE)
result <- regwq(x~grp, data=dataframe, alpha=0.05,MSE=1, df=Inf, silent = FALSE) # known variance
result <- regwq(x~grp, data=dataframe, alpha=0.05,MSE=1, df=1000, silent = FALSE) # known variance}

\name{twostageBR}
\alias{twostageBR}
\title{Blanchard-Roquain (2009) 2-stage adaptive step-up...}
\usage{twostageBR(pValues, alpha, lambda=1, silent=FALSE)}
\description{Blanchard-Roquain (2009) 2-stage adaptive step-up}
\details{This is an adaptive linear step-up procedure where the proportion of true
nulls is estimated using the Blanchard-Roquain 1-stage procedure with parameter lambda,
via the formula

estimated pi_0 = ( m - R(alpha,lambda)  + 1) / ( m*( 1 - lambda * alpha ) )

where R(alpha,lambda) is the number of hypotheses rejected by the BR 1-stage procedure,
alpha is the level at which FDR should be controlled and lambda an 
arbitrary parameter belonging to (0, 1/alpha) with default value 1.
This procedure controls FDR at the desired level when the p-values are independent.}
\value{A list containing:

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{GillesBlanchard}
\references{Blanchard, G. and Roquain, E. (2009) 
Adaptive False Discovery Rate Control under Independence and Dependence
Journal of Machine Learning Research 10:2837-2871.}
\arguments{\item{pValues}{the used p-values (assumed to be independent)}
\item{alpha}{the level at which the FDR should be controlled.}
\item{lambda}{parameter of the procedure, should belong to
(0, 1/alpha) (lambda=1 default)}
\item{silent}{if true any output on the console will be suppressed.}}

\name{mutoss.apply}
\alias{mutoss.apply}
\title{Applies a function to a Mutoss object.}
\usage{mutoss.apply(mutossObj, f, label = deparse(substitute(f)) , recordHistory = TRUE , ...)}
\description{Applies a function to a Mutoss object.}
\details{This functions is intended for applying functions for multiplicity control on Mutoss class objects using the console and not the Mutoss GUI.}
\value{A Mutoss object after applying the given function

\item{mutossObj}{Object of S4 class Mutoss}}
\author{Kornelius Rohmeyer}
\arguments{\item{mutossObj}{the Mutoss object the function should be applied to}
\item{f}{the function that should be applied}
\item{label}{the label affixed to all slots of the Mutoss object that
  are changed by the procedure, defaults to the name of parameter f}
\item{recordHistory}{if true, the calling command is concatenated verbatim
  to the commandHistory slot}
\item{...}{additional parameters that are passed to the function}}
\examples{newObjectBonf <- mutoss.apply(new(Class="Mutoss", pValues=runif(10)), 
  f=bonferroni, label="Bonferroni Correction", alpha=0.05)
\dontrun{ TODO: EXAMPLE PROBLEM
newObjectHolm <- mutoss.apply(new(Class="Mutoss", pValues=runif(10)), 
  f=holm, label="Holm's step-down-procedure", alpha=0.05, silent=T)

newObjectAORC <- mutoss.apply(new(Class="Mutoss", pValues=runif(10)), 
  f=aorc, label="Asymptotically optimal rejection curve", alpha=0.05, startIDX_SUD = 1, silent=T)
}}

\name{adaptiveBH}
\alias{adaptiveBH}
\title{Benjamini-Hochberg (2000) adaptive linear step-up procedure}
\usage{adaptiveBH(pValues, alpha, silent=FALSE)}
\description{The adaptive Benjamini-Hochberg step-up procedure is applied to pValues.
It controls the FDR at level alpha for independent or positive regression dependent test statistics.}
\details{In the adaptive Benjamini-Hochberg step-up procedure the number of true null hypotheses is estimated first as in Hochberg and
Benjamini (1990), and this estimate is used in the procedure of Benjamini and
Hochberg (1995) with alpha'=alpha*m/m0.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test} 

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{pi0}{An estimate of the proportion of true null hypotheses among all hypotheses (pi0=m0/m). }

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{WerftWiebke}
\references{Benjamini, Y. and Hochberg, Y. (2000). On the Adaptive Control of the False Discovery Rate
in Multiple Testing With Independent Statistics.
Journal of Educational and Behavioral Statistics, 25(1): 60-83.\eqn{n}

Hochberg, Y. and Benjamini, Y. (1990). More powerful procedures for multiple significance testing. 
Statistics in Medicine 9, 811-818.\eqn{n}

Benjamini, Y. and Hochberg, Y. (1995). Controlling the false discovery rate: A practical and powerful approach to mulitple testing.
Journal of the Royal Statistical Society, Series B, 57:289-300.}
\arguments{\item{pValues}{The used raw pValues.}
\item{alpha}{The level at which the FDR shall be controlled.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9, max=1))
result <- adaptiveBH(p, alpha)
result <- adaptiveBH(p, alpha, silent=TRUE)}

\name{ranktruncated}
\alias{ranktruncated}
\title{Rank truncated p-Value procedure...}
\usage{ranktruncated(pValues, K, silent=FALSE)}
\description{Rank truncated p-Value procedure
The program computes the exact distribution and with it the p-Value}
\details{This function computes the exact distribution of the product of
at most K significant p-values of \eqn{L>K} observed p-values. Thus, one gets the
pvalue from the exact distribution. This has certain advantages for genomewide 
association scans: K can be chosen on the basis of a hypothesised 
disease model, and is independent of sample size. Furthermore, 
the alternative hypothesis corresponds more closely to the 
experimental situation where all loci have fixed effects.

Please note that this method is implemented with factorials and binomial coefficients and the computation becomes numerical instable for large number of p-values.}
\value{Used.pValue: List information about the used pValues; RTP: Test statistic and pValue}
\author{Frank Konietschke}
\references{Dubridge, F., Koeleman, B.P.C. (2003). Rank truncated product of P-values, with application to genomewide association scans. Genet Epidemiol. 2003 Dec;25(4):360-6}
\arguments{\item{pValues}{Vector of p-Values (not sorted)}
\item{K}{the number of hypotheses / p-Values being in w}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{pvalues<-runif(10)
result <- ranktruncated(pvalues,K=2,silent=FALSE) # take the K=2 smallest pvalues
result <- ranktruncated(pvalues,K=2,silent=TRUE) # take the K=2 smallest pvalues
result <- ranktruncated(pvalues,K=5,silent=TRUE) # take the K=5 smallest pvalues}

\name{printRejected}
\alias{printRejected}
\title{Internal MuTossProjekt-Function}
\usage{printRejected(rejected, pValues=NULL, adjPValues=NULL)}
\description{Generates standard output for pValues, rejected and
adjustedPValues.}
\details{It generates an output on the console with the number of
hypotheses (number of pValues) and the number of rejected
hypotheses (number of rejected pValues). Further a data.frame
is constructed, one column containing the rejected pValues, 
one the index number of the rejected pValues and if given 
one column with the corresponding adjusted pValues.}
\author{MarselScheer}
\arguments{\item{rejected}{logical Vector indicating which pValue is rejected.}
\item{pValues}{the used pValues.}
\item{adjPValues}{the adjusted pValues.}}

\name{TSBKY_pi0_est}
\alias{TSBKY_pi0_est}
\title{Two-step estimation method of Benjamini, Krieger and Yekutieli for estimating pi0}
\usage{TSBKY_pi0_est(pValues, alpha)}
\description{The two-step estimation method of Benjamini, Krieger and Yekutieli for estimating pi0 is applied to pValues.
It consists of the following two steps:
Step 1. Use the linear step-up procedure at level alpha' =alpha/(1+alpha). Let r1 be the number of
rejected hypotheses. If r1=0 do not reject any hypothesis and stop; if r1=m reject all m
hypotheses and stop; otherwise continue.
Step 2. Let \eqn{\hat{m0} =(m - r1)} and \eqn{\hat{pi0} = \hat{m0} / m}.}
\value{\item{pi0.TSBKY}{The estimated proportion of true null hypotheses.}}
\author{WerftWiebke}
\references{Benjamini, Y., Krieger, A. and Yekutieli, D. (2006). Adaptive linear step-up procedures that control the false discovery rate
Biometrika 93, 3, page 495.}
\arguments{\item{pValues}{The raw p-values for the marginal test problems}
\item{alpha}{The parameter (to be interpreted as significance level) for the procedure}}
\examples{my.pvals <- c(runif(50), runif(50, 0, 0.01))
result <- TSBKY_pi0_est(my.pvals, 0.1)}

\name{ErrorControl-class}
\docType{class}
\alias{ErrorControl-class}
\title{Class ErrorControl}
\description{This class holds the information related to the error rate}
\section{Slots}{
  \describe{
    \item{\code{type}:}{a character specifying the error rate. For example FWER, FDR, ... }
    \item{\code{alpha}:}{the level at which the error rate shall be controlled}
    \item{\code{k}:}{an additional parameter for generalised FWER}
    \item{\code{q}:}{an additional parameter for FDX (false discovery exceedance)}
  }
}
\author{MuToss-Coding Team}

\name{Qvalue}
\alias{Qvalue}
\title{Storey's (2001) q-value Procedure...}
\usage{Qvalue(pValues, lambda=seq(0, 0.9, 0.05), pi0.method="smoother",
    fdr.level=NULL, robust=FALSE, smooth.df=3, smooth.log.pi0=FALSE,
    silent=FALSE)}
\description{Storey's (2001) q-value Procedure}
\details{The Qvalue procedure estimates the q-values for a given set of p-values. The q-value of a test measures the
proportion of false positive incurred when that particular test is called sigificant.
It gives the scientist a hypothesis testing error measure for each observed statistic with respect to the pFDR.

Note: If no options are selected, then the method used to estimate pi0 is the smoother method desribed in Storey and Tibshirani (2003). 
The bootstrap method is described in Storey, Taylor and Siegmund (2004).}
\value{A list containing:

\item{qValues}{A vector of the estimated q-values}

\item{pi0}{An estimate of the proportion of null hypotheses} 

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{HackNiklas}
\references{Storey, John (2001). The Positive False Discovery Rate: A Baysian Interpretation and the Q-Value.
The Annals of Statistics, Vol. 31, No. 6, 2013-2035.}
\arguments{\item{pValues}{pValues to be used (only necessary input)}
\item{lambda}{Value of the tuning parameter to be used}
\item{pi0.method}{Method for automatically choosing tuning parameter in the estimation of pi_0. Either 'smoother' or 'bootstrap'}
\item{fdr.level}{Level at which to control the FDR}
\item{robust}{Logical, whether to make estimate more robust for small p-values.}
\item{smooth.df}{Number of degrees of freedom to use when estimating pi_0 with the smoother.}
\item{smooth.log.pi0}{Logical, if TRUE and pi0.method = 'smoother', pi0 will be estimated by applying a smoother 
to a scatterplot of log(pi_0) estimates against the tuning parameter lambda.}
\item{silent}{logical scalar. If \code{TRUE} no output is generated.}}
\examples{pval <- c(runif(50), runif(50, 0, 0.01))
result <- Qvalue(pval)
result <- Qvalue(pval, lambda=0.5)}

\name{notterman}
\docType{data}
\alias{notterman}
\alias{notterman.grpLabel}
\alias{T.Test.tumor.vs.normal}
\title{Notterman data set}
\description{
The \code{notterman} data set is a data.frame containing 18 paired samples of 7457 gene expression values from Notterman et al. (2001).

The vector \code{notterman.grpLabel} contains 36 labels of type \code{Tumor} or \code{Normal} specifying the type of tissue for each column of the \code{notterman} data set.

The vector \code{T.Test.tumor.vs.normal} contains the resulting 7457 numeric p-values from the 7457 t-tests applied to each row of the data set.
}
\usage{notterman
notterman.grpLabel
T.Test.tumor.vs.normal
}
\format{
\itemize{
		\item{notterman - A data.frame containing 36 columns with 7457 observations}
		\item{notterman.grpLabel - A vector containing 36 labels of type \code{Tumor} or \code{Normal}}
		\item{T.Test.tumor.vs.normal - A vector containing 7457 numeric p-values}
	}
}
\source{D.A. Notterman, U. Alon, A.J. Sierk, and A.J. Levine:
\emph{Transcriptional Gene Expression Profiles of Colorectal
Adenoma, Adenocarcinoma, and Normal Tissue Examined by
Oligonucleotide Arrays}, Cancer Research, 2001, vol. 61, pp.
3124-3130.}
\keyword{datasets}

\name{mu.test.rates}
\alias{mu.test.rates}
\title{Tests that different procedures used the same error rates.}
\usage{mu.test.rates(rates)}
\description{Tests that different procedures used the same error rates.}
\details{Internal muToss function.}
\value{A notice if error rates differ.}
\author{MuToss-Coding Team.}
\arguments{\item{rates}{Numeric vector error rates extracted fomr muToss objects.}}

\name{calculateBetaAdjustment}
\alias{calculateBetaAdjustment}
\title{Calculating the beta adjustment factor for the asymptotically optimal rejection curve.}
\usage{calculateBetaAdjustment(n, startIDX_SUD, alpha, silent=FALSE,
    initialBeta=1, maxBinarySteps=50, tolerance=1e-04)}
\description{Calculates the beta to adjust the asymptotically optimal rejection curve
used by the function aorc() for a finite sample size. Then  
aorc(..., betaAdjustment = beta) controls the FDR also in the
finite sample situation.}
\details{The asymptotically optimal rejection curve, denoted by f(t), does not 
provide finite control of the FDR. calculateBetaAdjustment() calculates
a factor, denoted by beta, such that (1 + beta/n) * f(t) provides
finite control of the FDR.

The beta is calculated with the bisection approach. Assume there are beta1
and beta2 such that the choosing beta1 controls the FDR and beta2 not, then the 
optimal beta lies in [beta2, beta1]. If the choice (beta2 + beta1)/2 controls
the FDR, the optimal FDR lies in [(beta2 + beta1)/2, beta1]
and so on.}
\value{The adjustment factor that is needed to ensure control of
the FDR with the adjusted asymptotically optimal rejection curve
at the specified level and sample size.}
\author{MarselScheer}
\arguments{\item{n}{Number of tests for which the adjusted beta should be calculated.}
\item{startIDX_SUD}{Starting index of the step-up-down procedure}
\item{alpha}{The level at which the FDR shall be controlled.}
\item{silent}{If true any output on the console will be suppressed.}
\item{initialBeta}{Initial beta.}
\item{maxBinarySteps}{Maximum number of steps that will be performed.}
\item{tolerance}{The tolerance to search for an upper FDR bound element in [alpha - tolerance, alpha]}}

\name{BlaRoq}
\alias{BlaRoq}
\title{Blanchard-Roquain (2008) step-up Procedure for arbitrary dependent p-Values...}
\usage{BlaRoq(pValues, alpha, pii, silent=FALSE)}
\description{Blanchard-Roquain (2008) step-up Procedure for arbitrary dependent p-Values
Also proposed independently by Sarkar (2008)}
\details{A generalization of the Benjamini-Yekutieli procedure, taking as an additional parameter
a distribution pii on [1..k] (k is the number of hypotheses)
representing prior belief on the number of hypotheses that will be rejected.

It is a step-up Procedure with critical values C_i defined as alpha/k times
the sum for j in [1..i] of j*pii[j]. For any fixed prior pii, the FDR is controlled at
level alpha for arbitrary dependence structure of the p-Values. The particular case of the
Benjamini-Yekutieli step-up is recovered by taking pii[i] proportional to 1/i.

If pii is missing, a default prior distribution proportional to exp( -i/(0.15*k) ) is taken.
It should perform better than the BY procedure if more than about 0.05 to 0.1 of hypotheses are rejected,
and worse otherwise.

Note: the procedure automatically normalizes the prior pii to sum to one if this is not the case.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up test} 

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{GillesBlanchard,HackNiklas}
\references{Blanchard, G. and Roquain, E. (2008). Two simple sufficient conditions for FDR control.
Electronic Journal of Statistics, 2:963-992. 
Sarkar, S.K. (2008) On methods controlling the false discovery rate. 
Sankhya, Series A, 70:135-168.}
\arguments{\item{pValues}{pValues to be used. They can have arbitrary dependence.}
\item{alpha}{the level at which the FDR should be controlled}
\item{pii}{Prior for the proportion of true null hypotheses, same size as pValues}
\item{silent}{if true any output on the console will be suppressed.}}

\name{adjPValuesPlot}
\alias{adjPValuesPlot}
\title{A function plotting adjusted p-values...}
\usage{adjPValuesPlot(adjPValues, alpha)}
\description{A function plotting adjusted p-values}
\author{MuToss-Coding Team}
\arguments{
\item{adjPValues}{A numeric containing the adjusted pValues to plot.}
\item{alpha}{A numeric containing the overall type I error rate alpha..}
}

\name{hochberg}
\alias{hochberg}
\title{Hochberg (1988) step-up procedure}
\usage{hochberg(pValues, alpha, silent=FALSE)}
\description{The Hochberg step-up procedure is based on marginal p-values. It controls the FWER in the strong 
sense under joint null distributions of the test statistics that satisfy Simes' inequality.}
\details{The Hochberg procedure is more powerful than Holm's (1979) procedure, but the test statistics need to be
independent or have a distribution with multivariate total positivity of order two or a scale mixture
thereof for its validity (Sarkar, 1998).
Both procedures use the same set of critical values c(i)=alpha/(m-i+1). Whereas Holm's procedure is a step-down 
version of the Bonferroni test, and Hochberg's is a step-up version of the Bonferroni test.
Note that Holm's method is based on the Bonferroni inequality and is valid regardless of the joint
distribution of the test statistics.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{WerftWiebke}
\references{Hochberg, Y. (1988). A sharper Bonferroni procedure for multiple tests of significance. 
Biometrika, 75:800-802.\eqn{n}

Huang, Y. and Hsu, J. (2007). Hochberg's step-up method: cutting corners off Holm's step-down method. Biometrika, 94(4):965-975.}
\arguments{\item{pValues}{The used raw pValues.}
\item{alpha}{The level at which the FDR shall be controlled.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9,max=1))
result <- hochberg(p, alpha)
result <- hochberg(p, alpha, silent=TRUE)}

\name{SUD}
\alias{SUD}
\title{A general step-up-down procedure.}
\usage{SUD(pValues, criticalValues, startIDX_SUD)}
\description{A general step-up-down procedure.}
\details{Suppose we have n pValues and they are already sorted.
The procedure compares pValues[startIDX_SUD] with criticalValues[startIDX_SUD]
and then proceeds in step-up or step-down manner,   
depending on the result of this initial comparision.

If pValues[startIDX_SUD] <= criticalValues[startIDX_SUD], then the procedure
rejects the hypotheses associated with pValues[1], ..., pValues[startIDX_SUD]  
and carries on in a step-down manner from startIDX_SUD to n to reject additional hypotheses.

If pValues[startIDX_SUD] > criticalValues[startIDX_SUD], then the procedure
retains hypotheses associated with pValues[startIDX_SUD], ..., pValues[n] and 
carries on in a step-up manner with pValues[startIDX_SUD - 1], ..., pValues[1]. 

If startIDX_SUD equals n the algorithm behaves like a step-up procedure.

If startIDX_SUD equals 1 the algorithm behaves like a step-down procedure.}
\value{rejected logical vector indicating if hypotheses are rejected or retained.}
\author{MuToss-Coding Team}
\arguments{\item{pValues}{pValues to be used.}
\item{criticalValues}{criticalValues for the step-up-down procedure}
\item{startIDX_SUD}{the index (between 1 and length(pValues)) used for the first 
comparison of pValues[startIDX_SUD] and criticalValues[startIDX_SUD]. Depending
on the result of this comparison the algorithm decides to proceed in 
step-up or step-down manner.}}

\name{mutoss.models}
\alias{fisher23.model}
\alias{fisher22.model}
\alias{twosamp.model}
\alias{paired.model}
\alias{onesamp.model}
\alias{ftest.model}
\title{Mutoss Models}
\description{Fisher-type (2x3) table as model for marginal hypotheses testing problems,
Fisher (2x2) table as model for marginal hypotheses testing problems and others...}
\value{A list containing the model description}

\name{augmentation}
\alias{augmentation}
\title{Wrapper function to the augmentation methods of the multtest package.}
\usage{augmentation(adjPValues, newErrorControl, newK, newQ, silent=FALSE)}
\description{Wrapper function to the augmentation methods of the multtest package.}
\details{The augmentation method turns a vector of p-values which are already adjusted for FWER control
into p-values that are adjusted for gFWER, FDX or FDR. The underlying idea (for gFWER and FDX) 
is that the set of hypotheses rejected at a given level alpha under FWER can be "augmented"
by rejecting some additional hypotheses while still ensuring (strong) control of the desired weaker type I criterion.
For FDR, it uses the fact that FDX control for q=alpha=1-sqrt(1-beta) entails FDR control at level beta.

Use of these augmentation methods is recommended only in the situation where FWER-controlled p-values are
directly available from the data (using some specific method). When only marginal p-values are available,
it is generally prerefable to use other adjustment methods directly aimed at the intended criterion
(as opposed to first adjust for FWER, then augment)

Note: in the multtest package, two methods ("restricted" and "conservative") are available for FDR augmentation. 
Here the 'restricted' method is forced for FDR augmentation since it is in fact always valid and better than 
"conservative" (M. van der Laan, personal communication) with respect to power.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the new adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected. Currently always NULL.}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{KornRohm, GillesBlanchard}
\references{Dudoit, S. and van der Laan, M.J. (2008)
Multiple Testing Procedures with Applications to Genomics, Springer. (chapter 6)}
\arguments{\item{adjPValues}{a vector of p-values that are *already* adjusted for FWER}
\item{newErrorControl}{new error control type to adjust for. One of ("gFWER", "FDX", "FDR")}
\item{newK}{k-parameter if newErrorControl is "gFWER"}
\item{newQ}{q-parameter if newErrorControl is "FDX"}
\item{silent}{if true any output on the console will be suppressed.}}
\examples{\dontrun{ TODO NH (MS) EXAMPLE PROBLEM
p <- c(runif(50), runif(50, 0, 0.01))
fwer_adj <- bonferroni(p)    # adjust for FWER using Bonferroni correction
fdx_adj  <- augmentation(fwer_adj$adjPValues , "FDX", q=0.1, silent = TRUE) #augment to FDX (q=0.1)
}}

\name{tukey.wrapper}
\alias{tukey.wrapper}
\title{Tukey HSD test and simultaneous confidence intervals for all pairs comparisons...}
\usage{tukey.wrapper(model, alpha, factorC)}
\description{Tukey HSD test and simultaneous confidence intervals for all pairs comparisons
in factorial designs. The procedure controls the FWER in the strong sense.}
\details{this function, it is possible to compute all pairs comparisons for expectations and 
simultaneous confidence intervals in factorial linear models. Hereby, the all-pairs comparisons
can be performed for user given effects. The overall variance is estimated by the linear model
as well as the degree of freedom used by the studentized range distribution.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{confIntervals}{A matrix containing the estimates and the lower and upper confidence bound}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{Frank Konietschke et al.}
\arguments{\item{model}{A fitted model, for example an object returned by lm, glm, or aov etc. It is
assumed that coef and vcov methods are available for model. Usually, it is an aov fit}
\item{alpha}{The significance level}
\item{factorC}{Specifies a factor}}
\examples{data(warpbreaks)
# Tukey contrast on the levels of the factor "Tension"

tukey.wrapper(aov(breaks ~ tension, data = warpbreaks), factorC="tension",alpha=0.05)


# Two-way anova with interaction
tukey.wrapper(aov(breaks ~ tension*wool, data = warpbreaks),alpha=0.05,factorC="tension")
# Two-way anova without interaction

tukey.wrapper(aov(breaks ~ tension+wool, data = warpbreaks),alpha=0.05,factorC="tension")
tukey.wrapper(aov(breaks ~ tension, data = warpbreaks),alpha=0.05,factorC="tension")


data(iris)
tukey.wrapper(lm(Sepal.Length ~ Species, data=iris),alpha=0.05, factorC="Species")}

\name{SU}
\alias{SU}
\title{A general step-up procedure.}
\usage{SU(pValues, criticalValues)}
\description{A general step-up procedure.}
\details{Suppose we have n pValues and they are already sorted. 
The procedure starts with comparing pValues[n] with criticalValues[n]. If
pValues[n] > criticalValues[n], then the hypothesis associated with pValues[n] 
is retained and the algorithm carries on with the next pValue and criticalValue, here for example pValues[n-1]
and criticalValues[n-1]. The algorithm stops retaining at the first index i for
which pValues[i] <= criticalValues[i]. Thus pValues[j] is rejected if and only if
their exists an index i with j <= i and pValues[i] <= criticalValues[i].}
\value{rejected logical vector indicating if hypotheses are rejected or retained.}
\author{MarselScheer}
\arguments{\item{pValues}{pValues to be used.}
\item{criticalValues}{criticalValues for the step-up procedure}}

\name{jointCDF.orderedUnif}
\alias{jointCDF.orderedUnif}
\title{Joint cumulative distribution function of order statistics of n iid. U(0,1)-distributed random variables}
\usage{jointCDF.orderedUnif(vec)}
\description{Calculates the joint cumulative distribution function of order statistics 
of n iid. U(0,1)-distributed random variables at argument vec. 
Because of numerical issues n should not be greater than 100.}
\details{Following Shorack, Wellner (1986) or Finner, Roters (2002) by applying
Bolshev's recursion the joint distribution is calculated.}
\value{The return value is the following probability 
P(U_(1:n) <= vec[1], ..., U_(n:n) <= vec[n]), where
U_1, ..., U_n are assumed to be iid. uniformly 
distributed on [0,1]. The i-th ordered value is denoted by
U_(i:n) and n equals length(vec)}
\references{Shorack, G. R. and Wellner, J. A. (1986). 
Empirical Processes with Applications to Statistics.
Wiley, New York.

Finner, H. and Roters, M. (2002). Multiple hypotheses testing and
expected type I errors. Ann. Statist. 30, 220-238.}
\author{MarselScheer}
\arguments{\item{vec}{a numeric vector. The length of the vector also
determines the number of random variables considered.}}

\name{storey_pi0_est}
\alias{storey_pi0_est}
\title{Storey-Taylor-Siegmund estimation of pi0 (finite sample version)}
\usage{storey_pi0_est(pValues, lambda)}
\description{The Storey-Taylor-Siegmund procedure for estimating pi0 is applied to pValues.
The formula is equivalent to that in Schweder and Spjotvoll (1982),
page 497, except the additional '+1' in the nominator that
introduces a conservative bias which is proven to be sufficiently large
for FDR control in finite families of hypotheses if the estimation
is used for adjusting the nominal level of a linear step-up test.}
\value{A list containing:

\item{pi0}{A numeric number containing the estimated value of pi0}

\item{lambda}{A numeric number containing the tuning parameter for the estimation}}
\author{MarselScheer}
\references{Schweder, T. and Spjotvoll, E. (1982). Plots of P-values to evaluate many tests simultaneously. 
Biometrika 69, 3, 493-502.

Storey, J. D., Taylor, J. E. and Siegmund, D. (2004). Strong control, conservative point estimation and
simultaneous conservative consistency of false discovery rates: a unified approach. JRSS B 66, 1, 187-205.}
\arguments{\item{pValues}{The raw p-values for the marginal test problems}
\item{lambda}{A tuning parameter in the interval (0, 1)}}
\examples{my.pvals <- c(runif(50), runif(50, 0, 0.01))
result <- storey_pi0_est(my.pvals, 0.5)}

\name{nparcomp}
\alias{nparcomp}
\title{Simultaneous confidence intervals for relative contrast effects...}
\usage{nparcomp(formula, data, type=c("UserDefined", "Tukey", "Dunnett",
    "Sequen", "Williams", "Changepoint", "AVE", "McDermott", "Marcus",
    "UmbrellaWilliams"), control=NULL, conflevel=0.95,
    alternative=c("two.sided", "less", "greater"), rounds=3,
    correlation=FALSE, asy.method=c("logit", "probit", "normal",
    "mult.t"), plot.simci=FALSE, info=TRUE, contrastMatrix=NULL)}
\description{Simultaneous confidence intervals for relative contrast effects
The procedure controls the FWER in the strong sense.}
\details{With this function, it is possible to compute nonparametric simultaneous confidence
intervals for relative contrast effects in the unbalanced one way layout. Moreover, it computes
adjusted p-values. The simultaneous confidence intervals can be computed using
multivariate normal distribution, multivariate t-distribution with a Satterthwaite Approximation
of the degree of freedom or using multivariate range preserving transformations with Logit or
Probit as transformation function. There is no assumption on the underlying distribution function, only
that the data have to be at least ordinal numbers}
\value{A list containing:  
\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{confIntervals}{A matrix containing the estimates and the lower and upper confidence bound}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{FrankKonietschke}
\arguments{\item{formula}{A two-sided 'formula' specifying a numeric response variable and
a factor with more than two levels. If the factor contains less than 3 levels, an
error message will be returned}
\item{data}{data A dataframe containing the variables specified in formula}
\item{type}{type Character string defining the type of contrast. It should be one of "Tukey",
"Dunnett", "Sequen", "Williams", "Changepoint", "AVE", "McDermott", "Marcus"}
\item{control}{control Character string defining the control group in Dunnett comparisons.
By default it is the first group by lexicographical ordering}
\item{conflevel}{The confidence level for the 1 - conflevel confidence intervals. By
default it is 0.05}
\item{alternative}{Character string defining the alternative hypothesis, one of
"two.sided", "less" or "greater"}
\item{rounds}{Number of rounds for the numeric values of the output. By default it is rounds=3}
\item{correlation}{Correlation A logical whether the estimated correlation matrix and covariance
matrix should be printed}
\item{asy.method}{asy.method character string defining the asymptotic approximation method,
one of "logit", for using the logit transformation function, "probit", for using
the probit transformation function, "normal", for using the multivariate normal
distribution or "mult.t" for using a multivariate t-distribution with a Satterthwaite
Approximation}
\item{plot.simci}{plot.simci A logical indicating whether you want a plot of the confidence
intervals}
\item{info}{info A logical whether you want a brief overview with informations about the
output}
\item{contrastMatrix}{arbitrary contrast matrix given by the user}}
\examples{
\dontrun{# TODO Check this example and set a seed!
grp <- rep(1:5,10)
x <- rnorm(50, grp)
dataframe <- data.frame(x,grp)
# Williams Contrast
nparcomp(x ~grp, data=dataframe, asy.method = "probit",
type = "Williams", alternative = "two.sided", plot.simci = TRUE, info = TRUE)

# Dunnett Contrast
nparcomp(x ~grp, data=dataframe, asy.method = "probit",control=1,
type = "Dunnett", alternative = "two.sided", plot.simci = TRUE, info = TRUE)

# Dunnett dose 3 is baseline
nparcomp(x ~grp, data=dataframe, asy.method = "probit",
type = "Dunnett", control = "3",alternative = "two.sided",
plot.simci = TRUE, info = TRUE)
}
}

\name{fisher22.marginal}
\alias{fisher22.marginal}
\title{Fisher (2x2) table association analysis for calculating all marginal p-values...}
\usage{fisher22.marginal(data, model)}
\description{Fisher (2x2) table association analysis for calculating all marginal p-values}
\value{A list containing the m marginal p-values}
\author{ThorstenDickhaus}
\arguments{\item{data}{A tensor of dimension (2x2xm) where m is the number of endpoints (genes, etc.)}
\item{model}{A model object indicating that this type of analysis shall be performed}}

\name{fisher22_fast}
\alias{fisher22_fast}
\title{Fisher (2x2) table association analysis for calculating one (marginal) p-value...}
\usage{fisher22_fast(obs, epsilon)}
\description{Fisher (2x2) table association analysis for calculating one (marginal) p-value}
\value{A list containing:

\item{nonrand_p}{The non-randomized (conservative) p-value}

\item{rand_p}{The randomized (non-conservative) p-value}

\item{prob_table}{The conditional probability of the observed table (given the merginals)}}
\author{ThorstenDickhaus}
\arguments{\item{obs}{The observed (2x2) table}
\item{epsilon}{A threshold for comparing real numbers to zero}}

\name{multiple.down.adjust}
\alias{multiple.down.adjust}
\title{A service function used by multiple...}
\usage{multiple.down.adjust(sorted, m)}
\description{A service function used by \code{multiple.down}}
\author{JonathanRosenblatt}
\arguments{
\item{sorted}{A sorted pvalue vector.}
\item{m}{The number of hypotheses tested.}
}

\name{nparcomp.wrapper}
\alias{nparcomp.wrapper}
\title{Simultaneous confidence intervals for relative contrast effects...}
\usage{nparcomp.wrapper(model, data, hypotheses, alpha, alternative,
    asy.method)}
\description{Simultaneous confidence intervals for relative contrast effects
The procedure controls the FWER in the strong sense.}
\details{With this function, it is possible to compute nonparametric simultaneous confidence
intervals for relative contrast effects in the unbalanced one way layout. Moreover, it computes
adjusted p-values. The simultaneous confidence intervals can be computed using
multivariate normal distribution, multivariate t-distribution with a Satterthwaite Approximation
of the degree of freedom or using multivariate range preserving transformations with Logit or
Probit as transformation function. There is no assumption on the underlying distribution function, only
that the data have to be at least ordinal numbers}
\value{A list containing:  
\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{confIntervals}{A matrix containing the estimates and the lower and upper confidence bound}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\author{FrankKonietschke}
\arguments{\item{model}{A two-sided formula specifying a numeric response variable and
a factor with more than two levels.}
\item{data}{A dataframe containing the variables specified the model}
\item{hypotheses}{Character string defining the type of contrast. It should be one of "Tukey",
"Dunnett", "Sequen", "Williams", "Changepoint", "AVE", "McDermott", "Marcus".}
\item{alpha}{the significance level}
\item{alternative}{Character string defining the alternative hypothesis, one of
"two.sided", "less" or "greater"}
\item{asy.method}{A character string defining the asymptotic approximation method,
one of "logit", for using the logit transformation function, "probit", for using
the probit transformation function, "normal", for using the multivariate normal}}

\name{gatherStatistics}
\alias{gatherStatistics}
\title{Gathering statistics from simulation()-Object}
\usage{gatherStatistics(simObject, listOfStatisticFunctions,
    listOfAvgFunctions)}
\description{This function facilitates gathering statistics from the object returned by simulation().}
\details{For every simulation()-Object in simObject$results all statistics in listOfStatisticFunctions are calculated. 
If in addition listOfAvgFunctions is provided then the statistics of the objects that have the same
parameter constellation are averaged by the functions in listOfAvgFunctions. 
The resulting data.frame will then keep only one row for every parameter constellation.}
\value{\item{statisticDF}{A data.frame that can be of two different kinds. 

If listOfAvgFunctions is provided, then the resulting data.frame will have a row for every 
parameter constellation that occurs in the simObject$results. There will be columns for every
parameter used. Also length(listOfAvgFunctions) * length(listOfStatisticFunctions) additional
columns will be created. Every statistic is calculated for every applied procedure and then
all values that belong to a specific parameter constellation are "averaged" by applying all 
function from listOfAvgFunctions. For example, suppose FDP is a function from listOfStatisticFunctions
that calculates the realized false discovery proportion, that is number of true hypotheses that
were rejected divided by the number of all rejected hypotheses. Also suppose mean and sd are 
functions in listOfAvgFunctions, then the mean and the standard deviation of the 
statistic FDP are calculated.  

If listOfAvgFunctions is not provided, then the resulting data.frame will have a row for every
simObject$results. Every parameter will have its own column. Additional columns for every function
in listOfStatisticFunction will be created.}}
\author{MarselScheer}
\arguments{\item{simObject}{An object returned by simulation()}
\item{listOfStatisticFunctions}{List of statistics that shall be calculated for the elements in simObject}
\item{listOfAvgFunctions}{List of functions that will be used to summarize/average the calculated statistics for
all elements in simObject$results with the same parameter constellation. If this is argument is missing no averaging
will be done. Instead the resulting data.frame will keep one row
for every object in simObject$results.}}
\examples{#' # this function generates pValues 
myGen <- function(n, n0) {
  list(procInput=list(pValues = c(runif(n-n0, 0, 0.01), runif(n0))), 
       groundTruth = c(rep(FALSE, times=n-n0), rep(TRUE, times=n0)))
}

# need some simulation()-Object I can work with 
sim <- simulation(replications = 10, list(funName="myGen", fun=myGen, n=200, n0=c(50,100)), 
list(list(funName="BH", fun=function(pValues, alpha) BH(pValues, alpha, silent=TRUE), 
  alpha=c(0.25, 0.5)),
list(funName="holm", fun=holm, alpha=c(0.25, 0.5),silent=TRUE)))

# Make my own statistic function
NumberOfType1Error <- function(data, result) sum(data$groundTruth * result$rejected)

# Get now for every object in sim$results one row with the number of Type 1 Errors
result.all <- gatherStatistics(sim, list(NumOfType1Err = NumberOfType1Error))

# Average over all sim$results-Objects with common parameters 
result1 <- gatherStatistics(sim, list(NumOfType1Err = NumberOfType1Error), list(MEAN = mean))
print(result1)  
result2 <- gatherStatistics(sim, list(NumOfType1Err = NumberOfType1Error), 
             list(q05 = function(x) quantile(x, probs=0.05), 
             MEAN = mean, q95 = function(x) quantile(x, probs=0.95)))
print(result2)

# create some plots
require(lattice)
histogram(~NumOfType1Err | method*alpha, data = result.all$statisticDF)
barchart(NumOfType1Err.MEAN ~ method | alpha, data = result2$statisticDF)}

\name{reject}
\alias{reject}
\title{reject}
\usage{reject(sorted, criticals)}
\description{
	Returns the highest rejected p-value and its index given some critical values.
}
\arguments{
  \item{sorted}{
	Sorted p-values
  }
  \item{criticals}{
  	Critical values
  }
}
\value{
	A list with elements
\item{cutoff}{highest rejected p-value}
\item{cut.index }{index of highest rejected p-value}
}

\name{BH}
\alias{BH}
\title{Benjamini-Hochberg (1995) linear step-up procedure}
\usage{BH(pValues, alpha, silent=FALSE)}
\description{Benjamini-Hochbergs Linear Step-Up Procedure. 
The procedure controls the FDR when the test statistics are stochastically independent or satisfy positive regression dependency (PRDS) (see Benjamini and Yekutieli 2001 for details).
The Benjamini-Hochberg (BH) step-up procedure considers ordered pValues P_(i). 
It defines k as the largest i for which P_(i) <= i*alpha/m and then 
rejects all associated hypotheses H_(i) for i=1,...,k. In their seminal paper, Benjamini and Hochberg (1995) show 
that for 0 <= m_0 <= m independent pValues corresponding to true null hypotheses 
and for any joint distribution of the m_1 = m-m_0 p-values corresponding to the 
non null hypotheses, the FDR is controlled at level (m_0/m)*alpha.  Under the
assumption of the PRDS property, (for details see Benjamini and Yekutieli 2001). 
In Benjamini et al. (2006) the BH procedure is improved by adaptive procedures
which use an estimate of m_0 and apply the BH method al level alpha'=alpha*m/m_0, to 
fully exhaust the desired level alpha (see Adaptive Benjamini Hochberg and Two Stage Banjamini Yekutieli).}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{criticalValues}{A numeric vector containing critical values used in the step-up test} 

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{Werft Wiebke}
\references{Benjamini, Y. and Hochberg, Y. (1995). Controlling the false discovery rate: A practical and powerful approach to mulitple testing.
Journal of the Royal Statistical Society, Series B, 57:289-300.\eqn{n}

Benjamini, Y. and Yekutieli, D. (2001). The control of the false discovery rate in multiple testing under dependency.
Annals of Statistics, 29(4):1165-1188.\eqn{n}

Benjamini, Y., Krieger, A. and Yekutieli, D. (2006). Adaptive linear step-up procedures that control the false
discovery rate. Biometrika, 93(3):491-507.}
\arguments{\item{pValues}{The used unadjusted pValues.}
\item{alpha}{The level at which the FDR shall be controlled.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9, max=1))
result <- BH(p, alpha)
result <- BH(p, alpha, silent=TRUE)}

\name{multiple.down}
\alias{multiple.down}
\title{Benjamini-Krieger-Yekutieli (2006) Multi-Stage Step-Down}
\usage{multiple.down(pValues, alpha)}
\description{A p-value procedure which controls the FDR for independent test statistics.}
\details{A non-linear step-down p-value procedure which control the FDR for independent test 
statistics and enjoys more power then other non-adaptive procedure such as the linear step-up (BH).
For the case of non-independent test statistics, non-adaptive procedures such as the 
linear step-up (BH) or the all-purpose conservative Benjamini-Yekutieli (2001) are recommended.}
\value{A list containing:
\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test.} 

\item{adjPValues}{A numeric vector containing adjusted p-values.}

\item{pi0}{An estimate of the proportion of true null hypotheses among all hypotheses (pi0=m0/m). }

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{Jonathan Rosenblatt}
\arguments{\item{pValues}{A numeric vector of p-values}
\item{alpha}{The FDR error rate to control}}
\examples{pvals<- runif(100)^2
alpha<- 0.2
result<- multiple.down(pvals, alpha)
result 
plot(result[['criticalValues']]~pvals)
plot(result[['adjPValues']]~pvals)
abline(v=alpha)}

\name{gao}
\alias{gao}
\alias{gao.wrapper}
\title{Xin Gao's non-parametric multiple test procedure is applied to Data.}
\usage{gao(formula, data, alpha=0.05, control=NULL, silent=FALSE)
gao.wrapper(model, data, alpha, control)}
\description{Xin Gao's non-parametric multiple test procedure is applied to Data. 
The procedure controls the FWER in the strong sense. Here, only the Many-To-One comparisons are computed.}
\details{This function computes Xin Gao's nonparametric multiple test procedures in an unbalanced one way layout. 
It is based upon the following purely nonparametric effects: 
Let \eqn{F_i} denote the distribution function of sample \eqn{i, i=1,\ldots,a,} and let \eqn{G} denote the mean distribution function
of all distribution functions \eqn{(G=1/a\sum_i F_i)}. The effects \eqn{p_i=\int GdF_i} are called unweighted relative effects. If \eqn{p_i>1/2}, the random
variables from sample \eqn{i} tend (stochastically) to larger values than any randomly chosen number 
from the whole experiment. If \eqn{p_i = 1/2}, there is no tendency to smaller nor larger values. However,
this approach tests the hypothesis \eqn{H_0^F: F_1=F_j, j=2,\ldots,a} formulated in terms of the
distribution functions, simultaneously.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{confIntervals}{A matrix containing the estimates and the lower and upper confidence bound}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{Frank Konietschke}
\references{Gao, X. et al. (2008). Nonparametric multiple comparison procedures for unbalanced
one-way factorial designs. 
Journal of Statistical Planning and Inference 77, 2574-2591. \eqn{n}
The FWER is controlled by using the Hochberg adjustment
(Hochberg, Y. (1988). A sharper Bonferroni procedure for multiple tests of significance. Biometrika 75, 800-802.)}
\arguments{\item{formula}{Formula defining the statistical model, containing the response and the factors}
\item{model}{Model with formula, containing the response and the factors}
\item{data}{Dataset containing the response and the grouping factor}
\item{alpha}{The level at which the FWER shall be controlled. By default it is alpha=0.05.}
\item{silent}{If true any output on the console will be suppressed.}
\item{control}{The control group for the Many-To-One comparisons. By default it is the first group in lexicographical order.}}
\examples{x=c(rnorm(40))
f1=c(rep(1,10),rep(2,10),rep(3,10),rep(4,10))
my.data <- data.frame(x,f1)
result <- gao(x~f1,data=my.data, alpha=0.05,control=2, silent=FALSE)
result <- gao(x~f1,data=my.data, alpha=0.05,control=2, silent=TRUE)
result <- gao(x~f1,data=my.data, alpha=0.05)}

\name{aorc}
\alias{aorc}
\title{Step-up-down test based on the asymptotically optimal rejection curve.}
\usage{aorc(pValues, alpha, startIDX_SUD=length(pValues), betaAdjustment,
    silent=FALSE)}
\description{Performs a step-up-down test on pValues. The critical values are based 
on the asymptotically optimal rejection curve. To have finite FDR control,
an automatic adjustment of the critical values is done (see
details for more).}
\details{The graph of the function f(t) = t / (t * (1 - alpha) + alpha) is called the asymptotically
optimal rejection curve. Denote by finv(t) the inverse of f(t). Using the 
critical values finv(i/n) for i = 1, ..., n yields asymptotic FDR control.
To ensure finite control it is possible to adjust f(t) by a factor. The
function calculateBetaAdjustment() calculates a beta such that (1 + beta / n) * f(t)
can be used to control the FDR for a given finite sample size. If the 
parameter betaAdjustment is not provided, calculateBetaAdjustment() will be called automatically.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error (FDR)
controlled by the function and the level \code{alpha}.}}
\references{Finner, H., Dickhaus, T. & Roters, M. (2009).
On the false discovery rate and an asymptotically optimal rejection curve.
The Annals of Statistics 37, 596-618.}
\author{MarselScheer}
\arguments{\item{pValues}{pValues to be used. They are assumed to be stochastically independent.}
\item{alpha}{The level at which the FDR shall be controlled.}
\item{startIDX_SUD}{The index at which critical value the 
step-up-down test starts. Default is length(pValues) and thus the function
aorc() by default behaves like an step-up test.}
\item{betaAdjustment}{A numeric value to adjust the asymptotically optimal
rejection curve for the finite sample case. If betaAdjustment is not
set an algorithm will calculate it, but this can be time-consuming.}
\item{silent}{If true any output on the console will be suppressed.}}
\examples{  
\dontrun{ # Takes more than 6 seconds therefor CRAN should not check it:
r <- c(runif(10), runif(10, 0, 0.01))
result <- aorc(r, 0.05)
result <- aorc(r, 0.05, startIDX_SUD = 1)  ## step-down
result <- aorc(r, 0.05, startIDX_SUD = length(r))  ## step-up}
}

\name{pval2qval}
\alias{pval2qval}
\title{Strimmer et al.'s fdrtool-based q-values}
\usage{pval2qval(pValues, cutoff)}
\description{The function \code{pval2qval} takes a vector of p-values and
estimates for each case the tail area-based FDR, which can be regarded as a p-value corrected for multiplicity. 
This is done by calling the \code{fdrtool} function.
If a cutoff is supplied, a vector of rejected hypotheses will be returned as well.}
\value{A list containing:
\item{qValues}{A numeric vector with one q-value for each hypothesis.}
\item{rejected}{A logical vector indicating rejection/retention for each hypothesis when \code{cutoff} is supplied.}}
\author{JonathanRosenblatt}
\references{Strimmer, K. (2008). fdrtool: a versatile R package for estimating local and tail
area-based false discovery rates. Bioinformatics 24: 1461-1462. \cr
Storey, J. D. (2003)  The Positive False Discovery Rate: A Bayesian Interpretation and the q-Value.
The Annals of Statistics 31(6): 2013-2035}
\arguments{\item{pValues}{Numeric vector of p-values to be used.}
\item{cutoff}{The positive FDR cutoff for rejection. Hypotheses with \code{qValues} smaller then \code{cutoff} will be rejected.}}
\examples{pvals<- runif(1000)^2
pval2qval(pvals)
pval2qval(pValues=pvals, cutoff=0.1)}

\name{linearStepUp}
\alias{linearStepUp}
\title{Linear Step Up Service Function...}
\usage{linearStepUp(sorted, q, m, adjust=FALSE, m0=m, pi0, constant=1)}
\description{Linear Step Up Service Function}
\details{A Mutoss service function called by other procedures.}
\value{A list containing the following objects: 
\item{Cutoff}{The largest p-value of rejected hypotheses.}
\item{Pvals}{A data frame containing the original p-values, critical values, adjusted p-values and rejections.}}
\author{JonathanRosenblatt}
\arguments{\item{sorted}{Numeric vector of sorted pvalues}
\item{q}{Error rate to control for.}
\item{m}{Number of hypothesis tested.}
\item{adjust}{Logical value for p-value adjustmet (unusable).}
\item{m0}{Known or estimated number of true null hypotheses.}
\item{pi0}{Known or estimated proportion of true null hypothesis. Is redundant when \code{m0} is specified.}
\item{constant}{A Scaling constant for the denominator of the critical values.}}

\name{ftest.marginal}
\alias{ftest.marginal}
\title{Marginal F test}
\usage{ftest.marginal(data, model, robust)}
\description{The robust version uses the Kruskal-Wallis test, otherwise a F-test will be performed.}
\details{A classlabel needs to be provided to distinguish k sample groups.}
\value{\item{pValues}{A numeric vector containing the unadjusted pValues}}
\author{MuToss-Coding Team}
\references{Kruskal, W.H. und Wallis, W.A. (1952). Use of ranks in one-criterion variance analysis. JASA, 47:583-621}
\arguments{
  \item{data}{the data set}
  \item{model}{the result of a call to \code{ftest.model(classlabel)}}
  \item{robust}{a logical variable indicating whether a F-test or a Kruskal-Wallis test should be used.}
}

\name{indepBR}
\alias{indepBR}
\title{Blanchard-Roquain (2009) 1-stage adaptive step-up}
\usage{indepBR(pValues, alpha, lambda=1, silent=FALSE)}
\description{Blanchard-Roquain (2009) 1-stage adaptive step-up}
\details{This is a step-up procedure with critical values

C_i = alpha * min( i * ( 1 - lambda * alpha) / (m - i + 1) , lambda )

where alpha is the level at which FDR should be controlled and lambda an 
arbitrary parameter belonging to (0, 1/alpha) with default value 1.
This procedure controls FDR at the desired level when the p-values are independent.}
\value{A list containing:

\item{rejected}{A logical vector indicating which hypotheses are rejected}

\item{criticalValues}{A numeric vector containing critical values used in the step-up-down test}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function and the level \code{alpha}.}}
\author{GillesBlanchard}
\references{Blanchard, G. and Roquain, E. (2009) 
Adaptive False Discovery Rate Control under Independence and Dependence
Journal of Machine Learning Research 10:2837-2871.}
\arguments{\item{pValues}{the used p-values (assumed to be independent)}
\item{alpha}{the level at which the FDR should be controlled.}
\item{lambda}{parameter of the procedure, should belong to
(0, 1/alpha) (lambda=1 default)}
\item{silent}{if true any output on the console will be suppressed.}}

\name{sidak}
\alias{sidak}
\title{Sidak correction}
\usage{sidak(pValues, alpha, silent=FALSE)}
\description{The classical Sidak correction returns adjusted p-values, ensuring strong FWER control under
the assumption of independence of the input p-values. It only uses the fact that the probability of no incorrect
rejection is the product over true nulls of those marginal probabilities (using the assumed independence of p-values).}
\details{The procedure is more generally valid for positive orthant dependent test statistics.

It is recommended to use the step-down version of the Sidak correction instead (see SidakSD), 
which is valid under the exact same assumptions but is more powerful.}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the adjusted pValues}

\item{rejected}{(if alpha is given) A logical vector indicating which hypotheses are rejected}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\references{Sidak, Z. (1967). Rectangular confidence regions for the means of multivariate normal distributions.
Journal of the American Statistical Association, 62:626-633.}
\author{MuToss-Coding Team}
\arguments{
  \item{pValues}{pValues to be used.}
  \item{alpha}{The level at which the FWER shall be controlled (optional).}
  \item{silent}{logical scalar. If \code{TRUE} no output is generated.}
}
\examples{alpha <- 0.05
p <-c(runif(10, min=0, max=0.01), runif(10, min=0.9, max=1))
result <- sidak(p)
result <- sidak(p, alpha)
result <- sidak(p, alpha, silent=TRUE)}

\name{simulation}
\alias{simulation}
\title{Simulation studies}
\usage{simulation(replications, DataGen, listOfProcedures, discardProcInput=FALSE)}
\description{This function generates data according to a specified function and parameters
and then applies specified procedures to the generated data. The generated data is stored
in $data and the results are stored in $results. In order to recognize which
results and generated data belong together every result contains an element
$data.set.number. The data generating function must return a list which contains a list
$procInput. Every element in $procInput will be used as an input parameter
for the procedures provided by listOfProcedures.}
\value{A list with 2 elements. $data contains the objects generated by DataGen. $results
contains the objects generated by the procedures augmented by the data set number and
the parameter constellation.}
\author{MarselScheer}
\arguments{\item{replications}{The number of replications. This means how many simulation runs will be performed.}
\item{DataGen}{A list that contains the function and parameters for generating data 
which will be analyzed be the procedures in listOfProcedures. It must contain the elements
$funName (character) $fun (function)}
\item{listOfProcedures}{A list of lists which contains the procedures with their  parameters
for the simulation.}
\item{discardProcInput}{A list of lists which contains the procedures with their  parameters}}
\examples{# this function generates pValues 
myGen <- function(n, n0) {
  list(procInput=list(pValues = c(runif(n-n0, 0, 0.01), runif(n0))), 
  groundTruth = c(rep(FALSE, times=n-n0), rep(TRUE, times=n0)))
}

sim <- simulation(replications = 10, list(funName="myGen", fun=myGen, n=200, n0=c(50,100)), 
list(list(funName="BH", 
  fun=function(pValues, alpha) BH(pValues, alpha, silent=TRUE), alpha=c(0.25, 0.5)),
list(funName="holm", fun=holm, alpha=c(0.25, 0.5),silent=TRUE)))

# the following happend:
# Call myGen(200,50) and append result to sim$data
# Apply bonferroni and holm each with alpha 0.25 and 0.5 to this data set
# Append the results to sim$restults
# Repeat this 10 times.
# Call myGen(200, 100) and append restult to sim$data
# Apply bonferroni and holm each with alpha 0.25 and 0.5 to this data set
# Append the results to sim$restults
# Repeat this 10 times.

length(sim$data)
length(sim$results)

# we now reproduce the 6th item in results
print(sim$results[[6]]$data.set.number)
print(sim$results[[6]]$parameters)

all(BH(sim$data[[2]]$procInput$pValues, 0.5, silent=TRUE)$adjPValues == sim$results[[6]]$adjPValues)

#
# Just calculating some statistics and making some plots
NumberOfType1Error <- function(data, result) sum(data$groundTruth * result$rejected)
result.all <- gatherStatistics(sim, list(NumOfType1Err = NumberOfType1Error))
result <- gatherStatistics(sim, list(NumOfType1Err = NumberOfType1Error), 
  list(median=median, mean=mean, sd=sd))
print(result)
require(lattice)
histogram(~NumOfType1Err | method*alpha, data = result.all$statisticDF)
barchart(NumOfType1Err.median + NumOfType1Err.mean ~ method | alpha, data = result$statisticDF)}

\name{bonferroni}
\alias{bonferroni}
\title{Bonferroni correction...}
\usage{bonferroni(pValues, alpha, silent=FALSE)}
\description{Bonferroni correction}
\details{The classical Bonferroni correction outputs adjusted p-values, ensuring strong FWER control under arbitrary
dependence of the input p-values. It simply multiplies each input p-value by the total number of hypotheses
(and ceils at value 1).

It is recommended to use Holm's step-down instead, which is valid under the exact same assumptions and more powerful.}
\arguments{
  \item{pValues}{
	A numeric vector containing the unadjusted pValues. No assumption is made on the dependence structure.
  }
  \item{alpha}{
	The overall type I error at which the FWER shall be controlled (optional).
  }
  \item{silent}{
  	logical scalar. If \code{TRUE} no output is generated.  
  }
}
\value{A list containing:

\item{adjPValues}{A numeric vector containing the new adjusted pValues}

\item{rejected}{(if alpha is given) A logical vector indicating which hypotheses are rejected}

\item{errorControl}{A Mutoss S4 class of type \code{errorControl}, containing the type of error controlled by the function.}}
\references{Bonferroni, C. E. (1935) Il calcolo delle assicurazioni su gruppi di teste. 
'In Studi in Onore del Professore Salvatore Ortu Carboni. Rome: Italy, pp. 13-60.

Bonferroni, C. E. (1936) Teoria statistica delle classi e calcolo delle probabilita.
Pubblicazioni del R Istituto Superiore di Scienze Economiche e Commerciali di Firenze 8, 3-62, 1936.}
\author{MuToss-Coding Team}
